/**************************************************************************

			PROCEDIMENTS CONSULTES

**************************************************************************/

/*******************************************************************************************************

LLISTAT VOTACIONS PER PAS

NOM PROCEDIMENT: FUN_LLIST_VOTACIONS_PAIS

FUNCIONALITAT:

Funci que retorna llistat amb totes les votacions que shi han produt en un pas concret, retornant aquesta informaci:
-	Ttol de la votaci
-	Data dinici de la votaci
-	Data fi de la votaci
-	Nombre total de ciutadans dins dels censos associats a la votaci
-	Estat en que es troba la votaci, p.e : en edici, pendent dobertura, oberta, finalitzada.
-	Si la votaci ha estat finalitzada: percentatge de participaci total, opci guanyadora i percentatge de vots de lopci guanyadora. En cas que no estigui finalitzada hauria de sortir un - en aquests camps.

PARAMETRES
ENTRADA
	p_codpais	Identificaci del pas
SORTIDA
	Obligatoris:                               RSP

RETORNA:
   SI TOT CORRECTE
	Retorna un llistat amb totes les votacions que shi han produt en un pas concret.
   SI ES PRODUEIX UN ERROR
	ERROR: El codi de pas no pot ser nul.
	ERROR: El codi de pas no existeix a la Base de Dades.
	ERROR: Tipus derror. En qualsevol altre cas.
  
EXECUCI   SELECT * FROM TABLE(FUN_LLIST_VOTACIONS_PAIS (ES));

*******************************************************************************************************/

--TIPUS QUE S'INSERTA CADA REGISTRE DEL LLISTAT

CREATE OR REPLACE TYPE VOTACIONS_PAIS_OBJECT as OBJECT (
   TITOL		VARCHAR2(30),
   DATA_INICI		DATE,
   DATA_FI		DATE,
   TOTAL_CIUTADANS	NUMBER,
   ESTAT		VARCHAR2(200),
   P_PARTICIPACIO_TOTAL	INTEGER,
   OPCIO_GUANYADORA	VARCHAR2(30),
   P_OPCIO_GUANYADORA	INTEGER
);
/

--TIPUS EMMAGATZEMA ELS REGISTRES QUE RETORNA EL LLISTAT
 
CREATE OR REPLACE TYPE VOTACIONS_PAIS_TAB is table of VOTACIONS_PAIS_OBJECT;             
/

--FUNCI

CREATE OR REPLACE FUNCTION USUARITFC11.FUN_LLIST_VOTACIONS_PAIS(
   P_CODPAIS	IN VARCHAR2)
RETURN VOTACIONS_PAIS_TAB
IS

L_VOTACIONS_PAIS_TAB VOTACIONS_PAIS_TAB:= VOTACIONS_PAIS_TAB();

E_CODPAIS_NOEXISTEIX	EXCEPTION;
E_CAMPS_NULL  		EXCEPTION;
N                 	NUMBER := 0;

existeix		VARCHAR2(1);
p_entrada		VARCHAR2(1000);
error_num		NUMBER;
error_missatge		VARCHAR2(100);
RSP               	VARCHAR2(2000);

   CURSOR CUR_VOTACIONS_PAIS(P_CODPAIS IN VARCHAR2) IS
   SELECT V.TITOL as "TITOL", V.DATAINICI as "DATA_INICI",
	 V.DATAFI as "DATA_FI",
   (SELECT COUNT(*) as CODI 
    FROM OPCIO OPCIO, CIUTADAOPCIO CIUTADAOPCIO,
       CIUTADA CIUTADA 
    WHERE CIUTADAOPCIO.IDCIUTADA = CIUTADA.CODI
      AND OPCIO.IDOPCIO = CIUTADAOPCIO.IDOPCIO
      AND OPCIO.IDVOTACIO = V.IDVOTACIO) as "TOTAL_CIUTADANS",      
   CASE WHEN (SYSDATE BETWEEN V.DATAINICI AND V.DATAFI) THEN 'Oberta' 
        WHEN SYSDATE < V.DATAINICI THEN 'Pendent obertura'
        WHEN SYSDATE > V.DATAFI THEN 'Finalitzada'
        ELSE 'En edici'
   END "ESTAT",
   DECODE (V.PERCENTATGEVOTACIO, NULL, '-', V.PERCENTATGEVOTACIO) as "P_PARTICIPACIO_TOTAL",
   DECODE ( (SELECT opcio from opcio where idvotacio = V.IDVOTACIO and percentatge = (SELECT max(percentatge) FROM OPCIO WHERE opcio.idvotacio = V.IDVOTACIO)) 
   , NULL, '-', (SELECT opcio from opcio where idvotacio = V.IDVOTACIO and percentatge = (SELECT max(percentatge) FROM OPCIO WHERE opcio.idvotacio = V.IDVOTACIO)) ) as OPCIO_GUANYADORA,
   DECODE ( (SELECT max(percentatge) FROM OPCIO WHERE opcio.idvotacio = V.IDVOTACIO), NULL, '-', (SELECT max(percentatge) FROM OPCIO WHERE opcio.idvotacio = V.IDVOTACIO) ) as P_OPCIO_GUANYADORA
   FROM CENSPAIS CP, VOTACIO V 
   WHERE CP.IDVOTACIO = V.IDVOTACIO AND CP.IDPAIS = P_CODPAIS
   ORDER BY V.DATAINICI ASC;

   PRAGMA AUTONOMOUS_TRANSACTION;

BEGIN

   p_entrada := 'P_CODPAIS: ' ||p_codpais;

   --Es verifica que tots els camps que es pasan per parmetre contingin valor

   IF (TRIM(P_CODPAIS) IS NULL) THEN 
      RAISE E_CAMPS_NULL;   
   END IF;    

   --Es verifica que els camps que es pasen per parmetres existeixin

   BEGIN
   
     SELECT 'Y' INTO existeix 
     FROM PAIS
     WHERE codpais = p_codpais AND ROWNUM = 1;

   EXCEPTION
     WHEN NO_DATA_FOUND THEN
       RAISE E_CODPAIS_NOEXISTEIX;
     END;

  -- Es guarda a la taula

   FOR R IN CUR_VOTACIONS_PAIS(P_CODPAIS) LOOP
     L_VOTACIONS_PAIS_TAB.EXTEND;
     N := N + 1;
     L_VOTACIONS_PAIS_TAB(N) := VOTACIONS_PAIS_OBJECT(R.TITOL, R.DATA_INICI, R.DATA_FI, R.TOTAL_CIUTADANS, R.ESTAT, 
						R.P_PARTICIPACIO_TOTAL, R.OPCIO_GUANYADORA, R.P_OPCIO_GUANYADORA);            
   END LOOP;
  
   RSP := 'OK';  

   INSERT INTO LOG (NOMPROCEDIMENT, DATA, PARAMETREENTRADA, PARAMETRESORTIDA)
   VALUES ('FUN_LLIST_VOTACIONS_PAIS', SYSDATE, p_entrada, 'RSP: '||RSP);
  
   COMMIT;
  
   RETURN L_VOTACIONS_PAIS_TAB;
   
EXCEPTION

   WHEN E_CAMPS_NULL THEN 
	RSP:= 'ERROR: : El codi de pas no pot ser nul'; 

   INSERT INTO LOG (NOMPROCEDIMENT, DATA, PARAMETREENTRADA, PARAMETRESORTIDA)
   VALUES ('FUN_LLIST_VOTACIONS_PAIS', SYSDATE, p_entrada, 'RSP: '||RSP);

   COMMIT;

   WHEN E_CODPAIS_NOEXISTEIX THEN 
	RSP:= 'ERROR: : El codi de pas no existeix a la Base de Dades'; 

   INSERT INTO LOG (NOMPROCEDIMENT, DATA, PARAMETREENTRADA, PARAMETRESORTIDA)
   VALUES ('FUN_LLIST_VOTACIONS_PAIS', SYSDATE, p_entrada, 'RSP: '||RSP);

   COMMIT;

   WHEN OTHERS THEN
   error_num := SQLCODE;
   error_missatge := SUBSTR(SQLERRM, 1, 200);

   RSP:= 'ERROR '||TO_CHAR(error_num)||': '||error_missatge;

   INSERT INTO LOG (NOMPROCEDIMENT, DATA, PARAMETREENTRADA, PARAMETRESORTIDA)
   VALUES ('FUN_LLIST_VOTACIONS_PAIS', SYSDATE, p_entrada, 'RSP: '||RSP);

   COMMIT;


END;
/


/*******************************************************************************************************

Llistat de les 10 votacions amb ms diferencia percentual


NOM PROCEDIMENT: FUN_LLIST_MES_DIFERENCIA_P

FUNCIONALITAT:

Funci que retorna llistat amb les 10 votacions on hi ha hagut ms diferencia percentual de vots entre lopci ms votada i la menys votada:
-	Ttol de la votaci
-	Data dinici i de fi de la votaci
-	Opci ms votada i el seu percentatge respecte el total.
-	Opci menys votada i el seu percentatge respecte el total
-	Diferencia percentual entre les dues opcions anteriors.

PARAMETRES
SORTIDA
Obligatoris:                               RSP

RETORNA:
   SI TOT CORRECTE
	Retorna un llistat amb les 10 votacions on hi ha hagut ms diferencia percentual de vots entre lopci ms votada i la menys votada ordenat descendentment per la diferncia percentual de les dues votacions.
   SI ES PRODUEIX UN ERROR
	ERROR: Tipus derror. En qualsevol altre cas.

EXECUCI   SELECT * FROM TABLE(FUN_LLIST_MES_DIFERENCIA_P);

*******************************************************************************************************/

--TIPUS QUE S'INSERTA CADA REGISTRE DEL LLISTAT

CREATE OR REPLACE TYPE MES_DIFERENCIA_P_OBJECT as OBJECT (
   TITOL		VARCHAR2(30),
   DATA_INICI		DATE,
   DATA_FI		DATE,
   OPCIO_MES_VOTS	VARCHAR2(30),
   PERCENT_MES		INTEGER,
   OPCIO_MENYS_VOTS	VARCHAR2(30),
   PERCENT_MENYS	INTEGER,
   DIFERENCIA_P		INTEGER
);
/

--TIPUS EMMAGATZEMA ELS REGISTRES QUE RETORNA EL LLISTAT
 
CREATE OR REPLACE TYPE MES_DIFERENCIA_P_TAB is table of MES_DIFERENCIA_P_OBJECT;             
/

--FUNCI

CREATE OR REPLACE FUNCTION FUN_LLIST_MES_DIFERENCIA_P
RETURN MES_DIFERENCIA_P_TAB
IS

L_MES_DIFERENCIA_P_TAB MES_DIFERENCIA_P_TAB:= MES_DIFERENCIA_P_TAB();

N                 	NUMBER := 0;
error_num		NUMBER;
error_missatge		VARCHAR2(100);
RSP               	VARCHAR2(2000);

   CURSOR CUR_MES_DIFERENCIA_P IS
   SELECT TITOL, DATA_INICI, DATA_FI,
        DECODE (OPCIO_MES_VOTS, NULL, '-', OPCIO_MES_VOTS) AS OPCIO_MES_VOTS,
        DECODE (PERCENT_MES, NULL, '0', PERCENT_MES) AS PERCENT_MES,
        DECODE (OPCIO_MENYS_VOTS, NULL, '-', OPCIO_MENYS_VOTS) AS OPCIO_MENYS_VOTS,
        DECODE (PERCENT_MENYS, NULL, '0', PERCENT_MENYS) AS PERCENT_MENYS,
        DECODE (DIFERENCIA_P, NULL, '0', DIFERENCIA_P) AS DIFERENCIA_P
   FROM(
     SELECT Distinct(V.TITOL) as TITOL, 
     V.DATAINICI as DATA_INICI,
     V.DATAFI as DATA_FI,
     ( SELECT opcio from opcio where idvotacio = o.idvotacio and percentatge = (SELECT max(percentatge) FROM OPCIO WHERE opcio.idvotacio = o.idvotacio) ) as OPCIO_MES_VOTS,
     (SELECT max(percentatge) FROM OPCIO WHERE opcio.idvotacio = o.idvotacio) as PERCENT_MES,
     ( SELECT opcio from opcio where idvotacio = o.idvotacio and percentatge = (SELECT min(percentatge) FROM OPCIO WHERE opcio.idvotacio = o.idvotacio) ) as OPCIO_MENYS_VOTS,
     (SELECT min(percentatge) FROM OPCIO WHERE opcio.idvotacio = o.idvotacio) as PERCENT_MENYS,
     ( (SELECT max(percentatge) FROM OPCIO WHERE opcio.idvotacio = o.idvotacio) - (SELECT min(percentatge) FROM OPCIO WHERE opcio.idvotacio = o.idvotacio) ) as DIFERENCIA_P
     FROM OPCIO O, VOTACIO V 
     WHERE O.IDVOTACIO = V.IDVOTACIO     
   )
   WHERE rownum <=10
   ORDER BY DIFERENCIA_P DESC;

   PRAGMA AUTONOMOUS_TRANSACTION;

BEGIN

   FOR R IN CUR_MES_DIFERENCIA_P LOOP
     L_MES_DIFERENCIA_P_TAB.EXTEND;
     N := N + 1;
     L_MES_DIFERENCIA_P_TAB(N) := MES_DIFERENCIA_P_OBJECT(R.TITOL, R.DATA_INICI, R.DATA_FI, R.OPCIO_MES_VOTS, R.PERCENT_MES, 
						R.OPCIO_MENYS_VOTS, R.PERCENT_MENYS, R.DIFERENCIA_P);            
   END LOOP;
  
   RSP := 'OK';  

   INSERT INTO LOG (NOMPROCEDIMENT, DATA, PARAMETREENTRADA, PARAMETRESORTIDA)
   VALUES ('FUN_MES_DIFERENCIA_P_PAIS', SYSDATE, 'CAP', 'RSP: '||RSP);
  
   COMMIT;
  
   RETURN L_MES_DIFERENCIA_P_TAB;
   
EXCEPTION

   WHEN OTHERS THEN
   error_num := SQLCODE;
   error_missatge := SUBSTR(SQLERRM, 1, 200);

   RSP:= 'ERROR '||TO_CHAR(error_num)||': '||error_missatge;

   INSERT INTO LOG (NOMPROCEDIMENT, DATA, PARAMETREENTRADA, PARAMETRESORTIDA)
   VALUES ('FUN_MES_DIFERENCIA_P', SYSDATE, 'CAP', 'RSP: '||RSP);

   COMMIT;


END;
/


/*******************************************************************************************************

Llistat de les 10 votacions amb menys diferencia percentual


NOM PROCEDIMENT: FUN_LLIST_MENYS_DIFERENCIA_P

FUNCIONALITAT:

Funci que retorna llistat amb les 10 votacions on hi ha hagut menys diferencia percentual de vots entre lopci ms votada i la menys votada:
-	Ttol de la votaci
-	Data dinici i de fi de la votaci
-	Opci ms votada i el seu percentatge respecte el total.
-	Opci menys votada i el seu percentatge respecte el total
-	Diferencia percentual entre les dues opcions anteriors.

PARAMETRES:
SORTIDA
Obligatoris:                               RSP

RETORNA:
   SI TOT CORRECTE
	Retorna un llistat amb les 10 votacions on hi ha hagut menys diferencia percentual de vots entre lopci ms votada i la menys votada ordenat descendentment per la diferncia percentual de les dues votacions.
   SI ES PRODUEIX UN ERROR
	ERROR: Tipus derror. En qualsevol altre cas.

EXECUCI   SELECT * FROM TABLE(FUN_LLIST_MENYS_DIFERENCIA_P);

*******************************************************************************************************/

--TIPUS QUE S'INSERTA CADA REGISTRE DEL LLISTAT

CREATE OR REPLACE TYPE MENYS_DIFERENCIA_P_OBJECT as OBJECT (
   TITOL		VARCHAR2(30),
   DATA_INICI		DATE,
   DATA_FI		DATE,
   OPCIO_MES_VOTS	VARCHAR2(30),
   PERCENT_MES		INTEGER,
   OPCIO_MENYS_VOTS	VARCHAR2(30),
   PERCENT_MENYS	INTEGER,
   DIFERENCIA_P		INTEGER
);
/

--TIPUS EMMAGATZEMA ELS REGISTRES QUE RETORNA EL LLISTAT
 
CREATE OR REPLACE TYPE MENYS_DIFERENCIA_P_TAB is table of MENYS_DIFERENCIA_P_OBJECT;             
/

--FUNCI

CREATE OR REPLACE FUNCTION FUN_LLIST_MENYS_DIFERENCIA_P
RETURN MENYS_DIFERENCIA_P_TAB
IS

L_MENYS_DIFERENCIA_P_TAB MENYS_DIFERENCIA_P_TAB:= MENYS_DIFERENCIA_P_TAB();

N                 	NUMBER := 0;
error_num		        NUMBER;
error_missatge		  VARCHAR2(100);
RSP               	VARCHAR2(2000);

   CURSOR CUR_MENYS_DIFERENCIA_P IS
   SELECT TITOL, DATA_INICI, DATA_FI,
        DECODE (OPCIO_MES_VOTS, NULL, '-', OPCIO_MES_VOTS) AS OPCIO_MES_VOTS,
        DECODE (PERCENT_MES, NULL, '0', PERCENT_MES) AS PERCENT_MES,
        DECODE (OPCIO_MENYS_VOTS, NULL, '-', OPCIO_MENYS_VOTS) AS OPCIO_MENYS_VOTS,
        DECODE (PERCENT_MENYS, NULL, '0', PERCENT_MENYS) AS PERCENT_MENYS,
        DECODE (DIFERENCIA_P, NULL, '0', DIFERENCIA_P) AS DIFERENCIA_P
   FROM(
     SELECT Distinct(V.TITOL) as TITOL, 
     V.DATAINICI as DATA_INICI,
     V.DATAFI as DATA_FI,
     ( SELECT opcio from opcio where idvotacio = o.idvotacio and percentatge = (SELECT max(percentatge) FROM OPCIO WHERE opcio.idvotacio = o.idvotacio) ) as OPCIO_MES_VOTS,
     (SELECT max(percentatge) FROM OPCIO WHERE opcio.idvotacio = o.idvotacio) as PERCENT_MES,
     ( SELECT opcio from opcio where idvotacio = o.idvotacio and percentatge = (SELECT min(percentatge) FROM OPCIO WHERE opcio.idvotacio = o.idvotacio) ) as OPCIO_MENYS_VOTS,
     (SELECT min(percentatge) FROM OPCIO WHERE opcio.idvotacio = o.idvotacio) as PERCENT_MENYS,
     ( (SELECT max(percentatge) FROM OPCIO WHERE opcio.idvotacio = o.idvotacio) - (SELECT min(percentatge) FROM OPCIO WHERE opcio.idvotacio = o.idvotacio) ) as DIFERENCIA_P
     FROM OPCIO O, VOTACIO V 
     WHERE O.IDVOTACIO = V.IDVOTACIO     
   )
   WHERE rownum <=10
   ORDER BY DIFERENCIA_P ASC;

   PRAGMA AUTONOMOUS_TRANSACTION;

BEGIN

   FOR R IN CUR_MENYS_DIFERENCIA_P LOOP
     L_MENYS_DIFERENCIA_P_TAB.EXTEND;
     N := N + 1;
     L_MENYS_DIFERENCIA_P_TAB(N) := MENYS_DIFERENCIA_P_OBJECT(R.TITOL, R.DATA_INICI, R.DATA_FI, R.OPCIO_MES_VOTS, R.PERCENT_MES, 
						R.OPCIO_MENYS_VOTS, R.PERCENT_MENYS, R.DIFERENCIA_P);            
   END LOOP;
  
   RSP := 'OK';  

   INSERT INTO LOG (NOMPROCEDIMENT, DATA, PARAMETREENTRADA, PARAMETRESORTIDA)
   VALUES ('FUN_MENYS_DIFERENCIA_P_PAIS', SYSDATE, 'CAP', 'RSP: '||RSP);
  
   COMMIT;
  
   RETURN L_MENYS_DIFERENCIA_P_TAB;
   
EXCEPTION

   WHEN OTHERS THEN
   error_num := SQLCODE;
   error_missatge := SUBSTR(SQLERRM, 1, 200);

   RSP:= 'ERROR '||TO_CHAR(error_num)||': '||error_missatge;

   INSERT INTO LOG (NOMPROCEDIMENT, DATA, PARAMETREENTRADA, PARAMETRESORTIDA)
   VALUES ('FUN_MENYS_DIFERENCIA_P', SYSDATE, 'CAP', 'RSP: '||RSP);

   COMMIT;


END;
/


/*******************************************************************************************************

Llistat de tots els censos que pertany un ciutad en concret


NOM PROCEDIMENT: FUN_CENSOS_CIUTADA

FUNCIONALITAT:

Funci que retorna un llistat amb tots els censos al que pertany un ciutad:
-	Cens Localitat
-	Cens Regi
-	Cens Pas

PARAMETRES:
ENTRADA
	p_codi	Identificaci del ciutad

SORTIDA
Obligatoris:                               RSP

RETORNA:
   SI TOT CORRECTE
	Retorna un llistat amb tots els censos dun ciutad en concret.
   SI ES PRODUEIX UN ERROR
	ERROR: El codi del ciutad no pot ser nul.
	ERROR: El ciutad no existeix a la Base de Dades.
	ERROR: Tipus derror. En qualsevol altre cas.

EXECUCI   SELECT * FROM TABLE(FUN_LLIST_CENSOS_CIUTADA('A1'));

*******************************************************************************************************/

--TIPUS QUE S'INSERTA CADA REGISTRE DEL LLISTAT
CREATE OR REPLACE TYPE CENSOS_CIUTADA_OBJECT as OBJECT (
   CENS_LOCALITAT	VARCHAR2(30),
   CENS_REGIO		VARCHAR(20),
   CENS_PAIS		VARCHAR (20)

);
/
   
--TIPUS EMMAGATZEMA ELS REGISTRES QUE RETORNA EL LLISTAT

CREATE OR REPLACE TYPE CENSOS_CIUTADA_TAB is table of CENSOS_CIUTADA_OBJECT;             
/

--FUNCI

CREATE OR REPLACE FUNCTION FUN_LLIST_CENSOS_CIUTADA
(p_codi    IN VARCHAR2)
RETURN CENSOS_CIUTADA_TAB
IS

L_CENSOS_CIUTADA_TAB CENSOS_CIUTADA_TAB := CENSOS_CIUTADA_TAB();
E_CAMPS_NULL      	EXCEPTION;  
E_CODI_NOEXISTEIX	  EXCEPTION;
existeix		        VARCHAR2(1);
p_entrada		        VARCHAR2(1000);
N                   NUMBER := 0;
RSP                 VARCHAR2(2000);
error_num           NUMBER;
error_missatge      VARCHAR2(100);


   CURSOR CUR_CENSOS_CIUTADA(p_codi VARCHAR2) IS
    SELECT LOCALITAT AS CENS_LOCALITAT, REGIO AS CENS_REGIO, PAIS AS CENS_PAIS
    FROM(
    SELECT DISTINCT CIUTADA.CODI as CODI,
       LOCALITAT.LOCALITAT as LOCALITAT,
       REGIO.REGIO as REGIO,
       PAIS.PAIS as PAIS 
     from	 PAIS PAIS,
       PAISREGIO PAISREGIO,
       REGIO REGIO,
       LOCALITAT LOCALITAT,
       CENSLOCAL CENSLOCAL,
       CIUTADAOPCIO CIUTADAOPCIO,
       CIUTADA CIUTADA 
     where   CIUTADA.CODI= CIUTADAOPCIO.IDCIUTADA
     and	 CENSLOCAL.IDCENSLOCAL= CIUTADAOPCIO.IDCENSLOCAL
     and	 LOCALITAT.IDLOCALITAT = CENSLOCAL.IDLOCALITAT
     and	 REGIO.IDREGIO = LOCALITAT.IDREGIO
     and	 PAISREGIO.IDREGIO = REGIO.IDREGIO
     and	 PAIS.CODPAIS = PAISREGIO.IDPAIS
     )
     WHERE CODI = p_codi;

   PRAGMA AUTONOMOUS_TRANSACTION;
    
BEGIN

   p_entrada := 'P_CODI: ' ||p_codi;

   --Es verifica que tots els camps que es pasan per parmetre contingin valor

   IF (TRIM(P_CODI) IS NULL) THEN 
      RAISE E_CAMPS_NULL;   
   END IF;

   --Es verifica que els camps que es pasen per parmetres existeixin a la Base de Dades

   BEGIN
   
   SELECT 'Y' INTO existeix 
   FROM CIUTADA
   WHERE codi = p_codi AND ROWNUM = 1;

   EXCEPTION
   WHEN NO_DATA_FOUND THEN
     RAISE E_CODI_NOEXISTEIX;
   END;

   FOR R IN CUR_CENSOS_CIUTADA(p_codi) LOOP
     L_CENSOS_CIUTADA_TAB.EXTEND;
     N := N + 1;
     L_CENSOS_CIUTADA_TAB(N) := CENSOS_CIUTADA_OBJECT(R.CENS_LOCALITAT, R.CENS_REGIO, R.CENS_PAIS);            
   END LOOP;
  
   RSP := 'OK';  

   INSERT INTO LOG (NOMPROCEDIMENT, DATA, PARAMETREENTRADA, PARAMETRESORTIDA)
   VALUES ('FUN_LLIST_CENSOS_CIUTADA', SYSDATE, p_entrada, 'RSP: '||RSP);
  
   COMMIT;
  
   RETURN L_CENSOS_CIUTADA_TAB;
   
EXCEPTION

   WHEN E_CAMPS_NULL THEN 
	 RSP:= 'ERROR: : El codi del ciutad no pot ser nul'; 

   INSERT INTO LOG (NOMPROCEDIMENT, DATA, PARAMETREENTRADA, PARAMETRESORTIDA)
   VALUES ('FUN_LLIST_CENSOS_CIUTADA', SYSDATE, p_entrada, 'RSP: '||RSP);
   
   COMMIT;

   WHEN E_CODI_NOEXISTEIX THEN 
	 RSP:= 'ERROR: : El codi del ciutad no existeix a la Base de Dades'; 

   INSERT INTO LOG (NOMPROCEDIMENT, DATA, PARAMETREENTRADA, PARAMETRESORTIDA)
   VALUES ('FUN_LLIST_CENSOS_CIUTADA', SYSDATE, p_entrada, 'RSP: '||RSP);
   
   COMMIT;

   WHEN OTHERS THEN
   error_num := SQLCODE;
   error_missatge := SUBSTR(SQLERRM, 1, 200);

   RSP:= 'ERROR '||TO_CHAR(error_num)||': '||error_missatge;

   INSERT INTO LOG (NOMPROCEDIMENT, DATA, PARAMETREENTRADA, PARAMETRESORTIDA)
   VALUES ('FUN_LLIST_CENSOS_CIUTADA', SYSDATE, p_entrada, 'RSP: '||RSP);

   COMMIT;


END;
/

/*******************************************************************************************************

Llistat de les votacions que ha participat o no un ciutad en concret


NOM PROCEDIMENT: FUN_PARTICIPACIO_CIUTADA

FUNCIONALITAT:

Funci que retorna un llistat amb totes les votacions en les que ha participat o no un ciutad:
-	Ttol votaci
-	Participaci o no

PARAMETRES:
ENTRADA
	p_codi	Identificaci del ciutad

SORTIDA
Obligatoris:                               RSP

RETORNA:
   SI TOT CORRECTE
	Retorna un llistat amb totes les votacions en les que ha participat o no un ciutad en concret.
   SI ES PRODUEIX UN ERROR
	ERROR: El codi del ciutad no pot ser nul.
	ERROR: El ciutad no existeix a la Base de Dades.
	ERROR: Tipus derror. En qualsevol altre cas.

EXECUCI   SELECT * FROM TABLE(FUN_LLIST_PARTICIPACIO_CIUTADA('ES'));

*******************************************************************************************************/

--TIPUS QUE S'INSERTA CADA REGISTRE DEL LLISTAT
CREATE OR REPLACE TYPE PARTICIPACIO_CIUTADA_OBJECT as OBJECT (
   TITOL		VARCHAR2(30),
   PARTICIPACIO		VARCHAR(5),
);
/
   
--TIPUS EMMAGATZEMA ELS REGISTRES QUE RETORNA EL LLISTAT

CREATE OR REPLACE TYPE PARTICIPACIO_CIUTADA_TAB is table of PARTICIPACIO_CIUTADA_OBJECT;             
/

--FUNCI

CREATE OR REPLACE FUNCTION FUN_LLIST_PARTICIPACIO_CIUTADA
(p_codi    IN VARCHAR2)
RETURN PARTICIPACIO_CIUTADA_TAB
IS

L_PARTICIPACIO_CIUTADA_TAB PARTICIPACIO_CIUTADA_TAB := PARTICIPACIO_CIUTADA_TAB();
E_CAMPS_NULL      	EXCEPTION;  
E_CODI_NOEXISTEIX	EXCEPTION;

existeix		VARCHAR2(1);
p_entrada		VARCHAR2(1000);
N                 NUMBER := 0;
RSP               VARCHAR2(2000);
err_num           NUMBER;
missatge          VARCHAR2(100);


   CURSOR CUR_PARTICIPACIO_CIUTADA(p_codi VARCHAR2) IS
    SELECT TITOL, 
    DECODE (CODI, NULL, 'NO', 'YES') AS PARTICIPAT
    FROM(
      SELECT V.TITOL,
        (SELECT CIUTADA.CODI 
         from	 VOTACIO VOTACIO, OPCIO OPCIO,
           CIUTADAOPCIO CIUTADAOPCIO, CIUTADA CIUTADA 
         where CIUTADA.CODI = CIUTADAOPCIO.IDCIUTADA
         and	 OPCIO.IDOPCIO = CIUTADAOPCIO.IDOPCIO
         and	 VOTACIO.IDVOTACIO = OPCIO.IDVOTACIO
         AND VOTACIO.IDVOTACIO = V.IDVOTACIO
         AND CIUTADA.CODI = p_codi
         ) AS CODI
       FROM VOTACIO V
       ORDER BY TITOL DESC
    );

   PRAGMA AUTONOMOUS_TRANSACTION;
    
BEGIN

   p_entrada := 'P_CODI: ' ||p_codi;

   --Es verifica que tots els camps que es pasan per parmetre contingin valor

   IF (TRIM(P_CODI) IS NULL) THEN 
      RAISE E_CAMPS_NULL;   
   END IF;

   --Es verifica que els camps que es pasen per parmetres existeixin a la Base de Dades

   BEGIN
   
   SELECT 'Y' INTO existeix 
   FROM CIUTADA
   WHERE codi = p_codi AND ROWNUM = 1;

   EXCEPTION
   WHEN NO_DATA_FOUND THEN
     RAISE E_CODI_NOEXISTEIX;
   END;

   BEGIN


   FOR R IN CUR_PARTICIPACIO_CIUTADA(p_codi) LOOP
     L_PARTICIPACIO_CIUTADA_TAB.EXTEND;
     N := N + 1;
     L_PARTICIPACIO_CIUTADA_TAB(N) := PARTICIPACIO_CIUTADA_OBJECT(R.CENS_LOCALITAT, R.CENS_REGIO, R.CENS_PAIS);            
   END LOOP;
  
   RSP := 'OK';  

   INSERT INTO LOG (NOMPROCEDIMENT, DATA, PARAMETREENTRADA, PARAMETRESORTIDA)
   VALUES ('FUN_LLIST_PARTICIPACIO_CIUTADA', SYSDATE, p_entrada, 'RSP: '||RSP);
  
   COMMIT;
  
   RETURN L_CENSOS_CIUTADA_TAB;
   
EXCEPTION

   WHEN E_CODI_NOEXISTEIX THEN 
	RSP:= 'ERROR: : El codi del ciutad no existeix a la Base de Dades'; 

   INSERT INTO LOG (NOMPROCEDIMENT, DATA, PARAMETREENTRADA, PARAMETRESORTIDA)
   VALUES ('FUN_LLIST_PARTICIPACIO_CIUTADA', SYSDATE, p_entrada, 'RSP: '||RSP);
   
   COMMIT;

   WHEN OTHERS THEN
   error_num := SQLCODE;
   error_missatge := SUBSTR(SQLERRM, 1, 200);

   RSP:= 'ERROR '||TO_CHAR(error_num)||': '||error_missatge;

   INSERT INTO LOG (NOMPROCEDIMENT, DATA, PARAMETREENTRADA, PARAMETRESORTIDA)
   VALUES ('FUN_LLIST_PARTICIPACIO_CIUTADA', SYSDATE, p_entrada, 'RSP: '||RSP);

   COMMIT;


END;
/








