
/**************************************************************************

			PACKAGE CENSOS

**************************************************************************/

CREATE OR REPLACE PACKAGE USUARITFC11.PKG_CENSOS AS

PROCEDURE PR_ALTA_PAIS  (
	p_codpais 	IN VARCHAR2,
        p_pais	   	IN VARCHAR2,
	RSP		OUT VARCHAR2
);

PROCEDURE PR_BAIXA_PAIS  (
	p_codpais   	IN VARCHAR2,
	RSP		OUT VARCHAR2
);

PROCEDURE PR_MODIFICACIO_PAIS (
	p_codpais	      IN VARCHAR2,
	p_pais		      IN VARCHAR2,	
	RSP		      OUT VARCHAR2
);

PROCEDURE PR_ALTA_REGIO  (
        p_regio	   	IN VARCHAR2,
	RSP		OUT VARCHAR2
);

PROCEDURE PR_BAIXA_REGIO  (
	p_idregio   	IN NUMBER,
	RSP		OUT VARCHAR2
);

PROCEDURE PR_MODIFICACIO_REGIO (
	p_idregio	      IN NUMBER,
	p_regio		      IN VARCHAR2,	
	RSP		      OUT VARCHAR2
);

PROCEDURE PR_ALTA_PAISREGIO  (
  p_idpais	IN VARCHAR2,
	p_idregio	IN NUMBER,
	RSP		OUT VARCHAR2
);

PROCEDURE PR_BAIXA_PAISREGIO  (
	p_idpaisregio   IN NUMBER,
	RSP		OUT VARCHAR2
);

PROCEDURE PR_MODIFICACIO_PAISREGIO (
	p_idpaisregio	IN NUMBER,
	p_idpais	IN VARCHAR2,
	p_idregio	IN NUMBER,	
	RSP		OUT VARCHAR2
);

PROCEDURE PR_ALTA_LOCALITAT  (
	p_idregio 	   IN NUMBER,
  p_localitat	   IN VARCHAR2,
	RSP		   OUT VARCHAR2
);

PROCEDURE PR_BAIXA_LOCALITAT  (
	p_idlocalitat  	IN NUMBER,
	RSP		OUT VARCHAR2
);

PROCEDURE PR_MODIFICACIO_LOCALITAT (
	p_idlocalitat	      IN NUMBER,
	p_idregio	      IN NUMBER,	
	p_localitat 	      IN VARCHAR2,
	RSP		      OUT VARCHAR2
);

PROCEDURE PR_ALTA_CENSPAIS  (
	p_idcenspais	   IN VARCHAR2,
	p_idvotacio 	   IN NUMBER,
  p_idpais	   IN VARCHAR2,
	RSP		   OUT VARCHAR2
);

PROCEDURE PR_BAIXA_CENSPAIS  (
	p_idcenspais   	IN VARCHAR2,
	RSP		OUT VARCHAR2
);
PROCEDURE PR_MODIFICACIO_CENSPAIS (
	p_idcenspais	      IN VARCHAR2,
	p_idvotacio	      IN NUMBER,	
	p_idpais 	      IN VARCHAR2,
	RSP		      OUT VARCHAR2
);

PROCEDURE PR_ALTA_CENSREGIO  (
        p_idregio   	IN NUMBER,
	RSP		OUT VARCHAR2
);

PROCEDURE PR_BAIXA_CENSREGIO  (
	p_idcensregio  	IN NUMBER,
	RSP		OUT VARCHAR2
);

PROCEDURE PR_MODIFICACIO_CENSREGIO (
	p_idcensregio	IN NUMBER,
	p_idregiO	IN NUMBER,	
	RSP		OUT VARCHAR2
);

PROCEDURE PR_ALTA_CENSPAISREGIO  (
  p_idcenspais   	IN VARCHAR2,
  p_idcensregio  	IN NUMBER,
  p_idpais   	IN VARCHAR2,
  p_idregio   	IN NUMBER,
	RSP		OUT VARCHAR2
);

PROCEDURE PR_BAIXA_CENSPAISREGIO  (
	p_idcenspaisregio  	IN NUMBER,
	RSP			OUT VARCHAR2
);

PROCEDURE PR_MODIFICACIO_CENSPAISREGIO (
	p_idcenspaisregio	IN NUMBER,
	p_idcenspais		IN VARCHAR2,
	p_idcensregio		IN NUMBER,
	p_idpais		IN VARCHAR2,
	p_idregio		IN NUMBER,	
	RSP			OUT VARCHAR2
);

PROCEDURE PR_ALTA_CENSLOCAL  (
	p_idcensregio 	   IN NUMBER,
        p_idlocalitat	   IN NUMBER,
	RSP		   OUT VARCHAR2
);

PROCEDURE PR_BAIXA_CENSLOCAL  (
	p_idcenslocal  	IN NUMBER,
	RSP		OUT VARCHAR2
);

PROCEDURE PR_MODIFICACIO_CENSLOCAL (
	p_idcenslocal	      IN NUMBER,
	p_idcensregio	      IN NUMBER,	
	p_idlocalitat 	      IN NUMBER,
	RSP		      OUT VARCHAR2
);


END PKG_CENSOS;
/

CREATE OR REPLACE PACKAGE BODY USUARITFC11.PKG_CENSOS AS

/********************************************************
*							*
*		MANTENIMENT PAIS			*
*							*
********************************************************/

/*******************************************************************************************************

NOM PROCEDIMENT: P_ALTA_PAIS

FUNCIONALITAT: 

Aquest procediment permet donar dalta pas a la Base de dades, si prviament supera els controls de:
	- Tots els camps obligatoris continguin valor.
	- No hi hagi un altre registre inserit amb el mateix identificador (CODPAIS).

PARAMETRES:
ENTRADA
Obligatoris:
	p_codpais	Identificador del pas
	p_pais		Nom del pas

SORTIDA
Obligatoris:
	RSP
	
PRECONDICI: No existeix el pas a crear.
POSTCONDICI: Es dona dalta el pas.

RETORNA:
   SI TOT CORRECTE
	OK si loperaci sha realitzat amb xit
   SI ES PRODUEIX UNA EXCEPCI
	ERROR: Els camps obligatoris no poden ser nul.
	ERROR: El pas ja existeix a la Base de dades.
	ERROR: Tipus derror. En qualsevol altre cas.


*******************************************************************************************************/

PROCEDURE PR_ALTA_PAIS  (
	p_codpais 	IN VARCHAR2,
  p_pais	   	IN VARCHAR2,
	RSP		      OUT VARCHAR2
)

IS

E_PAIS_EXISTEIX		EXCEPTION;
E_CAMPS_NUL		EXCEPTION;

existeix	VARCHAR2(1);
p_entrada	VARCHAR2(1000);
error_num	NUMBER;
error_missatge	VARCHAR2(100);

BEGIN

   --S'emmagatzema els parmetres d'entrada a la variable p_entrada

   p_entrada := 'P_CODPAIS: ' ||p_codpais||', P_PAIS: '||p_pais; 

   --Es verifica que tots els camps obligatoris contingin valor

   IF (TRIM(p_codpais) IS NULL OR  
	TRIM(p_pais) IS NULL)
   THEN
      RAISE E_CAMPS_NUL;
   END IF;

   --Es verifica que el registre no s'hagi insertat anteriorment amb altre identificador.

   BEGIN
   
   SELECT 'Y' INTO existeix 
   FROM PAIS
   WHERE codpais = p_codpais AND ROWNUM = 1;

   RAISE E_PAIS_EXISTEIX;

   EXCEPTION
   WHEN NO_DATA_FOUND THEN
     NULL;
   END;


   --Si es pasa totes les proves de validaci, es realitzar l'inserci del nou registre a la Base de Dades.

   INSERT INTO PAIS (CODPAIS, PAIS)
   VALUES (p_codpais, p_pais);
	
   RSP := 'OK';

   INSERT INTO LOG (NOMPROCEDIMENT, DATA, PARAMETREENTRADA, PARAMETRESORTIDA)
   VALUES ('PKG_CENSOS.PR_ALTA_PAIS', SYSDATE, p_entrada, 'RSP: '||RSP);

   COMMIT;

   --Excepcions
EXCEPTION

 --El pais existeix a la Base de Dades
   WHEN E_PAIS_EXISTEIX THEN RSP:= 'ERROR: : El pas ja existeix a la Base de Dades'; 

   INSERT INTO LOG (NOMPROCEDIMENT, DATA, PARAMETREENTRADA, PARAMETRESORTIDA)
   VALUES ('PKG_CENSOS.PR_ALTA_PAIS', SYSDATE, p_entrada, 'RSP: '||RSP);

   COMMIT;

 --Els camps obligatoris sn nuls
   WHEN E_CAMPS_NUL THEN RSP:= 'ERROR: : Els camps obligatoris no poden ser nul'; 

   INSERT INTO LOG (NOMPROCEDIMENT, DATA, PARAMETREENTRADA, PARAMETRESORTIDA)
   VALUES ('PKG_CENSOS.PR_ALTA_PAIS', SYSDATE, p_entrada, 'RSP: '||RSP);

   COMMIT;

   WHEN OTHERS THEN
   error_num := SQLCODE;
   error_missatge := SUBSTR(SQLERRM, 1, 200);

   RSP:= 'ERROR '||TO_CHAR(error_num)||': '||error_missatge;

   INSERT INTO LOG (NOMPROCEDIMENT, DATA, PARAMETREENTRADA, PARAMETRESORTIDA)
   VALUES ('PKG_CENSOS.PR_ALTA_PAIS', SYSDATE, p_entrada, 'RSP: '||RSP);

   COMMIT;

END PR_ALTA_PAIS;

/*******************************************************************************************************

NOM PROCEDIMENT: P_BAIXA_PAIS

FUNCIONALITAT:

Aquest procediment permet donar de baixa un pas a la Base de dades, si prviament supera els controls de:
	- Tots els camps obligatoris continguin valor.
	- Existeix el pas a la Base de Dades.

PARAMETRES:

ENTRADA
Obligatoris:
	p_codpais	Identificador del pas

SORTIDA
Obligatoris:
	RSP	

PRECONDICI: Existeix el pas a donar de baixa.
POSTCONDICI: Es dona de baixa el pas.

RETORNA
   SI TOT CORRECTE
	OK si loperaci sha realitzat amb xit
   SI ES PRODUEIX UNA EXCEPCI
	ERROR: El pas no existeix a la Base de Dades.
	ERROR: El camp CODPAIS no pot ser nul.
	ERROR: Tipus derror. En qualsevol altre cas.

*******************************************************************************************************/

PROCEDURE PR_BAIXA_PAIS  (
	p_codpais   	IN VARCHAR2,
	RSP		OUT VARCHAR2
)

IS

E_PAIS_NOEXISTEIX	EXCEPTION;
E_CAMPS_NUL		EXCEPTION;

existeix	VARCHAR2(1);
p_entrada	VARCHAR2(1000);
error_num	NUMBER;
error_missatge	VARCHAR2(100);

BEGIN

   --S'emmagatzema els parmetres d'entrada a la variable p_entrada

   p_entrada := 'P_CODPAIS: '||p_codpais;

   --Es verifica que tots els camps obligatoris contingin valor

   IF (TRIM(p_codpais) IS NULL)
   THEN
      RAISE E_CAMPS_NUL;
   END IF;

   --Es verifica que el registre no s'hagi insertat anteriorment amb altre identificador.

   BEGIN
   
   SELECT 'Y' INTO existeix 
   FROM PAIS 
   WHERE codpais = p_codpais AND ROWNUM = 1;

   EXCEPTION
   WHEN NO_DATA_FOUND THEN
     RAISE E_PAIS_NOEXISTEIX;
   END;

   --Si es pasa totes les proves de validaci, es realitzar la baixa del registre a la Base de Dades.

   DELETE FROM PAIS
   WHERE codpais = p_codpais;
	
   RSP := 'OK';

   INSERT INTO LOG (NOMPROCEDIMENT, DATA, PARAMETREENTRADA, PARAMETRESORTIDA)
   VALUES ('PKG_CENSOS.PR_BAIXA_PAIS', SYSDATE, p_entrada, 'RSP: '||RSP);

   COMMIT;

   --Excepcions
EXCEPTION

 --El pas no existeix a la Base de Dades
   WHEN E_PAIS_NOEXISTEIX THEN RSP:= 'ERROR: : El pas no existeix a la Base de Dades'; 

   INSERT INTO LOG (NOMPROCEDIMENT, DATA, PARAMETREENTRADA, PARAMETRESORTIDA)
   VALUES ('PKG_CENSOS.PR_BAIXA_PAIS', SYSDATE, p_entrada, 'RSP: '||RSP);

   COMMIT;

 --Els camps obligatoris sn nuls
   WHEN E_CAMPS_NUL THEN RSP:= 'ERROR: : El camp CODPAIS no pot ser nul'; 

   INSERT INTO LOG (NOMPROCEDIMENT, DATA, PARAMETREENTRADA, PARAMETRESORTIDA)
   VALUES ('PKG_CENSOS.PR_BAIXA_PAIS', SYSDATE, p_entrada, 'RSP: '||RSP);

   COMMIT;

   WHEN OTHERS THEN
   error_num := SQLCODE;
   error_missatge := SUBSTR(SQLERRM, 1, 200);

   RSP:= 'ERROR '||TO_CHAR(error_num)||': '||error_missatge;

   INSERT INTO LOG (NOMPROCEDIMENT, DATA, PARAMETREENTRADA, PARAMETRESORTIDA)
   VALUES ('PKG_CENSOS.PR_BAIXA_PAIS', SYSDATE, p_entrada, 'RSP: '||RSP);

   COMMIT;

END PR_BAIXA_PAIS;

/*******************************************************************************************************

NOM PROCEDIMENT: P_MODIFICACIO_PAIS

FUNCIONALITAT:

Aquest procediment permet modificar un pas a la Base de dades, si prviament supera els controls de:
	- El pas existeix a la Base de Dades.
	- Tots els camps obligatoris continguin valor.

PARAMETRES:
ENTRADA
Obligatoris:
	p_codpais	Identificador del pas
Opcionals:
	p_pais		Nom del pas

SORTIDA
Obligatoris:
	RSP	

PRECONDICI: Existeix el pas a modificar.
POSTCONDICI: Es modifica el pas.

RETORNA:
   SI TOT CORRECTE
	OK si loperaci sha realitzat amb xit
   SI ES PRODUEIX UNA EXCEPCI
	ERROR: Els camps obligatoris no poden ser nul.
	ERROR: El pas no existeix a la Base de Dades.
	ERROR: Tipus derror. En qualsevol altre cas.

*******************************************************************************************************/

PROCEDURE PR_MODIFICACIO_PAIS (
	p_codpais	      IN VARCHAR2,
	p_pais		      IN VARCHAR2,	
	RSP		      OUT VARCHAR2
)

IS

E_CAMPS_NUL		EXCEPTION;
E_PAIS_NOEXISTEIX	EXCEPTION;

existeix	VARCHAR2(1);
p_entrada	VARCHAR2(1000);
error_num	NUMBER;
error_missatge	VARCHAR2(100);

BEGIN

   --S'emmagatzema els parmetres d'entrada a la variable p_entrada
 
   p_entrada := 'P_CODPAIS: '||p_codpais||', P_PAIS: ' ||p_pais;

   --Es verifica que tots els camps obligatoris contingin valor

   IF (TRIM(p_codpais) IS NULL) THEN
      RAISE E_CAMPS_NUL;
   END IF;

   --Es verifica que el pas existeix a la Base de Dades
   BEGIN

   SELECT 'Y' INTO existeix 
   FROM PAIS
   WHERE codpais = p_codpais AND ROWNUM = 1;

   EXCEPTION
   WHEN NO_DATA_FOUND THEN
      RAISE E_PAIS_NOEXISTEIX;
   END;

   --Si es pasa totes les proves de validaci, es realitzar l'actualitzaci a la Base de Dades.

   UPDATE PAIS
        SET  codpais		= nvl(p_codpais, codpais),
             pais	    	= nvl(p_pais, pais)
   WHERE     codpais     	= p_codpais;

   RSP := 'OK';

   INSERT INTO LOG (NOMPROCEDIMENT, DATA, PARAMETREENTRADA, PARAMETRESORTIDA)
   VALUES ('PKG_CENSOS.PR_MODIFICACIO_PAIS', SYSDATE, p_entrada, 'RSP: '||RSP);

   COMMIT;

   --Excepcions
EXCEPTION

 --Els camps obligatoris sn nuls
   WHEN E_CAMPS_NUL THEN RSP:= 'ERROR: : Els camps obligatoris no poden ser nul'; 

   INSERT INTO LOG (NOMPROCEDIMENT, DATA, PARAMETREENTRADA, PARAMETRESORTIDA)
   VALUES ('PKG_CENSOS.PR_MODIFICACIO_PAIS', SYSDATE, p_entrada, 'RSP: '||RSP);

   COMMIT;

 --El pas no existeix a la Base de Dades
   WHEN E_PAIS_NOEXISTEIX THEN RSP:= 'ERROR: : El pas no existeix a la Base de Dades'; 

   INSERT INTO LOG (NOMPROCEDIMENT, DATA, PARAMETREENTRADA, PARAMETRESORTIDA)
   VALUES ('PKG_CENSOS.PR_MODIFICACIO_PAIS', SYSDATE, p_entrada, 'RSP: '||RSP);

   COMMIT; 

   WHEN OTHERS THEN
   error_num := SQLCODE;
   error_missatge := SUBSTR(SQLERRM, 1, 200);

   RSP:= 'ERROR '||TO_CHAR(error_num)||': '||error_missatge;

   INSERT INTO LOG (NOMPROCEDIMENT, DATA, PARAMETREENTRADA, PARAMETRESORTIDA)
   VALUES ('PKG_CENSOS.PR_MODIFICACIO_PAIS', SYSDATE, p_entrada, 'RSP: '||RSP);

   COMMIT;

END PR_MODIFICACIO_PAIS;

/********************************************************
*							*
*		MANTENIMENT REGIO			*
*							*
********************************************************/

/*******************************************************************************************************

NOM PROCEDIMENT: P_ALTA_REGIO

FUNCIONALITAT:

Aquest procediment permet donar dalta duna regi a la Base de dades, si prviament supera els controls de:
	- Tots els camps obligatoris continguin valor.
	- El registre no shagi inserit amb un altre identificador.

PARAMETRES:
ENTRADA
Obligatoris:
	p_regio	Nom de la regi

SORTIDA
Obligatoris:
	RSP
	
PRECONDICI: No existeix la regi a crear.
POSTCONDICI: Es dona dalta la regi.

RETORNA:
   SI TOT CORRECTE
	OK si loperaci sha realitzat amb xit
   SI ES PRODUEIX UNA EXCEPCI
	ERROR: La regi ja existeix a la Base de Dades.
	'ERROR: : Els camps obligatoris no poden ser nul.'
	ERROR: Tipus derror. En qualsevol altre cas.


*******************************************************************************************************/

PROCEDURE PR_ALTA_REGIO  (
        p_regio	   	IN VARCHAR2,
	RSP		OUT VARCHAR2
)

IS

E_REGIO_EXISTEIX	EXCEPTION;
E_CAMPS_NUL		EXCEPTION;

existeix	VARCHAR2(1);
p_entrada	VARCHAR2(1000);
error_num	NUMBER;
error_missatge	VARCHAR2(100);

BEGIN

   --S'emmagatzema els parmetres d'entrada a la variable p_entrada

   p_entrada := 'P_REGIO: ' ||p_regio; 

   --Es verifica que tots els camps obligatoris contingin valor

   IF (TRIM(p_regio) IS NULL)
   THEN
      RAISE E_CAMPS_NUL;
   END IF;

   --Es verifica que el registre no s'hagi insertat anteriorment amb altre identificador.

   BEGIN
   
   SELECT 'Y' INTO existeix 
   FROM REGIO
   WHERE regio = p_regio AND ROWNUM = 1;

   RAISE E_REGIO_EXISTEIX;

   EXCEPTION
   WHEN NO_DATA_FOUND THEN
     NULL;
   END;

   --Si es pasa totes les proves de validaci, es realitzar l'inserci del nou registre a la Base de Dades.

   INSERT INTO REGIO (REGIO)
   VALUES (p_regio);
	
   RSP := 'OK';

   INSERT INTO LOG (NOMPROCEDIMENT, DATA, PARAMETREENTRADA, PARAMETRESORTIDA)
   VALUES ('PKG_CENSOS.PR_ALTA_REGIO', SYSDATE, p_entrada, 'RSP: '||RSP);

   COMMIT;

   --Excepcions
EXCEPTION

 --La regi existeix a la Base de Dades
   WHEN E_REGIO_EXISTEIX THEN RSP:= 'ERROR: : La regi ja existeix a la Base de Dades'; 

   INSERT INTO LOG (NOMPROCEDIMENT, DATA, PARAMETREENTRADA, PARAMETRESORTIDA)
   VALUES ('PKG_CENSOS.PR_ALTA_REGIO', SYSDATE, p_entrada, 'RSP: '||RSP);

   COMMIT;

 --Els camps obligatoris sn nuls
   WHEN E_CAMPS_NUL THEN RSP:= 'ERROR: : Els camps obligatoris no poden ser nul'; 

   INSERT INTO LOG (NOMPROCEDIMENT, DATA, PARAMETREENTRADA, PARAMETRESORTIDA)
   VALUES ('PKG_CENSOS.PR_ALTA_REGIO', SYSDATE, p_entrada, 'RSP: '||RSP);

   COMMIT;

   WHEN OTHERS THEN
   error_num := SQLCODE;
   error_missatge := SUBSTR(SQLERRM, 1, 200);

   RSP:= 'ERROR '||TO_CHAR(error_num)||': '||error_missatge;

   INSERT INTO LOG (NOMPROCEDIMENT, DATA, PARAMETREENTRADA, PARAMETRESORTIDA)
   VALUES ('PKG_CENSOS.PR_ALTA_REGIO', SYSDATE, p_entrada, 'RSP: '||RSP);

   COMMIT;

END PR_ALTA_REGIO;


/*******************************************************************************************************

NOM PROCEDIMENT: P_BAIXA_REGIO

FUNCIONALITAT:

Aquest procediment permet donar de baixa una regi a la Base de dades, si prviament supera els controls de:
	- Tots els camps obligatoris continguin valor.
	- Existeix la regi a la Base de Dades.

PARAMETRES:
ENTRADA
Obligatoris:
	p_idregio	Identificador de la regi

SORTIDA
Obligatoris:
	RSP	

PRECONDICI: Existeix la regi a donar de baixa.
POSTCONDICI: Es dona de baixa la regi.

RETORNA:
   SI TOT CORRECTE
	OK si loperaci sha realitzat amb xit
   SI ES PRODUEIX UNA EXCEPCI
	ERROR: La regi a donar de baixa no existeix a la Base de Dades.
	ERROR: El camp IDREGIO no pot ser nul.
	ERROR: Tipus derror. En qualsevol altre cas.


*******************************************************************************************************/

PROCEDURE PR_BAIXA_REGIO  (
	p_idregio   	IN NUMBER,
	RSP		OUT VARCHAR2
)

IS

E_REGIO_NOEXISTEIX	EXCEPTION;
E_CAMPS_NUL		EXCEPTION;

existeix	VARCHAR2(1);
p_entrada	VARCHAR2(1000);
error_num	NUMBER;
error_missatge	VARCHAR2(100);

BEGIN

   --S'emmagatzema els parmetres d'entrada a la variable p_entrada

   p_entrada := 'P_IDREGIO: '||p_idregio;

   --Es verifica que tots els camps obligatoris contingin valor

   IF (TRIM(p_idregio) IS NULL)
   THEN
      RAISE E_CAMPS_NUL;
   END IF;

   --Es verifica que existeix el registre a la Base de Dades.

   BEGIN
   
   SELECT 'Y' INTO existeix 
   FROM REGIO
   WHERE idregio = p_idregio AND ROWNUM = 1;

   EXCEPTION
   WHEN NO_DATA_FOUND THEN
     RAISE E_REGIO_NOEXISTEIX;
   END;

   --Si es pasa totes les proves de validaci, es realitzar la baixa del registre a la Base de Dades.

   DELETE FROM REGIO
   WHERE idregio = p_idregio;
	
   RSP := 'OK';

   INSERT INTO LOG (NOMPROCEDIMENT, DATA, PARAMETREENTRADA, PARAMETRESORTIDA)
   VALUES ('PKG_CENSOS.PR_BAIXA_REGIO', SYSDATE, p_entrada, 'RSP: '||RSP);

   COMMIT;

   --Excepcions
EXCEPTION

 --La regi no existeix a la Base de Dades
   WHEN E_REGIO_NOEXISTEIX THEN RSP:= 'ERROR: : La regi no existeix a la Base de Dades'; 

   INSERT INTO LOG (NOMPROCEDIMENT, DATA, PARAMETREENTRADA, PARAMETRESORTIDA)
   VALUES ('PKG_CENSOS.PR_BAIXA_REGIO', SYSDATE, p_entrada, 'RSP: '||RSP);

   COMMIT;

 --Els camps obligatoris sn nuls
   WHEN E_CAMPS_NUL THEN RSP:= 'ERROR: : El camp IDREGIO no pot ser nul'; 

   INSERT INTO LOG (NOMPROCEDIMENT, DATA, PARAMETREENTRADA, PARAMETRESORTIDA)
   VALUES ('PKG_CENSOS.PR_BAIXA_REGIO', SYSDATE, p_entrada, 'RSP: '||RSP);

   COMMIT;

   WHEN OTHERS THEN
   error_num := SQLCODE;
   error_missatge := SUBSTR(SQLERRM, 1, 200);

   RSP:= 'ERROR '||TO_CHAR(error_num)||': '||error_missatge;

   INSERT INTO LOG (NOMPROCEDIMENT, DATA, PARAMETREENTRADA, PARAMETRESORTIDA)
   VALUES ('PKG_CENSOS.PR_BAIXA_REGIO', SYSDATE, p_entrada, 'RSP: '||RSP);

   COMMIT;

END PR_BAIXA_REGIO;


/*******************************************************************************************************

NOM PROCEDIMENT: P_MODIFICACIO_REGIO

FUNCIONALITAT:

Aquest procediment permet modificar una regi a la Base de dades, si prviament supera els controls de:
	- La regi existeix a la Base de Dades.
	- Tots els camps obligatoris continguin valor.

PARAMETRES:
ENTRADA
Obligatoris:
	p_idregio	Identificador de la regi
Opcionals:
	p_regio	Nom de la regi

SORTIDA
Obligatoris:
	RSP
	
PRECONDICI: Existeix la regi a modificar.
POSTCONDICI: Es modifica la regi.

RETORNA:
   SI TOT CORRECTE
	OK si loperaci sha realitzat amb xit
   SI ES PRODUEIX UNA EXCEPCI
	ERROR: Lidentificador de la regi no pot ser nul.
	'ERROR: : La regi no existeix a la Base de Dades.'
	ERROR: Tipus derror. En qualsevol altre cas.


*******************************************************************************************************/

PROCEDURE PR_MODIFICACIO_REGIO (
	p_idregio	      IN NUMBER,
	p_regio		      IN VARCHAR2,	
	RSP		      OUT VARCHAR2
)

IS

E_CAMPS_NUL		EXCEPTION;
E_REGIO_NOEXISTEIX	EXCEPTION;

existeix	VARCHAR2(1);
p_entrada	VARCHAR2(1000);
error_num	NUMBER;
error_missatge	VARCHAR2(100);

BEGIN

   --S'emmagatzema els parmetres d'entrada a la variable p_entrada
 
   p_entrada := 'P_IDREGIO: '||p_idregio||', P_REGIO: ' ||p_regio;

   --Es verifica que tots els camps obligatoris contingin valor

   IF (TRIM(p_idregio) IS NULL) THEN
      RAISE E_CAMPS_NUL;
   END IF;

   --Es verifica que la regi existeix a la Base de Dades
   BEGIN

   SELECT 'Y' INTO existeix 
   FROM REGIO
   WHERE idregio = p_idregio AND ROWNUM = 1;

   EXCEPTION
   WHEN NO_DATA_FOUND THEN
      RAISE E_REGIO_NOEXISTEIX;
   END;

   --Si es pasa totes les proves de validaci, es realitzar l'actualitzaci a la Base de Dades.

   UPDATE REGIO
        SET  idregio		= nvl(p_idregio, idregio),
             regio	    	= nvl(p_regio, regio)
   WHERE     idregio     	= p_idregio;

   RSP := 'OK';

   INSERT INTO LOG (NOMPROCEDIMENT, DATA, PARAMETREENTRADA, PARAMETRESORTIDA)
   VALUES ('PKG_CENSOS.PR_MODIFICACIO_REGIO', SYSDATE, p_entrada, 'RSP: '||RSP);

   COMMIT;

   --Excepcions
EXCEPTION

 --Els camps obligatoris sn nuls
   WHEN E_CAMPS_NUL THEN RSP:= 'ERROR: : Lidentificador de la regi no pot ser nul'; 

   INSERT INTO LOG (NOMPROCEDIMENT, DATA, PARAMETREENTRADA, PARAMETRESORTIDA)
   VALUES ('PKG_CENSOS.PR_MODIFICACIO_REGIO', SYSDATE, p_entrada, 'RSP: '||RSP);

   COMMIT;

 --La regi no existeix a la Base de Dades
   WHEN E_REGIO_NOEXISTEIX THEN RSP:= 'ERROR: : La regi no existeix a la Base de Dades'; 

   INSERT INTO LOG (NOMPROCEDIMENT, DATA, PARAMETREENTRADA, PARAMETRESORTIDA)
   VALUES ('PKG_CENSOS.PR_MODIFICACIO_REGIO', SYSDATE, p_entrada, 'RSP: '||RSP);

   COMMIT; 

   WHEN OTHERS THEN
   error_num := SQLCODE;
   error_missatge := SUBSTR(SQLERRM, 1, 200);

   RSP:= 'ERROR '||TO_CHAR(error_num)||': '||error_missatge;

   INSERT INTO LOG (NOMPROCEDIMENT, DATA, PARAMETREENTRADA, PARAMETRESORTIDA)
   VALUES ('PKG_CENSOS.PR_MODIFICACIO_REGIO', SYSDATE, p_entrada, 'RSP: '||RSP);

   COMMIT;

END PR_MODIFICACIO_REGIO;


/********************************************************
*							*
*		MANTENIMENT PAISREGIO			*
*							*
********************************************************/

/*******************************************************************************************************

NOM PROCEDIMENT: P_ALTA_PAISREGIO

FUNCIONALITAT:

Aquest procediment assigna una regi a un pas, si prviament supera els controls de:
	- Tots els camps obligatoris continguin valor.
	- El pas no tingui assignat aquesta regi.
	- Existeix el pas a la base de dades.
	- Existeix la regi a la base de dades.

PARAMETRES:
ENTRADA
Obligatoris:
	p_idpais	Identificador del pas
	p_idregio	Identificador de la regi

SORTIDA
Obligatoris:
	RSP	

PRECONDICI: Existeix el pas i la regi a donar dalta.
POSTCONDICI: Es dona dalta lassignaci de la regi al pas.

RETORNA:
   SI TOT CORRECTE
	OK si loperaci sha realitzat amb xit
   SI ES PRODUEIX UNA EXCEPCI
	ERROR: Els camps obligatoris no poden ser nul.
	ERROR: La regi no existeix a la Base de Dades.
	ERROR: El pas no existeix a la Base de Dades.
	ERROR: El pas ja te assignada una regi.
	ERROR: Tipus derror. En qualsevol altre cas.

*******************************************************************************************************/

PROCEDURE PR_ALTA_PAISREGIO  (
        p_idpais	IN VARCHAR2,
	p_idregio	IN NUMBER,
	RSP		OUT VARCHAR2
)

IS

E_PAIS_NOEXISTEIX	EXCEPTION;
E_REGIO_NOEXISTEIX	EXCEPTION;
E_ASIGNACIO_EXISTEIX	EXCEPTION;
E_CAMPS_NUL		EXCEPTION;

existeix	VARCHAR2(1);
p_entrada	VARCHAR2(1000);
error_num	NUMBER;
error_missatge	VARCHAR2(100);

BEGIN

   --S'emmagatzema els parmetres d'entrada a la variable p_entrada

   p_entrada := 'P_IDPAIS: '||p_idpais||', P_IDREGIO:'||p_idregio; 

   --Es verifica que tots els camps obligatoris contingin valor

   IF (TRIM(p_idpais) IS NULL OR
	TRIM(p_idregio) IS NULL)
   THEN
      RAISE E_CAMPS_NUL;
   END IF;

   --Es verifica que l'assignaci no estigui feta a la Base de Dades.

   BEGIN
   
   SELECT 'Y' INTO existeix 
   FROM PAISREGIO
   WHERE idpais = p_idpais AND idregio = p_idregio AND ROWNUM = 1;

   RAISE E_ASIGNACIO_EXISTEIX;

   EXCEPTION
   WHEN NO_DATA_FOUND THEN
     NULL;
   END;

   --Es verifica que el pais existeix a la Base de Dades.

   BEGIN

   SELECT 'Y' INTO existeix 
   FROM PAIS
   WHERE codpais = p_idpais AND ROWNUM = 1;

   EXCEPTION
   WHEN NO_DATA_FOUND THEN
     RAISE E_PAIS_NOEXISTEIX;
   END;

   --Es verifica que la regi existeix a la Base de Dades.

   BEGIN

   SELECT 'Y' INTO existeix 
   FROM REGIO
   WHERE idregio = p_idregio AND ROWNUM = 1;

   EXCEPTION
   WHEN NO_DATA_FOUND THEN
     RAISE E_REGIO_NOEXISTEIX;
   END;

   --Si es pasa totes les proves de validaci, es realitzar l'inserci del nou registre a la Base de Dades.

   INSERT INTO PAISREGIO (IDPAIS, IDREGIO)
   VALUES (p_idpais, p_idregio);
	
   RSP := 'OK';

   INSERT INTO LOG (NOMPROCEDIMENT, DATA, PARAMETREENTRADA, PARAMETRESORTIDA)
   VALUES ('PKG_CENSOS.PR_ALTA_PAISREGIO', SYSDATE, p_entrada, 'RSP: '||RSP);

   COMMIT;

   --Excepcions
EXCEPTION

 --La regi no existeix a la Base de Dades
   WHEN E_REGIO_NOEXISTEIX THEN RSP:= 'ERROR: : La regi no existeix a la Base de Dades'; 

   INSERT INTO LOG (NOMPROCEDIMENT, DATA, PARAMETREENTRADA, PARAMETRESORTIDA)
   VALUES ('PKG_CENSOS.PR_ALTA_PAISREGIO', SYSDATE, p_entrada, 'RSP: '||RSP);

   COMMIT;

 --El pas no existeix a la Base de Dades
   WHEN E_PAIS_NOEXISTEIX THEN RSP:= 'ERROR: : El pas no existeix a la Base de Dades'; 

   INSERT INTO LOG (NOMPROCEDIMENT, DATA, PARAMETREENTRADA, PARAMETRESORTIDA)
   VALUES ('PKG_CENSOS.PR_ALTA_PAISREGIO', SYSDATE, p_entrada, 'RSP: '||RSP);

   COMMIT;

 --Els camps obligatoris sn nuls
   WHEN E_CAMPS_NUL THEN RSP:= 'ERROR: : Els camps obligatoris no poden ser nul'; 

   INSERT INTO LOG (NOMPROCEDIMENT, DATA, PARAMETREENTRADA, PARAMETRESORTIDA)
   VALUES ('PKG_CENSOS.PR_ALTA_PAISREGIO', SYSDATE, p_entrada, 'RSP: '||RSP);

   COMMIT;

   WHEN OTHERS THEN
   error_num := SQLCODE;
   error_missatge := SUBSTR(SQLERRM, 1, 200);

   RSP:= 'ERROR '||TO_CHAR(error_num)||': '||error_missatge;

   INSERT INTO LOG (NOMPROCEDIMENT, DATA, PARAMETREENTRADA, PARAMETRESORTIDA)
   VALUES ('PKG_CENSOS.PR_ALTA_PAISREGIO', SYSDATE, p_entrada, 'RSP: '||RSP);

   COMMIT;

END PR_ALTA_PAISREGIO;


/*******************************************************************************************************

NOM PROCEDIMENT: P_BAIXA_PAISREGIO

FUNCIONALITAT:

Aquest procediment permet donar de baixa lassignaci de la regi a un pas, si prviament supera els controls de:
	- Tots els camps obligatoris continguin valor.
	- Existeix lassignaci a la Base de Dades.

PARAMETRES:
ENTRADA
Obligatoris:
	p_idpaisregio	Identificaci de lassignaci duna regi a un pas.

SORTIDA
Obligatoris:
	RSP	

PRECONDICI: Existeix lassignaci de la regi al pas a donar de baixa.
POSTCONDICI: Es dona de baixa lassignaci de la regi al pas.

RETORNA:
   SI TOT CORRECTE
	OK si loperaci sha realitzat amb xit
   SI ES PRODUEIX UNA EXCEPCI
	ERROR: No existeix lassignaci a la Base de Dades.
	ERROR: El camp IDPAISREGIO no pot ser nul.
	ERROR: Tipus derror. En qualsevol altre cas.

*******************************************************************************************************/

PROCEDURE PR_BAIXA_PAISREGIO  (
	p_idpaisregio   IN NUMBER,
	RSP		OUT VARCHAR2
)

IS

E_PAISREGIO_NOEXISTEIX	EXCEPTION;
E_CAMPS_NUL		EXCEPTION;

existeix	VARCHAR2(1);
p_entrada	VARCHAR2(1000);
error_num	NUMBER;
error_missatge	VARCHAR2(100);

BEGIN

   --S'emmagatzema els parmetres d'entrada a la variable p_entrada

   p_entrada := 'P_IDPAISREGIO: '||p_idpaisregio;

   --Es verifica que tots els camps obligatoris contingin valor

   IF (TRIM(p_idpaisregio) IS NULL)
   THEN
      RAISE E_CAMPS_NUL;
   END IF;

   --Es verifica que existeix el registre a la Base de Dades.

   BEGIN
   
   SELECT 'Y' INTO existeix 
   FROM PAISREGIO
   WHERE idpaisregio = p_idpaisregio AND ROWNUM = 1;

   EXCEPTION
   WHEN NO_DATA_FOUND THEN
     RAISE E_PAISREGIO_NOEXISTEIX;
   END;

   --Si es pasa totes les proves de validaci, es realitzar la baixa del registre a la Base de Dades.

   DELETE FROM PAISREGIO
   WHERE idpaisregio = p_idpaisregio;
	
   RSP := 'OK';

   INSERT INTO LOG (NOMPROCEDIMENT, DATA, PARAMETREENTRADA, PARAMETRESORTIDA)
   VALUES ('PKG_CENSOS.PR_BAIXA_PAISREGIO', SYSDATE, p_entrada, 'RSP: '||RSP);

   COMMIT;

   --Excepcions
EXCEPTION

 --El paisregi no existeix a la Base de Dades
   WHEN E_PAISREGIO_NOEXISTEIX THEN RSP:= 'ERROR: : No existeix lassignaci a la Base de Dades'; 

   INSERT INTO LOG (NOMPROCEDIMENT, DATA, PARAMETREENTRADA, PARAMETRESORTIDA)
   VALUES ('PKG_CENSOS.PR_BAIXA_PAISREGIO', SYSDATE, p_entrada, 'RSP: '||RSP);

   COMMIT;

 --Els camps obligatoris sn nuls
   WHEN E_CAMPS_NUL THEN RSP:= 'ERROR: : El camp IDPAISREGIO no pot ser nul'; 

   INSERT INTO LOG (NOMPROCEDIMENT, DATA, PARAMETREENTRADA, PARAMETRESORTIDA)
   VALUES ('PKG_CENSOS.PR_BAIXA_PAISREGIO', SYSDATE, p_entrada, 'RSP: '||RSP);

   COMMIT;

   WHEN OTHERS THEN
   error_num := SQLCODE;
   error_missatge := SUBSTR(SQLERRM, 1, 200);

   RSP:= 'ERROR '||TO_CHAR(error_num)||': '||error_missatge;

   INSERT INTO LOG (NOMPROCEDIMENT, DATA, PARAMETREENTRADA, PARAMETRESORTIDA)
   VALUES ('PKG_CENSOS.PR_BAIXA_PAISREGIO', SYSDATE, p_entrada, 'RSP: '||RSP);

   COMMIT;

END PR_BAIXA_PAISREGIO;


/*******************************************************************************************************

NOM PROCEDIMENT: P_MODIFICACIO_PAISREGIO

FUNCIONALITAT:

Aquest procediment permet modificar lassignaci duna regi a un pas, si prviament supera els controls de:
	- El registre a modificar existeix a la Base de Dades.
	- Tots els camps obligatoris continguin valor.
	- Existeix la regi nova a la Base de Dades.
	- Existeix el nou pas a la Base de Dades.
	- No existeix el nou registre a la Base de Dades.

PARAMETRES:
ENTRADA
Obligatoris:
	p_idpaisregio	Identificaci de lassignaci duna regi a un pas.
	p_idpais	Identificador del pas
	p_idregio	Identificador de la regi

SORTIDA
Obligatoris:
	RSP	

PRECONDICI: Existeix lassignaci de la regi a un pas a modificar.
POSTCONDICI: Es modifica lassignaci duna regi a un pas.

RETORNA:
   SI TOT CORRECTE
	OK si loperaci sha realitzat amb xit
   SI ES PRODUEIX UNA EXCEPCI
	ERROR: Els camps obligatoris no poden ser nul.
	ERROR: El pas no t assignada la regi a modificar.
	ERROR: La regi no existeix a la Base de Dades .
	ERROR: El pas no existeix a la Base de Dades.
	ERROR: Els canvis a realitzar ja existeixen a la Base de Dades.
	ERROR: Tipus derror. En qualsevol altre cas.

*******************************************************************************************************/

PROCEDURE PR_MODIFICACIO_PAISREGIO (
	p_idpaisregio	IN NUMBER,
	p_idpais	IN VARCHAR2,
	p_idregio	IN NUMBER,	
	RSP		OUT VARCHAR2
)

IS

E_CAMPS_NUL		EXCEPTION;
E_REGIO_NOEXISTEIX	EXCEPTION;
E_PAISREGIO_NOEXISTEIX	EXCEPTION;
E_PAIS_NOEXISTEIX	EXCEPTION;
E_ASSIGNACIO_EXISTEIX	EXCEPTION;

existeix	VARCHAR2(1);
p_entrada	VARCHAR2(1000);
error_num	NUMBER;
error_missatge	VARCHAR2(100);

BEGIN

   --S'emmagatzema els parmetres d'entrada a la variable p_entrada
 
   p_entrada := 'P_IDPAISREGIO: '||p_idpaisregio||', P_IDPAIS: '||p_idpais||
		', P_IDREGIO: ' ||p_idregio;

   --Es verifica que tots els camps obligatoris contingin valor

   IF (TRIM(p_idpaisregio) IS NULL OR
	TRIM(p_idpais) IS NULL OR
	TRIM(p_idregio) IS NULL) THEN
      RAISE E_CAMPS_NUL;
   END IF;

   --Es verifica que el registre a modificar existeix
   BEGIN

   SELECT 'Y' INTO existeix 
   FROM PAISREGIO
   WHERE idpaisregio = p_idpaisregio AND ROWNUM = 1;

   EXCEPTION
   WHEN NO_DATA_FOUND THEN
      RAISE E_PAISREGIO_NOEXISTEIX;
   END;

   --Es verifica que la nova regi existeix a la Base de Dades
   BEGIN

   SELECT 'Y' INTO existeix 
   FROM REGIO
   WHERE idregio = p_idregio AND ROWNUM = 1;

   EXCEPTION
   WHEN NO_DATA_FOUND THEN
      RAISE E_REGIO_NOEXISTEIX;
   END;

   --Es verifica que el nou pas existeix a la Base de Dades
   BEGIN

   SELECT 'Y' INTO existeix 
   FROM PAIS
   WHERE codpais = p_idpais AND ROWNUM = 1;

   EXCEPTION
   WHEN NO_DATA_FOUND THEN
      RAISE E_PAIS_NOEXISTEIX;
   END;

   --Verifica la nova assignaci no existeix a la base de dades
   BEGIN

   SELECT 'Y' INTO existeix 
   FROM PAISREGIO
   WHERE idpais = p_idpais AND idregio = p_idregio AND ROWNUM = 1;

   RAISE E_ASSIGNACIO_EXISTEIX;

   EXCEPTION
   WHEN NO_DATA_FOUND THEN
	NULL;      
   END;

   --Si es pasa totes les proves de validaci, es realitzar l'actualitzaci a la Base de Dades.

   UPDATE PAISREGIO
        SET  idpais		= nvl(p_idpais, idpais),
             idregio	    	= nvl(p_idregio, idregio)
   WHERE     idpaisregio     	= p_idpaisregio;

   RSP := 'OK';

   INSERT INTO LOG (NOMPROCEDIMENT, DATA, PARAMETREENTRADA, PARAMETRESORTIDA)
   VALUES ('PKG_CENSOS.PR_MODIFICACIO_PAISREGIO', SYSDATE, p_entrada, 'RSP: '||RSP);

   COMMIT;

   --Excepcions
EXCEPTION

 --Els camps obligatoris sn nuls
   WHEN E_CAMPS_NUL THEN RSP:= 'ERROR: : Els camps obligatoris no poden ser nul'; 

   INSERT INTO LOG (NOMPROCEDIMENT, DATA, PARAMETREENTRADA, PARAMETRESORTIDA)
   VALUES ('PKG_CENSOS.PR_MODIFICACIO_PAISREGIO', SYSDATE, p_entrada, 'RSP: '||RSP);

   COMMIT;

 --L'assignaci no existeix a la Base de Dades
   WHEN E_PAISREGIO_NOEXISTEIX THEN RSP:= 'ERROR: : El pas no t assignada la regi a modificar'; 

   INSERT INTO LOG (NOMPROCEDIMENT, DATA, PARAMETREENTRADA, PARAMETRESORTIDA)
   VALUES ('PKG_CENSOS.PR_MODIFICACIO_PAISREGIO', SYSDATE, p_entrada, 'RSP: '||RSP);

   COMMIT; 

 --La regi no existeix a la Base de Dades
   WHEN E_REGIO_NOEXISTEIX THEN RSP:= 'ERROR: : La regi no existeix a la Base de Dades'; 

   INSERT INTO LOG (NOMPROCEDIMENT, DATA, PARAMETREENTRADA, PARAMETRESORTIDA)
   VALUES ('PKG_CENSOS.PR_MODIFICACIO_PAISREGIO', SYSDATE, p_entrada, 'RSP: '||RSP);

   COMMIT;

 --El pas no existeix a la Base de Dades
   WHEN E_PAIS_NOEXISTEIX THEN RSP:= 'ERROR: : El pas no existeix a la Base de Dades'; 

   INSERT INTO LOG (NOMPROCEDIMENT, DATA, PARAMETREENTRADA, PARAMETRESORTIDA)
   VALUES ('PKG_CENSOS.PR_MODIFICACIO_PAISREGIO', SYSDATE, p_entrada, 'RSP: '||RSP);

   COMMIT;

 --Ja existeix l'assignaci a la Base de Dades
   WHEN E_ASSIGNACIO_EXISTEIX THEN RSP:= 'ERROR: : Els canvis a realitzar ja existeixen a la Base de Dades'; 

   INSERT INTO LOG (NOMPROCEDIMENT, DATA, PARAMETREENTRADA, PARAMETRESORTIDA)
   VALUES ('PKG_CENSOS.PR_MODIFICACIO_PAISREGIO', SYSDATE, p_entrada, 'RSP: '||RSP);

   COMMIT;

   WHEN OTHERS THEN
   error_num := SQLCODE;
   error_missatge := SUBSTR(SQLERRM, 1, 200);

   RSP:= 'ERROR '||TO_CHAR(error_num)||': '||error_missatge;

   INSERT INTO LOG (NOMPROCEDIMENT, DATA, PARAMETREENTRADA, PARAMETRESORTIDA)
   VALUES ('PKG_CENSOS.PR_MODIFICACIO_PAISREGIO', SYSDATE, p_entrada, 'RSP: '||RSP);

   COMMIT;

END PR_MODIFICACIO_PAISREGIO;


/********************************************************
*							*
*		MANTENIMENT LOCALITAT			*
*							*
********************************************************/

/*******************************************************************************************************

NOM PROCEDIMENT: P_ALTA_LOCALITAT

FUNCIONALITAT:

Aquest procediment permet donar dalta una localitat a la Base de dades, si prviament supera els controls de:
	- Tots els camps obligatoris continguin valor.
	- El registre no shagi inserit amb un altre identificador.
	- Existeix la regi a introduir.

PARAMETRES:
ENTRADA
Obligatoris:
	p_idregio	Identificador de la regi
	p_localitat	Nom de la localitat

SORTIDA
Obligatoris:
	RSP	

PRECONDICI: No existeix la localitat a crear.
POSTCONDICI: Es dona dalta la localitat.

RETORNA:
   SI TOT CORRECTE
	OK si loperaci sha realitzat amb xit
   SI ES PRODUEIX UNA EXCEPCI
	ERROR: La localitat ja existeix a la Base de Dades.
	ERROR: La regi no existeix a la Base de Dades.
	ERROR: Els camps obligatoris no poden ser nuls.
	ERROR: Tipus derror. En qualsevol altre cas.

*******************************************************************************************************/

PROCEDURE PR_ALTA_LOCALITAT  (
	p_idregio 	   IN NUMBER,
        p_localitat	   IN VARCHAR2,
	RSP		   OUT VARCHAR2
)

IS

E_LOCALITAT_EXISTEIX	EXCEPTION;
E_CAMPS_NUL		EXCEPTION;
E_REGIO_NOEXISTEIX	EXCEPTION;

existeix	VARCHAR2(1);
p_entrada	VARCHAR2(1000);
error_num	NUMBER;
error_missatge	VARCHAR2(100);

BEGIN

   --S'emmagatzema els parmetres d'entrada a la variable p_entrada

   p_entrada := 'P_IDREGIO: ' ||p_idregio||', P_LOCALITAT: '||p_localitat;

   --Es verifica que tots els camps obligatoris contingin valor

   IF (TRIM(p_idregio) IS NULL OR 
	TRIM(p_localitat) IS NULL)
   THEN
      RAISE E_CAMPS_NUL;
   END IF;

   --Es verifica que el registre no s'hagi insertat anteriorment amb altre identificador.

   BEGIN
   
   SELECT 'Y' INTO existeix 
   FROM LOCALITAT
   WHERE idregio = p_idregio AND localitat = p_localitat AND ROWNUM = 1;

   RAISE E_LOCALITAT_EXISTEIX;

   EXCEPTION
   WHEN NO_DATA_FOUND THEN
     NULL;
   END;

   --Es verifica que la regio que es pasa per parmetre existeix

   BEGIN
   
   SELECT 'Y' INTO existeix 
   FROM REGIO
   WHERE idregio = p_idregio AND ROWNUM = 1;   

   EXCEPTION
   WHEN NO_DATA_FOUND THEN
     RAISE E_REGIO_NOEXISTEIX;
   END;

   --Si es pasa totes les proves de validaci, es realitzar l'inserci del nou registre a la Base de Dades.

   INSERT INTO LOCALITAT (IDREGIO, LOCALITAT)
   VALUES (p_idregio, p_localitat);
	
   RSP := 'OK';

   INSERT INTO LOG (NOMPROCEDIMENT, DATA, PARAMETREENTRADA, PARAMETRESORTIDA)
   VALUES ('PKG_CENSOS.PR_ALTA_LOCALITAT', SYSDATE, p_entrada, 'RSP: '||RSP);

   COMMIT;

   --Excepcions
EXCEPTION

 --La localitat existeix a la Base de Dades
   WHEN E_LOCALITAT_EXISTEIX THEN RSP:= 'ERROR: : La localitat ja existeix a la Base de Dades'; 

   INSERT INTO LOG (NOMPROCEDIMENT, DATA, PARAMETREENTRADA, PARAMETRESORTIDA)
   VALUES ('PKG_CENSOS.PR_ALTA_LOCALITAT', SYSDATE, p_entrada, 'RSP: '||RSP);

   COMMIT;

 --Els camps obligatoris sn nuls
   WHEN E_CAMPS_NUL THEN RSP:= 'ERROR: : Els camps obligatoris no poden ser nul'; 

   INSERT INTO LOG (NOMPROCEDIMENT, DATA, PARAMETREENTRADA, PARAMETRESORTIDA)
   VALUES ('PKG_CENSOS.PR_ALTA_LOCALITAT', SYSDATE, p_entrada, 'RSP: '||RSP);

   COMMIT;

 --La regi no existeix a la Base de Dades
   WHEN E_REGIO_NOEXISTEIX THEN RSP:= 'ERROR: : La regi no existeix a la Base de Dades'; 

   INSERT INTO LOG (NOMPROCEDIMENT, DATA, PARAMETREENTRADA, PARAMETRESORTIDA)
   VALUES ('PKG_CENSOS.PR_ALTA_LOCALITAT', SYSDATE, p_entrada, 'RSP: '||RSP);

   COMMIT;

   WHEN OTHERS THEN
   error_num := SQLCODE;
   error_missatge := SUBSTR(SQLERRM, 1, 200);

   RSP:= 'ERROR '||TO_CHAR(error_num)||': '||error_missatge;

   INSERT INTO LOG (NOMPROCEDIMENT, DATA, PARAMETREENTRADA, PARAMETRESORTIDA)
   VALUES ('PKG_CENSOS.PR_ALTA_LOCALITAT', SYSDATE, p_entrada, 'RSP: '||RSP);

   COMMIT;

END PR_ALTA_LOCALITAT;


/*******************************************************************************************************

NOM PROCEDIMENT: P_BAIXA_LOCALITAT

FUNCIONALITAT:

Aquest procediment permet donar de baixa una localitat a la Base de dades, si prviament supera els controls de:
	- Tots els camps obligatoris continguin valor.
	- Existeix la localitat a la Base de Dades.

PARAMETRES:
ENTRADA
Obligatoris:
	p_idlocalitat	Identificador de la localitat

SORTIDA
Obligatoris:
	RSP	

PRECONDICI: Existeix la localitat a donar de baixa.
POSTCONDICI: Es dona de baixa la localitat.

RETORNA:
   SI TOT CORRECTE
	OK si loperaci sha realitzat amb xit
   SI ES PRODUEIX UNA EXCEPCI
	ERROR: La localitat no existeix a la Base de Dades.
	ERROR: El camp IDLOCALITAT no pot ser nul.
	ERROR: Tipus derror. En qualsevol altre cas.

*******************************************************************************************************/

PROCEDURE PR_BAIXA_LOCALITAT  (
	p_idlocalitat  	IN NUMBER,
	RSP		OUT VARCHAR2
)

IS

E_LOCALITAT_NOEXISTEIX	EXCEPTION;
E_CAMPS_NUL		EXCEPTION;

existeix	VARCHAR2(1);
p_entrada	VARCHAR2(1000);
error_num	NUMBER;
error_missatge	VARCHAR2(100);

BEGIN

   --S'emmagatzema els parmetres d'entrada a la variable p_entrada

   p_entrada := 'P_IDLOCALITAT: '||p_idlocalitat;

   --Es verifica que tots els camps obligatoris contingin valor

   IF (TRIM(p_idlocalitat) IS NULL)
   THEN
      RAISE E_CAMPS_NUL;
   END IF;

   --Es verifica que el registre no s'hagi insertat anteriorment amb altre identificador.

   BEGIN
   
   SELECT 'Y' INTO existeix 
   FROM LOCALITAT 
   WHERE idlocalitat = p_idlocalitat AND ROWNUM = 1;

   EXCEPTION
   WHEN NO_DATA_FOUND THEN
     RAISE E_LOCALITAT_NOEXISTEIX;
   END;

   --Si es pasa totes les proves de validaci, es realitzar la baixa del registre a la Base de Dades.

   DELETE FROM LOCALITAT
   WHERE idlocalitat = p_idlocalitat;
	
   RSP := 'OK';

   INSERT INTO LOG (NOMPROCEDIMENT, DATA, PARAMETREENTRADA, PARAMETRESORTIDA)
   VALUES ('PKG_CENSOS.PR_BAIXA_LOCALITAT', SYSDATE, p_entrada, 'RSP: '||RSP);

   COMMIT;

   --Excepcions
EXCEPTION

 --La localitat no existeix a la Base de Dades
   WHEN E_LOCALITAT_NOEXISTEIX THEN RSP:= 'ERROR: : La localitat no existeix a la Base de Dades'; 

   INSERT INTO LOG (NOMPROCEDIMENT, DATA, PARAMETREENTRADA, PARAMETRESORTIDA)
   VALUES ('PKG_CENSOS.PR_BAIXA_LOCALITAT', SYSDATE, p_entrada, 'RSP: '||RSP);

   COMMIT;

 --Els camps obligatoris sn nuls
   WHEN E_CAMPS_NUL THEN RSP:= 'ERROR: : El camp IDLOCALITAT no pot ser nul'; 

   INSERT INTO LOG (NOMPROCEDIMENT, DATA, PARAMETREENTRADA, PARAMETRESORTIDA)
   VALUES ('PKG_CENSOS.PR_BAIXA_LOCALITAT', SYSDATE, p_entrada, 'RSP: '||RSP);

   COMMIT;

   WHEN OTHERS THEN
   error_num := SQLCODE;
   error_missatge := SUBSTR(SQLERRM, 1, 200);

   RSP:= 'ERROR '||TO_CHAR(error_num)||': '||error_missatge;

   INSERT INTO LOG (NOMPROCEDIMENT, DATA, PARAMETREENTRADA, PARAMETRESORTIDA)
   VALUES ('PKG_CENSOS.PR_BAIXA_LOCALITAT', SYSDATE, p_entrada, 'RSP: '||RSP);

   COMMIT;

END PR_BAIXA_LOCALITAT;


/*******************************************************************************************************

NOM PROCEDIMENT: P_MODIFICACIO_LOCALITAT

FUNCIONALITAT:

Aquest procediment permet modificar una localitat a la Base de dades, si prviament supera els controls de:
	- La localitat existeix a la Base de Dades.
	- Tots els camps obligatoris continguin valor.
	- Si es vol modificar la regi, sha de verificar que existeix la regi a la Base de Dades.

PARAMETRES:
ENTRADA
Obligatoris:
	p_idlocalitat	Identificador de la localitat
Opcionals:
	p_idregio	Identificador de la regi
	p_localitat	Nom de la localitat

SORTIDA
Obligatoris:
	RSP	

PRECONDICI: Existeix la localitat a modificar.
POSTCONDICI: Es modifica la localitat.

RETORNA:
   SI TOT CORRECTE
	OK si loperaci sha realitzat amb xit
   SI ES PRODUEIX UNA EXCEPCI
	ERROR: La localitat no existeix a la Base de Dades.
	ERROR: Els camps obligatoris no poden ser nul.
	ERROR: La regi no existeix a la Base de Dades.
	ERROR: Tipus derror. En qualsevol altre cas.

*******************************************************************************************************/

PROCEDURE PR_MODIFICACIO_LOCALITAT (
	p_idlocalitat	      IN NUMBER,
	p_idregio	      IN NUMBER,	
	p_localitat 	      IN VARCHAR2,
	RSP		      OUT VARCHAR2
)

IS

E_CAMPS_NUL		EXCEPTION;
E_LOCALITAT_NOEXISTEIX	EXCEPTION;
E_REGIO_NOEXISTEIX	EXCEPTION;

existeix	VARCHAR2(1);
p_entrada	VARCHAR2(1000);
error_num	NUMBER;
error_missatge	VARCHAR2(100);

BEGIN

   --S'emmagatzema els parmetres d'entrada a la variable p_entrada
 
   p_entrada := 'P_IDLOCALITAT: '||p_idlocalitat||', P_IDREGIO: ' ||p_idregio||
		', P_LOCALITAT: '||p_localitat;

   --Es verifica que tots els camps obligatoris contingin valor

   IF (TRIM(p_idlocalitat) IS NULL) THEN
      RAISE E_CAMPS_NUL;
   END IF;

   --Es verifica que la localitat existeix a la Base de Dades
   BEGIN

   SELECT 'Y' INTO existeix 
   FROM LOCALITAT
   WHERE idlocalitat = p_idlocalitat AND ROWNUM = 1;

   EXCEPTION
   WHEN NO_DATA_FOUND THEN
      RAISE E_LOCALITAT_NOEXISTEIX;
   END;

   --Si es modifica la regi, es verifica que la regi existeix a la Base de Dades
   BEGIN

   SELECT 'Y' INTO existeix 
   FROM REGIO
   WHERE idregio = p_idregio AND ROWNUM = 1;

   EXCEPTION
   WHEN NO_DATA_FOUND THEN
      RAISE E_REGIO_NOEXISTEIX;
   END;

   --Si es pasa totes les proves de validaci, es realitzar l'actualitzaci a la Base de Dades.

   UPDATE LOCALITAT
        SET  idregio		= nvl(p_idregio, idregio),
             localitat	    	= nvl(p_localitat, localitat)
   WHERE   idlocalitat      	= p_idlocalitat;

   RSP := 'OK';

   INSERT INTO LOG (NOMPROCEDIMENT, DATA, PARAMETREENTRADA, PARAMETRESORTIDA)
   VALUES ('PKG_CENSOS.PR_MODIFICACIO_LOCALITAT', SYSDATE, p_entrada, 'RSP: '||RSP);

   COMMIT;

   --Excepcions
EXCEPTION

 --Els camps obligatoris sn nuls
   WHEN E_CAMPS_NUL THEN RSP:= 'ERROR: : Els camps obligatoris no poden ser nul'; 

   INSERT INTO LOG (NOMPROCEDIMENT, DATA, PARAMETREENTRADA, PARAMETRESORTIDA)
   VALUES ('PKG_CENSOS.PR_MODIFICACIO_LOCALITAT', SYSDATE, p_entrada, 'RSP: '||RSP);

   COMMIT;

 --La localitat no existeix a la Base de Dades
   WHEN E_LOCALITAT_NOEXISTEIX THEN RSP:= 'ERROR: : La localitat no existeix a la Base de Dades'; 

   INSERT INTO LOG (NOMPROCEDIMENT, DATA, PARAMETREENTRADA, PARAMETRESORTIDA)
   VALUES ('PKG_CENSOS.PR_MODIFICACIO_LOCALITAT', SYSDATE, p_entrada, 'RSP: '||RSP);

   COMMIT; 

 --La regi no existeix a la Base de Dades
   WHEN E_REGIO_NOEXISTEIX THEN RSP:= 'ERROR: : La regi no existeix a la Base de Dades'; 

   INSERT INTO LOG (NOMPROCEDIMENT, DATA, PARAMETREENTRADA, PARAMETRESORTIDA)
   VALUES ('PKG_CENSOS.PR_MODIFICACIO_LOCALITAT', SYSDATE, p_entrada, 'RSP: '||RSP);

   COMMIT; 

   WHEN OTHERS THEN
   error_num := SQLCODE;
   error_missatge := SUBSTR(SQLERRM, 1, 200);

   RSP:= 'ERROR '||TO_CHAR(error_num)||': '||error_missatge;

   INSERT INTO LOG (NOMPROCEDIMENT, DATA, PARAMETREENTRADA, PARAMETRESORTIDA)
   VALUES ('PKG_CENSOS.PR_MODIFICACIO_LOCALITAT', SYSDATE, p_entrada, 'RSP: '||RSP);

   COMMIT;

END PR_MODIFICACIO_LOCALITAT;


/********************************************************
*							*
*		MANTENIMENT CENSPAIS			*
*							*
********************************************************/


/*******************************************************************************************************

NOM PROCEDIMENT: P_ALTA_CENSPAIS

FUNCIONALITAT:

Aquest procediment permet donar dalta el cens pas, si prviament supera els controls de:
	- Tots els camps obligatoris continguin valor.
	- El registre no shagi inserit amb un altre identificador.
	- La votaci est donada dalta a la Base de Dades. 
	- El pas est donat dalta a la Base de Dades.

PARAMETRES:
ENTRADA
Obligatoris:
	p_idcenspais	Identificador del censpais
	p_idvotacio	Identificador de la votaci
	p_idpais	Identificador del pas

SORTIDA
Obligatoris:
	RSP	

PRECONDICI: No existeix el cens pas crear.
POSTCONDICI: Es dona dalta el cens pas.

RETORNA
   SI TOT CORRECTE
	OK si loperaci sha realitzat amb xit
   SI ES PRODUEIX UNA EXCEPCI
	ERROR: El cens-pas ja existeix a la Base de Dades.
	ERROR: El pas no existeix a la Base de Dades.
	ERROR: La votaci no existeix a la Base de Dades.
	ERROR: Els camps obligatoris no poden ser nuls.
	ERROR: Tipus derror. En qualsevol altre cas.

*******************************************************************************************************/

PROCEDURE PR_ALTA_CENSPAIS  (
	p_idcenspais	   IN VARCHAR2,
	p_idvotacio 	   IN NUMBER,
        p_idpais	   IN VARCHAR2,
	RSP		   OUT VARCHAR2
)

IS

E_CAMPS_NUL		EXCEPTION;
E_CENSPAIS_EXISTEIX	EXCEPTION;
E_VOTACIO_NOEXISTEIX	EXCEPTION;
E_PAIS_NOEXISTEIX	EXCEPTION;

existeix	VARCHAR2(1);
p_entrada	VARCHAR2(1000);
error_num	NUMBER;
error_missatge	VARCHAR2(100);

BEGIN

   --S'emmagatzema els parmetres d'entrada a la variable p_entrada

   p_entrada := 'P_IDCENSPAIS: '||p_idcenspais||', P_IDVOTACIO: ' ||p_idvotacio||', P_IDPAIS: '||p_idpais;

   --Es verifica que tots els camps obligatoris contingin valor

   IF (TRIM(p_idcenspais) IS NULL OR 
	TRIM(p_idvotacio) IS NULL OR 
	TRIM(p_idpais) IS NULL)
   THEN
      RAISE E_CAMPS_NUL;
   END IF;

   --Es verifica que el registre no s'hagi insertat anteriorment amb altre identificador.

   BEGIN
   
   SELECT 'Y' INTO existeix 
   FROM CENSPAIS
   WHERE idcenspais=p_idcenspais AND ROWNUM = 1;

   RAISE E_CENSPAIS_EXISTEIX;

   EXCEPTION
   WHEN NO_DATA_FOUND THEN
     NULL;
   END;

   --Es verifica que la votacio que es pasa per parmetre existeix

   BEGIN
   
   SELECT 'Y' INTO existeix 
   FROM VOTACIO 
   WHERE idvotacio = p_idvotacio AND ROWNUM = 1;   

   EXCEPTION
   WHEN NO_DATA_FOUND THEN
     RAISE E_VOTACIO_NOEXISTEIX;
   END;

   --Es verifica que el pas que es pasa per parmetre existeix

   BEGIN
   
   SELECT 'Y' INTO existeix 
   FROM PAIS
   WHERE codpais = p_idpais AND ROWNUM = 1;   

   EXCEPTION
   WHEN NO_DATA_FOUND THEN
     RAISE E_PAIS_NOEXISTEIX;
   END;

   --Si es pasa totes les proves de validaci, es realitzar l'inserci del nou registre a la Base de Dades.

   INSERT INTO CENSPAIS (IDCENSPAIS, IDVOTACIO, IDPAIS)
   VALUES (p_idcenspais, p_idvotacio, p_idpais);
	
   RSP := 'OK';

   INSERT INTO LOG (NOMPROCEDIMENT, DATA, PARAMETREENTRADA, PARAMETRESORTIDA)
   VALUES ('PKG_CENSOS.PR_ALTA_CENSPAIS', SYSDATE, p_entrada, 'RSP: '||RSP);

   COMMIT;

   --Excepcions
EXCEPTION

 --El censpais existeix a la Base de Dades
   WHEN E_CENSPAIS_EXISTEIX THEN RSP:= 'ERROR: : El cens-pas ja existeix a la Base de Dades'; 

   INSERT INTO LOG (NOMPROCEDIMENT, DATA, PARAMETREENTRADA, PARAMETRESORTIDA)
   VALUES ('PKG_CENSOS.PR_ALTA_CENSPAIS', SYSDATE, p_entrada, 'RSP: '||RSP);

   COMMIT;

 --Els camps obligatoris sn nuls
   WHEN E_CAMPS_NUL THEN RSP:= 'ERROR: : Els camps obligatoris no poden ser nul'; 

   INSERT INTO LOG (NOMPROCEDIMENT, DATA, PARAMETREENTRADA, PARAMETRESORTIDA)
   VALUES ('PKG_CENSOS.PR_ALTA_CENSPAIS', SYSDATE, p_entrada, 'RSP: '||RSP);

   COMMIT;

 --La votaci no existeix a la Base de Dades
   WHEN E_VOTACIO_NOEXISTEIX THEN RSP:= 'ERROR: : La votaci no existeix a la Base de Dades'; 

   INSERT INTO LOG (NOMPROCEDIMENT, DATA, PARAMETREENTRADA, PARAMETRESORTIDA)
   VALUES ('PKG_CENSOS.PR_ALTA_CENSPAIS', SYSDATE, p_entrada, 'RSP: '||RSP);

   COMMIT;

 --El pas no existeix a la Base de Dades
   WHEN E_PAIS_NOEXISTEIX THEN RSP:= 'ERROR: : El pas no existeix a la Base de Dades'; 

   INSERT INTO LOG (NOMPROCEDIMENT, DATA, PARAMETREENTRADA, PARAMETRESORTIDA)
   VALUES ('PKG_CENSOS.PR_ALTA_CENSPAIS', SYSDATE, p_entrada, 'RSP: '||RSP);

   COMMIT;

   WHEN OTHERS THEN
   error_num := SQLCODE;
   error_missatge := SUBSTR(SQLERRM, 1, 200);

   RSP:= 'ERROR '||TO_CHAR(error_num)||': '||error_missatge;

   INSERT INTO LOG (NOMPROCEDIMENT, DATA, PARAMETREENTRADA, PARAMETRESORTIDA)
   VALUES ('PKG_CENSOS.PR_ALTA_CENSPAIS', SYSDATE, p_entrada, 'RSP: '||RSP);

   COMMIT;

END PR_ALTA_CENSPAIS;


/*******************************************************************************************************

NOM PROCEDIMENT: P_BAIXA_CENSPAIS

FUNCIONALITAT:

Aquest procediment permet donar de baixa un cens pas a la Base de dades, si prviament supera els controls de:
	- Tots els camps obligatoris continguin valor.
	- Existeix el cens pas a la Base de Dades.

PARAMETRES:
ENTRADA
Obligatoris:
	p_idcenspais	Identificador del cens pas

SORTIDA
Obligatoris:
	RSP	

PRECONDICI: Existeix el cens pas a donar de baixa.
POSTCONDICI: Es dona de baixa el cens pas.

RETORNA:
   SI TOT CORRECTE
	OK si loperaci sha realitzat amb xit
   SI ES PRODUEIX UNA EXCEPCI
	ERROR: El cens-pas no existeix a la Base de Dades.
	ERROR: El camp IDCENSPAIS no pot ser nul.
	ERROR: Tipus derror. En qualsevol altre cas.

*******************************************************************************************************/

PROCEDURE PR_BAIXA_CENSPAIS  (
	p_idcenspais   	IN VARCHAR2,
	RSP		OUT VARCHAR2
)

IS

E_CENSPAIS_NOEXISTEIX	EXCEPTION;
E_CAMPS_NUL		EXCEPTION;

existeix	VARCHAR2(1);
p_entrada	VARCHAR2(1000);
error_num	NUMBER;
error_missatge	VARCHAR2(100);

BEGIN

   --S'emmagatzema els parmetres d'entrada a la variable p_entrada

   p_entrada := 'P_IDCENSPAIS: '||p_idcenspais;

   --Es verifica que tots els camps obligatoris contingin valor

   IF (TRIM(p_idcenspais) IS NULL)
   THEN
      RAISE E_CAMPS_NUL;
   END IF;

   --Es verifica que el registre no s'hagi insertat anteriorment amb altre identificador.

   BEGIN
   
   SELECT 'Y' INTO existeix 
   FROM CENSPAIS 
   WHERE idcenspais = p_idcenspais AND ROWNUM = 1;

   EXCEPTION
   WHEN NO_DATA_FOUND THEN
     RAISE E_CENSPAIS_NOEXISTEIX;
   END;

   --Si es pasa totes les proves de validaci, es realitzar la baixa del registre a la Base de Dades.

   DELETE FROM CENSPAIS
   WHERE idcenspais = idcenspais;
	
   RSP := 'OK';

   INSERT INTO LOG (NOMPROCEDIMENT, DATA, PARAMETREENTRADA, PARAMETRESORTIDA)
   VALUES ('PKG_CENSOS.PR_BAIXA_CENSPAIS', SYSDATE, p_entrada, 'RSP: '||RSP);

   COMMIT;

   --Excepcions
EXCEPTION

 --El cens-pas no existeix a la Base de Dades
   WHEN E_CENSPAIS_NOEXISTEIX THEN RSP:= 'ERROR: : El cens-pas no existeix a la Base de Dades'; 

   INSERT INTO LOG (NOMPROCEDIMENT, DATA, PARAMETREENTRADA, PARAMETRESORTIDA)
   VALUES ('PKG_CENSOS.PR_BAIXA_CENSPAIS', SYSDATE, p_entrada, 'RSP: '||RSP);

   COMMIT;

 --Els camps obligatoris sn nuls
   WHEN E_CAMPS_NUL THEN RSP:= 'ERROR: : El camp IDCENSPAIS no pot ser nul'; 

   INSERT INTO LOG (NOMPROCEDIMENT, DATA, PARAMETREENTRADA, PARAMETRESORTIDA)
   VALUES ('PKG_CENSOS.PR_BAIXA_CENSPAIS', SYSDATE, p_entrada, 'RSP: '||RSP);

   COMMIT;

   WHEN OTHERS THEN
   error_num := SQLCODE;
   error_missatge := SUBSTR(SQLERRM, 1, 200);

   RSP:= 'ERROR '||TO_CHAR(error_num)||': '||error_missatge;

   INSERT INTO LOG (NOMPROCEDIMENT, DATA, PARAMETREENTRADA, PARAMETRESORTIDA)
   VALUES ('PKG_CENSOS.PR_BAIXA_CENSPAIS', SYSDATE, p_entrada, 'RSP: '||RSP);

   COMMIT;

END PR_BAIXA_CENSPAIS;


/*******************************************************************************************************

NOM PROCEDIMENT: P_MODIFICACIO_CENSPAIS

FUNCIONALITAT:

Aquest procediment permet modificar un cens pas a la Base de dades, si prviament supera els controls de:
	- El cens pas existeix a la Base de Dades.
	- Tots els camps obligatoris continguin valor.
	- Si es vol modificar el pas, sha de verificar que existeix el pas a la Base de Dades.
	- Si es vol modificar la votaci, sha de verificar que existeix la votaci a la Base de Dades.

PARAMETRES;
ENTRADA
Obligatoris:
	p_idcenspais	Identificador del cens pas
Opcionals:
	p_idvotacio	Identificador de la votaci
	p_idpais	Identificador del pas

SORTIDA
Obligatoris:
	RSP	

PRECONDICI: Existeix la el cens pas a modificar.
POSTCONDICI: Es modifica el cens pas.

RETORNA:
   SI TOT CORRECTE
	OK si loperaci sha realitzat amb xit
   SI ES PRODUEIX UNA EXCEPCI
	ERROR: El pas no existeix a la Base de Dades.
	ERROR: La votaci no existeix a la Base de Dades.
	ERROR: Els camps obligatoris no poden ser nul.
	ERROR: El cens-pas no existeix a la Base de Dades.
	ERROR: Tipus derror. En qualsevol altre cas.

*******************************************************************************************************/

PROCEDURE PR_MODIFICACIO_CENSPAIS (
	p_idcenspais	      IN VARCHAR2,
	p_idvotacio	      IN NUMBER,	
	p_idpais 	      IN VARCHAR2,
	RSP		      OUT VARCHAR2
)

IS

E_CAMPS_NUL		EXCEPTION;
E_CENSPAIS_NOEXISTEIX	EXCEPTION;
E_VOTACIO_NOEXISTEIX	EXCEPTION;
E_PAIS_NOEXISTEIX	EXCEPTION;

existeix	VARCHAR2(1);
p_entrada	VARCHAR2(1000);
error_num	NUMBER;
error_missatge	VARCHAR2(100);

BEGIN

   --S'emmagatzema els parmetres d'entrada a la variable p_entrada
 
   p_entrada := 'P_IDCENSPAIS: '||p_idcenspais||', P_IDVOTACIO: ' ||p_idvotacio||
		', P_PAIS: '||p_idpais;

   --Es verifica que tots els camps obligatoris contingin valor

   IF (TRIM(p_idcenspais) IS NULL) THEN
      RAISE E_CAMPS_NUL;
   END IF;

   --Es verifica que el cens pas existeix a la Base de Dades
   BEGIN

   SELECT 'Y' INTO existeix 
   FROM CENSPAIS
   WHERE idcenspais = p_idcenspais AND ROWNUM = 1;

   EXCEPTION
   WHEN NO_DATA_FOUND THEN
      RAISE E_CENSPAIS_NOEXISTEIX;
   END;

   --Si es modifica la votaci, es verifica que la votaci existeix a la Base de Dades
   BEGIN

   SELECT 'Y' INTO existeix 
   FROM VOTACIO 
   WHERE idvotacio = p_idvotacio AND ROWNUM = 1;

   EXCEPTION
   WHEN NO_DATA_FOUND THEN
      RAISE E_VOTACIO_NOEXISTEIX;
   END;

   --Si es modifica el pas, es verifica que el pas existeix a la Base de Dades
   BEGIN

   SELECT 'Y' INTO existeix 
   FROM PAIS
   WHERE codpais = p_idpais AND ROWNUM = 1;

   EXCEPTION
   WHEN NO_DATA_FOUND THEN
      RAISE E_PAIS_NOEXISTEIX;
   END;

   --Si es pasa totes les proves de validaci, es realitzar l'actualitzaci a la Base de Dades.

   UPDATE CENSPAIS
        SET  idvotacio		= nvl(p_idvotacio, idvotacio),
             idpais	    	= nvl(p_idpais, idpais)
   WHERE   idcenspais      	= p_idcenspais;

   RSP := 'OK';

   INSERT INTO LOG (NOMPROCEDIMENT, DATA, PARAMETREENTRADA, PARAMETRESORTIDA)
   VALUES ('PKG_CENSOS.PR_MODIFICACIO_CENSPAIS', SYSDATE, p_entrada, 'RSP: '||RSP);

   COMMIT;

   --Excepcions
EXCEPTION

 --Els camps obligatoris sn nuls
   WHEN E_CAMPS_NUL THEN RSP:= 'ERROR: : Els camps obligatoris no poden ser nul'; 

   INSERT INTO LOG (NOMPROCEDIMENT, DATA, PARAMETREENTRADA, PARAMETRESORTIDA)
   VALUES ('PKG_CENSOS.PR_MODIFICACIO_CENSPAIS', SYSDATE, p_entrada, 'RSP: '||RSP);

   COMMIT;

 --El cens-pas no existeix a la Base de Dades
   WHEN E_CENSPAIS_NOEXISTEIX THEN RSP:= 'ERROR: : El cens-pas no existeix a la Base de Dades'; 

   INSERT INTO LOG (NOMPROCEDIMENT, DATA, PARAMETREENTRADA, PARAMETRESORTIDA)
   VALUES ('PKG_CENSOS.PR_MODIFICACIO_CENSPAIS', SYSDATE, p_entrada, 'RSP: '||RSP);

   COMMIT; 

 --La votaci no existeix a la Base de Dades
   WHEN E_VOTACIO_NOEXISTEIX THEN RSP:= 'ERROR: : La votaci no existeix a la Base de Dades'; 

   INSERT INTO LOG (NOMPROCEDIMENT, DATA, PARAMETREENTRADA, PARAMETRESORTIDA)
   VALUES ('PKG_CENSOS.PR_MODIFICACIO_CENSPAIS', SYSDATE, p_entrada, 'RSP: '||RSP);

   COMMIT; 

 --El pas no existeix a la Base de Dades
   WHEN E_PAIS_NOEXISTEIX THEN RSP:= 'ERROR: : El pas no existeix a la Base de Dades'; 

   INSERT INTO LOG (NOMPROCEDIMENT, DATA, PARAMETREENTRADA, PARAMETRESORTIDA)
   VALUES ('PKG_CENSOS.PR_MODIFICACIO_CENSPAIS', SYSDATE, p_entrada, 'RSP: '||RSP);

   COMMIT; 

   WHEN OTHERS THEN
   error_num := SQLCODE;
   error_missatge := SUBSTR(SQLERRM, 1, 200);

   RSP:= 'ERROR '||TO_CHAR(error_num)||': '||error_missatge;

   INSERT INTO LOG (NOMPROCEDIMENT, DATA, PARAMETREENTRADA, PARAMETRESORTIDA)
   VALUES ('PKG_CENSOS.PR_MODIFICACIO_CENSPAIS', SYSDATE, p_entrada, 'RSP: '||RSP);

   COMMIT;

END PR_MODIFICACIO_CENSPAIS;


/********************************************************
*							*
*		MANTENIMENT CENSREGIO			*
*							*
********************************************************/

/*******************************************************************************************************

NOM PROCEDIMENT: P_ALTA_CENSREGIO

FUNCIONALITAT:
Aquest procediment permet donar dalta el cens regi, si prviament supera els controls de:
	- Tots els camps obligatoris continguin valor.
	- El registre no shagi inserit amb un altre identificador.
	- La regi est donada dalta a la Base de Dades.

PARAMETRES:
ENTRADA
Obligatoris:
	p_idregio	Identificador de la regi

SORTIDA
Obligatoris:
	RSP	

PRECONDICI: No existeix el cens regi crear.
POSTCONDICI: Es dona dalta el cens regi.

RETORNA:
   SI TOT CORRECTE
	OK si loperaci sha realitzat amb xit
   SI ES PRODUEIX UNA EXCEPCI
	ERROR: El cens-regi ja existeix a la Base de Dades.
	ERROR: Els camps obligatoris no poden ser nuls.
	ERROR: Tipus derror. En qualsevol altre cas.

*******************************************************************************************************/

PROCEDURE PR_ALTA_CENSREGIO  (
        p_idregio   	IN NUMBER,
	RSP		OUT VARCHAR2
)

IS

E_CENSREGIO_EXISTEIX	EXCEPTION;
E_CAMPS_NUL		EXCEPTION;

existeix	VARCHAR2(1);
p_entrada	VARCHAR2(1000);
error_num	NUMBER;
error_missatge	VARCHAR2(100);

BEGIN

   --S'emmagatzema els parmetres d'entrada a la variable p_entrada

   p_entrada := 'P_IDREGIO: ' ||p_idregio; 

   --Es verifica que tots els camps obligatoris contingin valor

   IF (TRIM(p_idregio) IS NULL)
   THEN
      RAISE E_CAMPS_NUL;
   END IF;

   --Es verifica que el registre no s'hagi insertat anteriorment amb altre identificador.

   BEGIN
   
   SELECT 'Y' INTO existeix 
   FROM CENSREGIO
   WHERE idregio = p_idregio AND ROWNUM = 1;

   EXCEPTION
   WHEN NO_DATA_FOUND THEN
        RAISE E_CENSREGIO_EXISTEIX;
   END;

   --Si es pasa totes les proves de validaci, es realitzar l'inserci del nou registre a la Base de Dades.

   INSERT INTO CENSREGIO (IDREGIO)
   VALUES (p_idregio);
	
   RSP := 'OK';

   INSERT INTO LOG (NOMPROCEDIMENT, DATA, PARAMETREENTRADA, PARAMETRESORTIDA)
   VALUES ('PKG_CENSOS.PR_ALTA_CENSREGIO', SYSDATE, p_entrada, 'RSP: '||RSP);

   COMMIT;

   --Excepcions
EXCEPTION

 --El cens-regi existeix a la Base de Dades
   WHEN E_CENSREGIO_EXISTEIX THEN RSP:= 'ERROR: : El cens-regi ja existeix a la Base de Dades'; 

   INSERT INTO LOG (NOMPROCEDIMENT, DATA, PARAMETREENTRADA, PARAMETRESORTIDA)
   VALUES ('PKG_CENSOS.PR_ALTA_CENSREGIO', SYSDATE, p_entrada, 'RSP: '||RSP);

   COMMIT;

 --Els camps obligatoris sn nuls
   WHEN E_CAMPS_NUL THEN RSP:= 'ERROR: : Els camps obligatoris no poden ser nul'; 

   INSERT INTO LOG (NOMPROCEDIMENT, DATA, PARAMETREENTRADA, PARAMETRESORTIDA)
   VALUES ('PKG_CENSOS.PR_ALTA_CENSREGIO', SYSDATE, p_entrada, 'RSP: '||RSP);

   COMMIT;

   WHEN OTHERS THEN
   error_num := SQLCODE;
   error_missatge := SUBSTR(SQLERRM, 1, 200);

   RSP:= 'ERROR '||TO_CHAR(error_num)||': '||error_missatge;

   INSERT INTO LOG (NOMPROCEDIMENT, DATA, PARAMETREENTRADA, PARAMETRESORTIDA)
   VALUES ('PKG_CENSOS.PR_ALTA_CENSREGIO', SYSDATE, p_entrada, 'RSP: '||RSP);

   COMMIT;

END PR_ALTA_CENSREGIO;


/*******************************************************************************************************

NOM PROCEDIMENT: P_BAIXA_CENSREGIO

FUNCIONALITAT: 

Aquest procediment permet donar de baixa un cens regi a la Base de dades, si prviament supera els controls de:
	- Tots els camps obligatoris continguin valor.
	- Existeix el cens regi a la Base de Dades.

PARAMETRES:
ENTRADA
Obligatoris:
	p_idcensregio	Identificador del cens regi

SORTIDA
Obligatoris:
	RSP	

PRECONDICI: Existeix el cens regi a donar de baixa.
POSTCONDICI: Es dona de baixa el cens regi.

RETORNA:
   SI TOT CORRECTE
	OK si loperaci sha realitzat amb xit
   SI ES PRODUEIX UNA EXCEPCI
	ERROR: El cens regi no existeix a la Base de Dades.
	ERROR: El camp IDCENSREGIO no pot ser nul.
	ERROR: Tipus derror. En qualsevol altre cas.

*******************************************************************************************************/

PROCEDURE PR_BAIXA_CENSREGIO  (
	p_idcensregio  	IN NUMBER,
	RSP		OUT VARCHAR2
)

IS

E_CENSREGIO_NOEXISTEIX	EXCEPTION;
E_CAMPS_NUL		EXCEPTION;

existeix	VARCHAR2(1);
p_entrada	VARCHAR2(1000);
error_num	NUMBER;
error_missatge	VARCHAR2(100);

BEGIN

   --S'emmagatzema els parmetres d'entrada a la variable p_entrada

   p_entrada := 'P_IDCENSREGIO: '||p_idcensregio;

   --Es verifica que tots els camps obligatoris contingin valor

   IF (TRIM(p_idcensregio) IS NULL)
   THEN
      RAISE E_CAMPS_NUL;
   END IF;

   --Es verifica que existeix el registre a la Base de Dades.

   BEGIN
   
   SELECT 'Y' INTO existeix 
   FROM CENSREGIO
   WHERE idcensregio = p_idcensregio AND ROWNUM = 1;

   EXCEPTION
   WHEN NO_DATA_FOUND THEN
     RAISE E_CENSREGIO_NOEXISTEIX;
   END;

   --Si es pasa totes les proves de validaci, es realitzar la baixa del registre a la Base de Dades.

   DELETE FROM CENSREGIO
   WHERE idcensregio = p_idcensregio;
	
   RSP := 'OK';

   INSERT INTO LOG (NOMPROCEDIMENT, DATA, PARAMETREENTRADA, PARAMETRESORTIDA)
   VALUES ('PKG_CENSOS.PR_BAIXA_CENSREGIO', SYSDATE, p_entrada, 'RSP: '||RSP);

   COMMIT;

   --Excepcions
EXCEPTION

 --El cens-regi no existeix a la Base de Dades
   WHEN E_CENSREGIO_NOEXISTEIX THEN RSP:= 'ERROR: : El cens-regi no existeix a la Base de Dades'; 

   INSERT INTO LOG (NOMPROCEDIMENT, DATA, PARAMETREENTRADA, PARAMETRESORTIDA)
   VALUES ('PKG_CENSOS.PR_BAIXA_CENSREGIO', SYSDATE, p_entrada, 'RSP: '||RSP);

   COMMIT;

 --Els camps obligatoris sn nuls
   WHEN E_CAMPS_NUL THEN RSP:= 'ERROR: : El camp IDCENSREGIO no pot ser nul'; 

   INSERT INTO LOG (NOMPROCEDIMENT, DATA, PARAMETREENTRADA, PARAMETRESORTIDA)
   VALUES ('PKG_CENSOS.PR_BAIXA_CENSREGIO', SYSDATE, p_entrada, 'RSP: '||RSP);

   COMMIT;

   WHEN OTHERS THEN
   error_num := SQLCODE;
   error_missatge := SUBSTR(SQLERRM, 1, 200);

   RSP:= 'ERROR '||TO_CHAR(error_num)||': '||error_missatge;

   INSERT INTO LOG (NOMPROCEDIMENT, DATA, PARAMETREENTRADA, PARAMETRESORTIDA)
   VALUES ('PKG_CENSOS.PR_BAIXA_CENSREGIO', SYSDATE, p_entrada, 'RSP: '||RSP);

   COMMIT;

END PR_BAIXA_CENSREGIO;


/*******************************************************************************************************

NOM PROCEDIMENT: P_MODIFICACIO_CENSREGIO

FUNCIONALITAT:

Aquest procediment permet modificar un cens regi a la Base de dades, si prviament supera els controls de:
	- El cens regi existeix a la Base de Dades.
	- Tots els camps obligatoris continguin valor.
	- Si es vol modificar el regi, sha de verificar que existeix el regi a la Base de Dades.

PARAMETRES:
ENTRADA
Obligatoris:
	p_idcensregio	Identificador del cens regi
Opcionals:
	p_idregio	Identificador de la regi

SORTIDA
Obligatoris:
	RSP	

PRECONDICI: Existeix la el cens regi a modificar.
POSTCONDICI: Es modifica el cens regi.

RETORNA:
   SI TOT CORRECTE
	OK si loperaci sha realitzat amb xit
   SI ES PRODUEIX UNA EXCEPCI
	ERROR: La regi no existeix a la Base de Dades.
	ERROR: El cens-regi no existeix a la Base de Dades.
	ERROR: Tipus derror. En qualsevol altre cas.

*******************************************************************************************************/

PROCEDURE PR_MODIFICACIO_CENSREGIO (
	p_idcensregio	IN NUMBER,
	p_idregiO	IN NUMBER,	
	RSP		OUT VARCHAR2
)

IS

E_CAMPS_NUL		EXCEPTION;
E_CENSREGIO_NOEXISTEIX	EXCEPTION;
E_REGIO_NOEXISTEIX	EXCEPTION;

existeix	VARCHAR2(1);
p_entrada	VARCHAR2(1000);
error_num	NUMBER;
error_missatge	VARCHAR2(100);

BEGIN

   --S'emmagatzema els parmetres d'entrada a la variable p_entrada
 
   p_entrada := 'P_IDRCENSEGIO: '||p_idcensregio||', P_IDREGIO: ' ||p_idregio;

   --Es verifica que tots els camps obligatoris contingin valor

   IF (TRIM(p_idcensregio) IS NULL) THEN
      RAISE E_CAMPS_NUL;
   END IF;

   --Es verifica que el cens-regi existeix a la Base de Dades
   BEGIN

   SELECT 'Y' INTO existeix 
   FROM CENSREGIO
   WHERE idcensregio = p_idcensregio AND ROWNUM = 1;

   EXCEPTION
   WHEN NO_DATA_FOUND THEN
      RAISE E_CENSREGIO_NOEXISTEIX;
   END;

   --Si es vol modificar la regi, es verifica que la regi existeix a la Base de Dades
   BEGIN

   SELECT 'Y' INTO existeix 
   FROM REGIO
   WHERE idregio = p_idregio AND ROWNUM = 1;

   EXCEPTION
   WHEN NO_DATA_FOUND THEN
      RAISE E_REGIO_NOEXISTEIX;
   END;

   --Si es pasa totes les proves de validaci, es realitzar l'actualitzaci a la Base de Dades.

   UPDATE CENSREGIO
        SET  idregio		= nvl(p_idregio, idregio)
   WHERE     idcensregio     	= p_idcensregio;

   RSP := 'OK';

   INSERT INTO LOG (NOMPROCEDIMENT, DATA, PARAMETREENTRADA, PARAMETRESORTIDA)
   VALUES ('PKG_CENSOS.PR_MODIFICACIO_CENSREGIO', SYSDATE, p_entrada, 'RSP: '||RSP);

   COMMIT;

   --Excepcions
EXCEPTION

 --Els camps obligatoris sn nuls
   WHEN E_CAMPS_NUL THEN RSP:= 'ERROR: : El cens-regi no existeix a la Base de Dades'; 

   INSERT INTO LOG (NOMPROCEDIMENT, DATA, PARAMETREENTRADA, PARAMETRESORTIDA)
   VALUES ('PKG_CENSOS.PR_MODIFICACIO_CENSREGIO', SYSDATE, p_entrada, 'RSP: '||RSP);

   COMMIT;

 --La regi no existeix a la Base de Dades
   WHEN E_REGIO_NOEXISTEIX THEN RSP:= 'ERROR: : La regi no existeix a la Base de Dades'; 

   INSERT INTO LOG (NOMPROCEDIMENT, DATA, PARAMETREENTRADA, PARAMETRESORTIDA)
   VALUES ('PKG_CENSOS.PR_MODIFICACIO_CENSREGIO', SYSDATE, p_entrada, 'RSP: '||RSP);

   COMMIT; 

   WHEN OTHERS THEN
   error_num := SQLCODE;
   error_missatge := SUBSTR(SQLERRM, 1, 200);

   RSP:= 'ERROR '||TO_CHAR(error_num)||': '||error_missatge;

   INSERT INTO LOG (NOMPROCEDIMENT, DATA, PARAMETREENTRADA, PARAMETRESORTIDA)
   VALUES ('PKG_CENSOS.PR_MODIFICACIO_CENSREGIO', SYSDATE, p_entrada, 'RSP: '||RSP);

   COMMIT;

END PR_MODIFICACIO_CENSREGIO;


/********************************************************
*							*
*		MANTENIMENT CENSPAISREGIO		*
*							*
********************************************************/

/*******************************************************************************************************

NOM PROCEDIMENT: P_ALTA_CENSPAISREGIO

FUNCIONALITAT:

Aquest procediment assigna un cens regi a un cens pas, si prviament supera els controls de:
	- Tots els camps obligatoris continguin valor.
	- No existeix el registre a la Base de Dades.
	- Existeix el pas a la Base de Dades.
	- Existeix la regi a la Base de Dades.
	- Existeix el cens pas a la Base de Dades.
	- Existeix el cens regi a la Base de Dades.

PARAMETRES:
ENTRADA
Obligatoris:
	p_idcenspais	Identificador del cens pas
	p_idcensidregio	Identificador del cens regi
	p_idpais	Identificador del pas
	p_idregio	Identificador de la regi

SORTIDA
Obligatoris:
	RSP	

PRECONDICI: No sha donat dalta el cens pas-regi.
POSTCONDICI: Es dona dalta lassignaci del cens regi al cens pas amb el pas i la regi.

RETORNA:
   SI TOT CORRECTE
	OK si loperaci sha realitzat amb xit
   SI ES PRODUEIX UNA EXCEPCI
	ERROR: El censpaisregi ja existeix a la Base de Dades.
	ERROR: El cens-pas no existeix a la Base de Dades.
	ERROR: El cens-regi no existeix a la Base de Dades.
	ERROR: El pas no existeix a la Base de Dades.	
	ERROR: La regi no existeix a la Base de Dades.
	ERROR: Els camps obligatoris no poden ser nul. 
	ERROR: Tipus derror. En qualsevol altre cas.

*******************************************************************************************************/

PROCEDURE PR_ALTA_CENSPAISREGIO  (
        p_idcenspais   	IN VARCHAR2,
        p_idcensregio  	IN NUMBER,
        p_idpais   	IN VARCHAR2,
        p_idregio   	IN NUMBER,
	RSP		OUT VARCHAR2
)

IS

E_CAMPS_NUL			EXCEPTION;
E_CENSPAISREGIO_EXISTEIX	EXCEPTION;
E_CENSPAIS_NOEXISTEIX		EXCEPTION;
E_CENSREGIO_NOEXISTEIX		EXCEPTION;
E_PAIS_NOEXISTEIX		EXCEPTION;
E_REGIO_NOEXISTEIX		EXCEPTION;

existeix	VARCHAR2(1);
p_entrada	VARCHAR2(1000);
error_num	NUMBER;
error_missatge	VARCHAR2(100);

BEGIN

   --S'emmagatzema els parmetres d'entrada a la variable p_entrada

   p_entrada := 'P_IDCENSPAIS: ' ||p_idcenspais||', P_IDCENSREGIO: '||p_idcensregio||
		', P_IDPAIS :'||p_idpais||', P_IDREGIO: '||p_idregio; 

   --Es verifica que tots els camps obligatoris contingin valor

   IF (TRIM(p_idcenspais) IS NULL OR
	TRIM(p_idcensregio) IS NULL OR
	TRIM(p_idpais) IS NULL OR
	TRIM(p_idregio) IS NULL)
   THEN
      RAISE E_CAMPS_NUL;
   END IF;

   --Es verifica que el registre no s'hagi insertat anteriorment amb altre identificador.

   BEGIN
   
   SELECT 'Y' INTO existeix 
   FROM CENSPAISREGIO
   WHERE idcenspais = p_idcenspais AND idcensregio = p_idcensregio AND 
	idpais = p_idpais AND idregio = p_idregio AND ROWNUM = 1;

   RAISE E_CENSPAISREGIO_EXISTEIX;

   EXCEPTION
   WHEN NO_DATA_FOUND THEN
     NULL;
   END;

   --Es verifica que existeix el cens-pas a la Base de Dades.

   BEGIN
   
   SELECT 'Y' INTO existeix 
   FROM CENSPAIS
   WHERE idcenspais = p_idcenspais AND ROWNUM = 1;

   EXCEPTION
   WHEN NO_DATA_FOUND THEN
     RAISE E_CENSPAIS_NOEXISTEIX;
   END;

   --Es verifica que existeix el cens-regi a la Base de Dades.

   BEGIN
   
   SELECT 'Y' INTO existeix 
   FROM CENSREGIO
   WHERE idcensregio = p_idcensregio AND ROWNUM = 1;

   EXCEPTION
   WHEN NO_DATA_FOUND THEN
     RAISE E_CENSREGIO_NOEXISTEIX;
   END;

   --Es verifica que existeix el pas a la Base de Dades.

   BEGIN
   
   SELECT 'Y' INTO existeix 
   FROM PAIS
   WHERE codpais = p_idpais AND ROWNUM = 1;

   EXCEPTION
   WHEN NO_DATA_FOUND THEN
     RAISE E_PAIS_NOEXISTEIX;
   END;

   --Es verifica que existeix la regi a la Base de Dades.

   BEGIN
   
   SELECT 'Y' INTO existeix 
   FROM REGIO
   WHERE idregio = p_idregio AND ROWNUM = 1;

   EXCEPTION
   WHEN NO_DATA_FOUND THEN
     RAISE E_REGIO_NOEXISTEIX;
   END;

   --Si es pasa totes les proves de validaci, es realitzar l'inserci del nou registre a la Base de Dades.

   INSERT INTO CENSPAISREGIO (IDCENSPAIS, IDCENSREGIO, IDPAIS, IDREGIO)
   VALUES (p_idcenspais, p_idcensregio, p_idpais, p_idregio);
	
   RSP := 'OK';

   INSERT INTO LOG (NOMPROCEDIMENT, DATA, PARAMETREENTRADA, PARAMETRESORTIDA)
   VALUES ('PKG_CENSOS.PR_ALTA_CENSPAISREGIO', SYSDATE, p_entrada, 'RSP: '||RSP);

   COMMIT;

   --Excepcions
EXCEPTION

 --El censpaisregio existeix a la Base de Dades
   WHEN E_CENSPAISREGIO_EXISTEIX THEN RSP:= 'ERROR: : El censpaisregi ja existeix a la Base de Dades'; 

   INSERT INTO LOG (NOMPROCEDIMENT, DATA, PARAMETREENTRADA, PARAMETRESORTIDA)
   VALUES ('PKG_CENSOS.PR_ALTA_CENSPAISREGIO', SYSDATE, p_entrada, 'RSP: '||RSP);

   COMMIT;

 --El cens-pais existeix a la Base de Dades
   WHEN E_CENSPAIS_NOEXISTEIX THEN RSP:= 'ERROR: : El cens-pas no existeix a la Base de Dades'; 

   INSERT INTO LOG (NOMPROCEDIMENT, DATA, PARAMETREENTRADA, PARAMETRESORTIDA)
   VALUES ('PKG_CENSOS.PR_ALTA_CENSPAISREGIO', SYSDATE, p_entrada, 'RSP: '||RSP);

   COMMIT;

 --El cens-regi existeix a la Base de Dades
   WHEN E_CENSREGIO_NOEXISTEIX THEN RSP:= 'ERROR: : El cens-regi no existeix a la Base de Dades'; 

   INSERT INTO LOG (NOMPROCEDIMENT, DATA, PARAMETREENTRADA, PARAMETRESORTIDA)
   VALUES ('PKG_CENSOS.PR_ALTA_CENSPAISREGIO', SYSDATE, p_entrada, 'RSP: '||RSP);

   COMMIT;

 --El pas existeix a la Base de Dades
   WHEN E_PAIS_NOEXISTEIX THEN RSP:= 'ERROR: : El pas no existeix a la Base de Dades'; 

   INSERT INTO LOG (NOMPROCEDIMENT, DATA, PARAMETREENTRADA, PARAMETRESORTIDA)
   VALUES ('PKG_CENSOS.PR_ALTA_CENSPAISREGIO', SYSDATE, p_entrada, 'RSP: '||RSP);

   COMMIT;

 --La regi existeix a la Base de Dades
   WHEN E_REGIO_NOEXISTEIX THEN RSP:= 'ERROR: : La regi no existeix a la Base de Dades'; 

   INSERT INTO LOG (NOMPROCEDIMENT, DATA, PARAMETREENTRADA, PARAMETRESORTIDA)
   VALUES ('PKG_CENSOS.PR_ALTA_CENSPAISREGIO', SYSDATE, p_entrada, 'RSP: '||RSP);

   COMMIT;

 --Els camps obligatoris sn nuls
   WHEN E_CAMPS_NUL THEN RSP:= 'ERROR: : Els camps obligatoris no poden ser nul'; 

   INSERT INTO LOG (NOMPROCEDIMENT, DATA, PARAMETREENTRADA, PARAMETRESORTIDA)
   VALUES ('PKG_CENSOS.PR_ALTA_CENSPAISREGIO', SYSDATE, p_entrada, 'RSP: '||RSP);

   COMMIT;

   WHEN OTHERS THEN
   error_num := SQLCODE;
   error_missatge := SUBSTR(SQLERRM, 1, 200);

   RSP:= 'ERROR '||TO_CHAR(error_num)||': '||error_missatge;

   INSERT INTO LOG (NOMPROCEDIMENT, DATA, PARAMETREENTRADA, PARAMETRESORTIDA)
   VALUES ('PKG_CENSOS.PR_ALTA_CENSPAISREGIO', SYSDATE, p_entrada, 'RSP: '||RSP);

   COMMIT;

END PR_ALTA_CENSPAISREGIO;


/*******************************************************************************************************

NOM PROCEDIMENT: P_BAIXA_CENSPAISREGIO

FUNCIONALITAT:

Aquest procediment permet donar de baixa lassignaci del cens regi a un cens pas, si prviament supera els controls de:
	- Tots els camps obligatoris continguin valor.
	- El cens pas te assignat un cens regi.

PARAMETRES:
ENTRADA
Obligatoris:
	p_idcenspaisregio	Identificaci de lassignaci dun cens regi a un cens pas.

SORTIDA
Obligatoris:
	RSP	

PRECONDICI: Existeix lassignaci del cens regi al cens pas a donar de baixa.
POSTCONDICI: Es dona de baixa lassignaci del cens regi al cens pas.

RETORNA:
   SI TOT CORRECTE
	OK si loperaci sha realitzat amb xit
   SI ES PRODUEIX UNA EXCEPCI
	ERROR: El cens-pas-regi no existeix a la Base de Dades.
	ERROR: El camp IDCENSPAISREGIO no pot ser nul.
	ERROR: Tipus derror. En qualsevol altre cas.

*******************************************************************************************************/

PROCEDURE PR_BAIXA_CENSPAISREGIO  (
	p_idcenspaisregio  	IN NUMBER,
	RSP			OUT VARCHAR2
)

IS

E_CENSPAISREGIO_NOEXISTEIX	EXCEPTION;
E_CAMPS_NUL			EXCEPTION;

existeix	VARCHAR2(1);
p_entrada	VARCHAR2(1000);
error_num	NUMBER;
error_missatge	VARCHAR2(100);

BEGIN

   --S'emmagatzema els parmetres d'entrada a la variable p_entrada

   p_entrada := 'P_IDCENSPAISREGIO: '||p_idcenspaisregio;

   --Es verifica que tots els camps obligatoris contingin valor

   IF (TRIM(p_idcenspaisregio) IS NULL)
   THEN
      RAISE E_CAMPS_NUL;
   END IF;

   --Es verifica que existeix el registre a la Base de Dades.

   BEGIN
   
   SELECT 'Y' INTO existeix 
   FROM CENSPAISREGIO
   WHERE idcenspaisregio = p_idcenspaisregio AND ROWNUM = 1;

   EXCEPTION
   WHEN NO_DATA_FOUND THEN
     RAISE E_CENSPAISREGIO_NOEXISTEIX;
   END;

   --Si es pasa totes les proves de validaci, es realitzar la baixa del registre a la Base de Dades.

   DELETE FROM CENSPAISREGIO
   WHERE idcenspaisregio = p_idcenspaisregio;
	
   RSP := 'OK';

   INSERT INTO LOG (NOMPROCEDIMENT, DATA, PARAMETREENTRADA, PARAMETRESORTIDA)
   VALUES ('PKG_CENSOS.PR_BAIXA_CENSPAISREGIO', SYSDATE, p_entrada, 'RSP: '||RSP);

   COMMIT;

   --Excepcions
EXCEPTION

 --El cens-pas-regi no existeix a la Base de Dades
   WHEN E_CENSPAISREGIO_NOEXISTEIX THEN RSP:= 'ERROR: : El cens-pas-regi no existeix a la Base de Dades'; 

   INSERT INTO LOG (NOMPROCEDIMENT, DATA, PARAMETREENTRADA, PARAMETRESORTIDA)
   VALUES ('PKG_CENSOS.PR_BAIXA_CENSPAISREGIO', SYSDATE, p_entrada, 'RSP: '||RSP);

   COMMIT;

 --Els camps obligatoris sn nuls
   WHEN E_CAMPS_NUL THEN RSP:= 'ERROR: : El camp IDCENSPAISREGIO no pot ser nul'; 

   INSERT INTO LOG (NOMPROCEDIMENT, DATA, PARAMETREENTRADA, PARAMETRESORTIDA)
   VALUES ('PKG_CENSOS.PR_BAIXA_CENSPAISREGIO', SYSDATE, p_entrada, 'RSP: '||RSP);

   COMMIT;

   WHEN OTHERS THEN
   error_num := SQLCODE;
   error_missatge := SUBSTR(SQLERRM, 1, 200);

   RSP:= 'ERROR '||TO_CHAR(error_num)||': '||error_missatge;

   INSERT INTO LOG (NOMPROCEDIMENT, DATA, PARAMETREENTRADA, PARAMETRESORTIDA)
   VALUES ('PKG_CENSOS.PR_BAIXA_CENSPAISREGIO', SYSDATE, p_entrada, 'RSP: '||RSP);

   COMMIT;

END PR_BAIXA_CENSPAISREGIO;


/*******************************************************************************************************

NOM PROCEDIMENT: P_MODIFICACIO_CENSPAISREGIO

FUNCIONALITAT:

Aquest procediment permet modificar lassignaci duna regi a un pas, si prviament supera els controls de:
	- El registre a modificar existeix a la Base de Dades.
	- Tots els camps obligatoris continguin valor.
	- Existeix cel cens pas a la Base de Dades.
	- Existeix cel cens regi a la Base de Dades.
	- Existeix el pas a la Base de Dades.
	- Existeix la regi a la Base de Dades.

PARAMETRES:
ENTRADA
Obligatoris:
	p_idcenspaisregio	Identificaci de lassignaci duna regi a un pas.
	p_idcenspais		Identificador del cens pas
	p_idcensregio		Identificador del cens regi
	p_idpais		Identificador del pas
	p_idregio		Identificador de la regi

SORTIDA
Obligatoris:
	RSP	

PRECONDICI: Existeix lassignaci a modificar.
POSTCONDICI: Es modifica lassignaci.

RETORNA:
   SI TOT CORRECTE
	OK si loperaci sha realitzat amb xit
   SI ES PRODUEIX UNA EXCEPCI
	ERROR: Els camps obligatoris no poden ser nul.
	'ERROR: : El cens-pas-regi no t assignada la regi a modificar.'
	'ERROR: : El cens-pas no existeix a la Base de Dades.'
	'ERROR: : El cens-regi no existeix a la Base de Dades.'
	'ERROR: : El pas no existeix a la Base de Dades.'
	'ERROR: : La regi no existeix a la Base de Dades.'
	'ERROR: : Els canvis a realitzar ja existeixen a la Base de Dades'
	ERROR: Tipus derror. En qualsevol altre cas.

*******************************************************************************************************/

PROCEDURE PR_MODIFICACIO_CENSPAISREGIO (
	p_idcenspaisregio	IN NUMBER,
	p_idcenspais		IN VARCHAR2,
	p_idcensregio		IN NUMBER,
	p_idpais		IN VARCHAR2,
	p_idregio		IN NUMBER,	
	RSP			OUT VARCHAR2
)

IS

E_CAMPS_NUL			EXCEPTION;
E_IDCENSPAISREGIO_NOEXISTEIX	EXCEPTION;
E_IDCENSPAIS_NOEXISTEIX		EXCEPTION;
E_IDCENSREGIO_NOEXISTEIX	EXCEPTION;
E_IDPAIS_NOEXISTEIX		EXCEPTION;
E_IDREGIO_NOEXISTEIX		EXCEPTION;
E_ASSIGNACIO_EXISTEIX		EXCEPTION;

existeix	VARCHAR2(1);
p_entrada	VARCHAR2(1000);
error_num	NUMBER;
error_missatge	VARCHAR2(100);

BEGIN

   --S'emmagatzema els parmetres d'entrada a la variable p_entrada
 
   p_entrada := 'P_IDCENSPAISREGIO: '||p_idcenspaisregio||', P_IDCENSPAIS: '||p_idcenspais||
		', P_IDCENSREGIO: '||p_idcensregio||', P_IDPAIS: '||p_idpais||
		', P_IDREGIO: '||p_idregio;

   --Es verifica que tots els camps obligatoris contingin valor

   IF (TRIM(p_idcenspaisregio) IS NULL OR
	TRIM(p_idcenspais) IS NULL OR
	TRIM(p_idcensregio) IS NULL OR
	TRIM(p_idpais) IS NULL OR
	TRIM(p_idregio) IS NULL) THEN
      RAISE E_CAMPS_NUL;
   END IF;

   --Verifica la nova assignaci no existeix a la base de dades
   BEGIN

   SELECT 'Y' INTO existeix 
   FROM CENSPAISREGIO
   WHERE idcenspaisregio = p_idcenspaisregio AND idcenspais = p_idcenspais 
	AND idcensregio = p_idcensregio AND idpais = p_idpais AND idregio = p_idregio 
	AND ROWNUM = 1;

   RAISE E_ASSIGNACIO_EXISTEIX;

   EXCEPTION
   WHEN NO_DATA_FOUND THEN
	NULL;      
   END;

   --Es verifica que el cens pais regio existeix a la Base de Dades.

   BEGIN
   
   SELECT 'Y' INTO existeix 
   FROM CENSPAISREGIO
   WHERE idcenspaisregio = p_idcenspaisregio AND ROWNUM = 1;

   EXCEPTION
   WHEN NO_DATA_FOUND THEN
     RAISE E_IDCENSPAISREGIO_NOEXISTEIX;
   END;

   --Es verifica que existeix el cens-pas a la Base de Dades.

   BEGIN
   
   SELECT 'Y' INTO existeix 
   FROM CENSPAIS
   WHERE idcenspais = p_idcenspais AND ROWNUM = 1;

   EXCEPTION
   WHEN NO_DATA_FOUND THEN
     RAISE E_IDCENSPAIS_NOEXISTEIX;
   END;

   --Es verifica que existeix el cens-regi a la Base de Dades.

   BEGIN
   
   SELECT 'Y' INTO existeix 
   FROM CENSREGIO
   WHERE idcensregio = p_idcensregio AND ROWNUM = 1;

   EXCEPTION
   WHEN NO_DATA_FOUND THEN
     RAISE E_IDCENSREGIO_NOEXISTEIX;
   END;

   --Es verifica que existeix el pas a la Base de Dades.

   BEGIN
   
   SELECT 'Y' INTO existeix 
   FROM PAIS
   WHERE codpais = p_idpais AND ROWNUM = 1;

   EXCEPTION
   WHEN NO_DATA_FOUND THEN
     RAISE E_IDPAIS_NOEXISTEIX;
   END;

   --Es verifica que existeix la regi a la Base de Dades.

   BEGIN
   
   SELECT 'Y' INTO existeix 
   FROM REGIO
   WHERE idregio = p_idregio AND ROWNUM = 1;

   EXCEPTION
   WHEN NO_DATA_FOUND THEN
     RAISE E_IDREGIO_NOEXISTEIX;
   END;

   --Si es pasa totes les proves de validaci, es realitzar l'actualitzaci a la Base de Dades.

   UPDATE CENSPAISREGIO
        SET  idpais		      = nvl(p_idpais, idpais),
             idregio	    	= nvl(p_idregio, idregio),
             idcenspais		  = nvl(p_idcenspais, idcenspais),
             idcensregio    = nvl(p_idcensregio, idcensregio)
   WHERE     idcenspaisregio = p_idcenspaisregio;

   RSP := 'OK';

   INSERT INTO LOG (NOMPROCEDIMENT, DATA, PARAMETREENTRADA, PARAMETRESORTIDA)
   VALUES ('PKG_CENSOS.PR_MODIFICACIO_CENSPAISREGIO', SYSDATE, p_entrada, 'RSP: '||RSP);

   COMMIT;

   --Excepcions
EXCEPTION

 --Els camps obligatoris sn nuls
   WHEN E_CAMPS_NUL THEN RSP:= 'ERROR: : Els camps obligatoris no poden ser nul'; 

   INSERT INTO LOG (NOMPROCEDIMENT, DATA, PARAMETREENTRADA, PARAMETRESORTIDA)
   VALUES ('PKG_CENSOS.PR_MODIFICACIO_CENSPAISREGIO', SYSDATE, p_entrada, 'RSP: '||RSP);

   COMMIT;

 --El cens-pas-regi no existeix a la Base de Dades
   WHEN E_IDCENSPAISREGIO_NOEXISTEIX THEN RSP:= 'ERROR: : El cens-pas-regi no t assignada la regi a modificar'; 

   INSERT INTO LOG (NOMPROCEDIMENT, DATA, PARAMETREENTRADA, PARAMETRESORTIDA)
   VALUES ('PKG_CENSOS.PR_MODIFICACIO_CENSPAISREGIO', SYSDATE, p_entrada, 'RSP: '||RSP);

   COMMIT; 

 --El cens pas no existeix a la Base de Dades
   WHEN E_IDCENSPAIS_NOEXISTEIX THEN RSP:= 'ERROR: : El cens-pas no existeix a la Base de Dades'; 

   INSERT INTO LOG (NOMPROCEDIMENT, DATA, PARAMETREENTRADA, PARAMETRESORTIDA)
   VALUES ('PKG_CENSOS.PR_MODIFICACIO_CENSPAISREGIO', SYSDATE, p_entrada, 'RSP: '||RSP);

   COMMIT;

 --El cens regi no existeix a la Base de Dades
   WHEN E_IDCENSREGIO_NOEXISTEIX THEN RSP:= 'ERROR: : El cens-regi no existeix a la Base de Dades'; 

   INSERT INTO LOG (NOMPROCEDIMENT, DATA, PARAMETREENTRADA, PARAMETRESORTIDA)
   VALUES ('PKG_CENSOS.PR_MODIFICACIO_PAISREGIO', SYSDATE, p_entrada, 'RSP: '||RSP);

   COMMIT;

 --El pas no existeix a la Base de Dades
   WHEN E_IDPAIS_NOEXISTEIX THEN RSP:= 'ERROR: : El pas no existeix a la Base de Dades'; 

   INSERT INTO LOG (NOMPROCEDIMENT, DATA, PARAMETREENTRADA, PARAMETRESORTIDA)
   VALUES ('PKG_CENSOS.PR_MODIFICACIO_CENSPAISREGIO', SYSDATE, p_entrada, 'RSP: '||RSP);

   COMMIT;

 --La regi no existeix a la Base de Dades
   WHEN E_IDREGIO_NOEXISTEIX THEN RSP:= 'ERROR: : La regi no existeix a la Base de Dades'; 

   INSERT INTO LOG (NOMPROCEDIMENT, DATA, PARAMETREENTRADA, PARAMETRESORTIDA)
   VALUES ('PKG_CENSOS.PR_MODIFICACIO_PAISREGIO', SYSDATE, p_entrada, 'RSP: '||RSP);

   COMMIT;

 --Ja existeix l'assignaci a la Base de Dades
   WHEN E_ASSIGNACIO_EXISTEIX THEN RSP:= 'ERROR: : Els canvis a realitzar ja existeixen a la Base de Dades'; 

   INSERT INTO LOG (NOMPROCEDIMENT, DATA, PARAMETREENTRADA, PARAMETRESORTIDA)
   VALUES ('PKG_CENSOS.PR_MODIFICACIO_CENSPAISREGIO', SYSDATE, p_entrada, 'RSP: '||RSP);

   COMMIT;

   WHEN OTHERS THEN
   error_num := SQLCODE;
   error_missatge := SUBSTR(SQLERRM, 1, 200);

   RSP:= 'ERROR '||TO_CHAR(error_num)||': '||error_missatge;

   INSERT INTO LOG (NOMPROCEDIMENT, DATA, PARAMETREENTRADA, PARAMETRESORTIDA)
   VALUES ('PKG_CENSOS.PR_MODIFICACIO_CENSPAISREGIO', SYSDATE, p_entrada, 'RSP: '||RSP);

   COMMIT;

END PR_MODIFICACIO_CENSPAISREGIO;


/********************************************************
*							*
*		MANTENIMENT CENSLOCAL			*
*							*
********************************************************/


/*******************************************************************************************************

NOM PROCEDIMENT: P_ALTA_CENSLOCAL

FUNCIONALITAT: 
Aquest procediment permet donar dalta el cens local, si prviament supera els controls de:
	- Tots els camps obligatoris continguin valor.
	- El registre no shagi inserit amb un altre identificador.
	- La localitat est donada dalta a la Base de Dades. 
	- El cens pas est donat dalta a la Base de Dades.

PARAMETRES:
ENTRADA
Obligatoris:
	p_idcensregio	Identificador del cens regi
	p_idlocalitat	Identificador de la localitat

SORTIDA
Obligatoris:
	RSP

PRECONDICI: No existeix el cens localitat crear.
POSTCONDICI: Es dona dalta el cens localitat.

RETORNA:
   SI TOT CORRECTE
	OK si loperaci sha realitzat amb xit
   SI ES PRODUEIX UNA EXCEPCI
	ERROR: Els camps obligatoris no poden ser nuls.
	ERROR: El cens-local ja existeix a la Base de Dades.
	ERROR: El cens-regi no existeix a la Base de Dades.
	ERROR: La localitat no existeix a la Base de Dades.
	ERROR: Tipus derror. En qualsevol altre cas.

*******************************************************************************************************/

PROCEDURE PR_ALTA_CENSLOCAL  (
	p_idcensregio 	   IN NUMBER,
        p_idlocalitat	   IN NUMBER,
	RSP		   OUT VARCHAR2
)

IS

E_CAMPS_NUL		EXCEPTION;
E_CENSLOCAL_EXISTEIX	EXCEPTION;
E_CENSREGIO_NOEXISTEIX	EXCEPTION;
E_LOCALITAT_NOEXISTEIX	EXCEPTION;

existeix	VARCHAR2(1);
p_entrada	VARCHAR2(1000);
error_num	NUMBER;
error_missatge	VARCHAR2(100);

BEGIN

   --S'emmagatzema els parmetres d'entrada a la variable p_entrada

   p_entrada := 'P_IDCENSREGIO: ' ||p_idcensregio||', P_IDLOCALITAT: '||p_idlocalitat;

   --Es verifica que tots els camps obligatoris contingin valor

   IF (TRIM(p_idcensregio) IS NULL OR 
	TRIM(p_idlocalitat) IS NULL)
   THEN
      RAISE E_CAMPS_NUL;
   END IF;

   --Es verifica que el registre no s'hagi insertat anteriorment amb altre identificador.

   BEGIN
   
   SELECT 'Y' INTO existeix 
   FROM CENSLOCAL
   WHERE idcensregio = p_idcensregio AND idlocalitat= p_idlocalitat AND ROWNUM = 1;

   RAISE E_CENSLOCAL_EXISTEIX;

   EXCEPTION
   WHEN NO_DATA_FOUND THEN
     NULL;
   END;

   --Es verifica que el cens-regi que es pasa per parmetre existeix

   BEGIN
   
   SELECT 'Y' INTO existeix 
   FROM CENSREGIO
   WHERE idcensregio = p_idcensregio AND ROWNUM = 1;   

   EXCEPTION
   WHEN NO_DATA_FOUND THEN
     RAISE E_CENSREGIO_NOEXISTEIX;
   END;

   --Es verifica que la localitat que es pasa per parmetre existeix

   BEGIN
   
   SELECT 'Y' INTO existeix 
   FROM LOCALITAT
   WHERE idlocalitat = p_idlocalitat AND ROWNUM = 1;   

   EXCEPTION
   WHEN NO_DATA_FOUND THEN
     RAISE E_LOCALITAT_NOEXISTEIX;
   END;

   --Si es pasa totes les proves de validaci, es realitzar l'inserci del nou registre a la Base de Dades.

   INSERT INTO CENSLOCAL (IDCENSREGIO, IDLOCALITAT)
   VALUES (p_idcensregio, p_idlocalitat);
	
   RSP := 'OK';

   INSERT INTO LOG (NOMPROCEDIMENT, DATA, PARAMETREENTRADA, PARAMETRESORTIDA)
   VALUES ('PKG_CENSOS.PR_ALTA_CENSLOCAL', SYSDATE, p_entrada, 'RSP: '||RSP);

   COMMIT;

   --Excepcions
EXCEPTION


 --Els camps obligatoris sn nuls
   WHEN E_CAMPS_NUL THEN RSP:= 'ERROR: : Els camps obligatoris no poden ser nul'; 

   INSERT INTO LOG (NOMPROCEDIMENT, DATA, PARAMETREENTRADA, PARAMETRESORTIDA)
   VALUES ('PKG_CENSOS.PR_ALTA_CENSLOCAL', SYSDATE, p_entrada, 'RSP: '||RSP);

   COMMIT;

 --El censlocal existeix a la Base de Dades
   WHEN E_CENSLOCAL_EXISTEIX THEN RSP:= 'ERROR: : El cens-local ja existeix a la Base de Dades'; 

   INSERT INTO LOG (NOMPROCEDIMENT, DATA, PARAMETREENTRADA, PARAMETRESORTIDA)
   VALUES ('PKG_CENSOS.PR_ALTA_CENSLOCAL', SYSDATE, p_entrada, 'RSP: '||RSP);

   COMMIT;

 --El cens-regi no existeix a la Base de Dades
   WHEN E_CENSREGIO_NOEXISTEIX THEN RSP:= 'ERROR: : El cens-regi no existeix a la Base de Dades'; 

   INSERT INTO LOG (NOMPROCEDIMENT, DATA, PARAMETREENTRADA, PARAMETRESORTIDA)
   VALUES ('PKG_CENSOS.PR_ALTA_CENSLOCAL', SYSDATE, p_entrada, 'RSP: '||RSP);

   COMMIT;

 --La localitat no existeix a la Base de Dades
   WHEN E_LOCALITAT_NOEXISTEIX THEN RSP:= 'ERROR: : La localitat no existeix a la Base de Dades'; 

   INSERT INTO LOG (NOMPROCEDIMENT, DATA, PARAMETREENTRADA, PARAMETRESORTIDA)
   VALUES ('PKG_CENSOS.PR_ALTA_CENSLOCAL', SYSDATE, p_entrada, 'RSP: '||RSP);

   COMMIT;

   WHEN OTHERS THEN
   error_num := SQLCODE;
   error_missatge := SUBSTR(SQLERRM, 1, 200);

   RSP:= 'ERROR '||TO_CHAR(error_num)||': '||error_missatge;

   INSERT INTO LOG (NOMPROCEDIMENT, DATA, PARAMETREENTRADA, PARAMETRESORTIDA)
   VALUES ('PKG_CENSOS.PR_ALTA_CENSLOCAL', SYSDATE, p_entrada, 'RSP: '||RSP);

   COMMIT;

END PR_ALTA_CENSLOCAL;


/*******************************************************************************************************

NOM PROCEDIMENT: P_BAIXA_CENSLOCAL

FUNCIONALITAT:

Aquest procediment permet donar de baixa un cens localitat a la Base de dades, si prviament supera els controls de:
	- Tots els camps obligatoris continguin valor.
	- Existeix el cens local a la Base de Dades.

PARAMETRES:
ENTRADA
Obligatoris:
	p_idcenslocal	Identificador del cens localitat

SORTIDA
Obligatoris:
	RSP	

PRECONDICI: Existeix el cens localtiat a donar de baixa.
POSTCONDICI: Es dona de baixa el cens localitat.

RETORNA:
   SI TOT CORRECTE
	OK si loperaci sha realitzat amb xit
   SI ES PRODUEIX UNA EXCEPCI
	ERROR: El cens localitat no existeix a la Base de Dades.
	ERROR: El camp IDCENSLOCAL no pot ser nul.
	ERROR: Tipus derror. En qualsevol altre cas.

*******************************************************************************************************/

PROCEDURE PR_BAIXA_CENSLOCAL  (
	p_idcenslocal  	IN NUMBER,
	RSP		OUT VARCHAR2
)

IS

E_CENSLOCAL_NOEXISTEIX	EXCEPTION;
E_CAMPS_NUL		EXCEPTION;

existeix	VARCHAR2(1);
p_entrada	VARCHAR2(1000);
error_num	NUMBER;
error_missatge	VARCHAR2(100);

BEGIN

   --S'emmagatzema els parmetres d'entrada a la variable p_entrada

   p_entrada := 'P_IDCENSLOCAL: '||p_idcenslocal;

   --Es verifica que tots els camps obligatoris contingin valor

   IF (TRIM(p_idcenslocal) IS NULL)
   THEN
      RAISE E_CAMPS_NUL;
   END IF;

   --Es verifica que el registre no s'hagi insertat anteriorment amb altre identificador.

   BEGIN
   
   SELECT 'Y' INTO existeix 
   FROM CENSLOCAL 
   WHERE idcenslocal = p_idcenslocal AND ROWNUM = 1;

   EXCEPTION
   WHEN NO_DATA_FOUND THEN
     RAISE E_CENSLOCAL_NOEXISTEIX;
   END;

   --Si es pasa totes les proves de validaci, es realitzar la baixa del registre a la Base de Dades.

   DELETE FROM CENSLOCAL
   WHERE idcenslocal = p_idcenslocal;
	
   RSP := 'OK';

   INSERT INTO LOG (NOMPROCEDIMENT, DATA, PARAMETREENTRADA, PARAMETRESORTIDA)
   VALUES ('PKG_CENSOS.PR_BAIXA_CENSLOCAL', SYSDATE, p_entrada, 'RSP: '||RSP);

   COMMIT;

   --Excepcions
EXCEPTION

 --El cens-pas no existeix a la Base de Dades
   WHEN E_CENSLOCAL_NOEXISTEIX THEN RSP:= 'ERROR: : El cens localitat no existeix a la Base de Dades'; 

   INSERT INTO LOG (NOMPROCEDIMENT, DATA, PARAMETREENTRADA, PARAMETRESORTIDA)
   VALUES ('PKG_CENSOS.PR_BAIXA_CENSLOCAL', SYSDATE, p_entrada, 'RSP: '||RSP);

   COMMIT;

 --Els camps obligatoris sn nuls
   WHEN E_CAMPS_NUL THEN RSP:= 'ERROR: : El camp IDCENSLOCAL no pot ser nul'; 

   INSERT INTO LOG (NOMPROCEDIMENT, DATA, PARAMETREENTRADA, PARAMETRESORTIDA)
   VALUES ('PKG_CENSOS.PR_BAIXA_CENSLOCAL', SYSDATE, p_entrada, 'RSP: '||RSP);

   COMMIT;

   WHEN OTHERS THEN
   error_num := SQLCODE;
   error_missatge := SUBSTR(SQLERRM, 1, 200);

   RSP:= 'ERROR '||TO_CHAR(error_num)||': '||error_missatge;

   INSERT INTO LOG (NOMPROCEDIMENT, DATA, PARAMETREENTRADA, PARAMETRESORTIDA)
   VALUES ('PKG_CENSOS.PR_BAIXA_CENSLOCAL', SYSDATE, p_entrada, 'RSP: '||RSP);

   COMMIT;

END PR_BAIXA_CENSLOCAL;


/*******************************************************************************************************

NOM PROCEDIMENT: P_MODIFICACIO_CENSLOCAL

FUNCIONALITAT:

Aquest procediment permet modificar un cens localitat a la Base de dades, si prviament supera els controls de:
	_ El cens localitat existeix a la Base de Dades.
	_ Tots els camps obligatoris continguin valor.
	- Si es vol modificar el cens regi, sha de verificar que existeix el cens regi a la Base de Dades.
	- Si es vol modificar la localitat, sha de verificar que existeix la localitat a la Base de Dades.

PARAMETRES:
ENTRADA
Obligatoris:
	p_idcenslocal	Identificador del cens local
Opcionals:
	p_idcensregio	Identificador del cens regi
	p_idlocalitat	Identificador de la localitat

SORTIDA:
Obligatoris:
	RSP	

PRECONDICI: Existeix la el cens localitat a modificar.
POSTCONDICI: Es modifica el cens localitat.

RETORNA:
   SI TOT CORRECTE
	OK si loperaci sha realitzat amb xit
   SI ES PRODUEIX UNA EXCEPCI
	ERROR: El cens-localitat no existeix a la Base de Dades.
	ERROR: El cens-regi no existeix a la Base de Dades.
	'ERROR: La localitat no existeix a la Base de Dades.'
	ERROR: Els camps obligatoris no poden ser nul.
	ERROR: Els canvis ha realitzar ja existeixen a la Base de Dades.
	ERROR: Tipus derror. En qualsevol altre cas.

*******************************************************************************************************/

PROCEDURE PR_MODIFICACIO_CENSLOCAL (
	p_idcenslocal	      IN NUMBER,
	p_idcensregio	      IN NUMBER,	
	p_idlocalitat 	      IN NUMBER,
	RSP		      OUT VARCHAR2
)

IS

E_CAMPS_NUL		EXCEPTION;
E_ASSIGNACIO_EXISTEIX	EXCEPTION;
E_CENSLOCAL_NOEXISTEIX	EXCEPTION;
E_CENSREGIO_NOEXISTEIX	EXCEPTION;
E_LOCALITAT_NOEXISTEIX	EXCEPTION;

existeix	VARCHAR2(1);
p_entrada	VARCHAR2(1000);
error_num	NUMBER;
error_missatge	VARCHAR2(100);

BEGIN

   --S'emmagatzema els parmetres d'entrada a la variable p_entrada
 
   p_entrada := 'P_IDCENSLOCAL: '||p_idcenslocal||', P_IDCENSREGIO: ' ||p_idcensregio||
		', P_IDLOCALITAT: '||p_IDLOCALITAT;

   --Es verifica que tots els camps obligatoris contingin valor

   IF (TRIM(p_idcenslocal) IS NULL) THEN
      RAISE E_CAMPS_NUL;
   END IF;

   --Es verifica que el cens local existeix a la Base de Dades
   BEGIN

   SELECT 'Y' INTO existeix 
   FROM CENSLOCAL
   WHERE idcenslocal = p_idcenslocal AND idcensregio = p_idcensregio 
	AND idlocalitat = p_idlocalitat AND ROWNUM = 1;

   RAISE E_ASSIGNACIO_EXISTEIX;

   EXCEPTION
   WHEN NO_DATA_FOUND THEN
      NULL;
   END;

   --Es verifica que el cens local existeix a la Base de Dades
   BEGIN

   SELECT 'Y' INTO existeix 
   FROM CENSLOCAL
   WHERE idcenslocal = p_idcenslocal AND ROWNUM = 1;

   EXCEPTION
   WHEN NO_DATA_FOUND THEN
      RAISE E_CENSLOCAL_NOEXISTEIX;
   END;
   --Si es modifica el cens regi, es verifica que el cens regi existeix a la Base de Dades
   BEGIN

   SELECT 'Y' INTO existeix 
   FROM CENSREGIO
   WHERE idcensregio = p_idcensregio AND ROWNUM = 1;

   EXCEPTION
   WHEN NO_DATA_FOUND THEN
      RAISE E_CENSREGIO_NOEXISTEIX;
   END;

   --Si es modifica la localitat, es verifica que la localitat existeix a la Base de Dades
   BEGIN

   SELECT 'Y' INTO existeix 
   FROM LOCALITAT
   WHERE idlocalitat = p_idlocalitat AND ROWNUM = 1;

   EXCEPTION
   WHEN NO_DATA_FOUND THEN
      RAISE E_LOCALITAT_NOEXISTEIX;
   END;

   --Si es pasa totes les proves de validaci, es realitzar l'actualitzaci a la Base de Dades.

   UPDATE CENSLOCAL
        SET  idcensregio	= nvl(p_idcensregio, idcensregio),
             idlocalitat    	= nvl(p_idlocalitat, idlocalitat)
   WHERE   idcenslocal      	= p_idcenslocal;

   RSP := 'OK';

   INSERT INTO LOG (NOMPROCEDIMENT, DATA, PARAMETREENTRADA, PARAMETRESORTIDA)
   VALUES ('PKG_CENSOS.PR_MODIFICACIO_CENSLOCAL', SYSDATE, p_entrada, 'RSP: '||RSP);

   COMMIT;

   --Excepcions
EXCEPTION

 --Els camps obligatoris sn nuls
   WHEN E_CAMPS_NUL THEN RSP:= 'ERROR: : Els camps obligatoris no poden ser nul'; 

   INSERT INTO LOG (NOMPROCEDIMENT, DATA, PARAMETREENTRADA, PARAMETRESORTIDA)
   VALUES ('PKG_CENSOS.PR_MODIFICACIO_CENSLOCAL', SYSDATE, p_entrada, 'RSP: '||RSP);

   COMMIT;

 --El cens-local no existeix a la Base de Dades
   WHEN E_CENSLOCAL_NOEXISTEIX THEN RSP:= 'ERROR: : El cens-localitat no existeix a la Base de Dades'; 

   INSERT INTO LOG (NOMPROCEDIMENT, DATA, PARAMETREENTRADA, PARAMETRESORTIDA)
   VALUES ('PKG_CENSOS.PR_MODIFICACIO_CENSLOCAL', SYSDATE, p_entrada, 'RSP: '||RSP);

   COMMIT; 

 --El cens regi no existeix a la Base de Dades
   WHEN E_CENSREGIO_NOEXISTEIX THEN RSP:= 'ERROR: : El cens-regi no existeix a la Base de Dades'; 

   INSERT INTO LOG (NOMPROCEDIMENT, DATA, PARAMETREENTRADA, PARAMETRESORTIDA)
   VALUES ('PKG_CENSOS.PR_MODIFICACIO_CENSLOCAL', SYSDATE, p_entrada, 'RSP: '||RSP);

   COMMIT; 

 --La localitat no existeix a la Base de Dades
   WHEN E_LOCALITAT_NOEXISTEIX THEN RSP:= 'ERROR: : La localitat no existeix a la Base de Dades'; 

   INSERT INTO LOG (NOMPROCEDIMENT, DATA, PARAMETREENTRADA, PARAMETRESORTIDA)
   VALUES ('PKG_CENSOS.PR_MODIFICACIO_CENSLOCAL', SYSDATE, p_entrada, 'RSP: '||RSP);

   COMMIT; 

 --L'assignaci existeix a la Base de Dades
   WHEN E_ASSIGNACIO_EXISTEIX THEN RSP:= 'ERROR: : Els canvis ha realitzar ja existeixen a la Base de Dades'; 

   INSERT INTO LOG (NOMPROCEDIMENT, DATA, PARAMETREENTRADA, PARAMETRESORTIDA)
   VALUES ('PKG_CENSOS.PR_MODIFICACIO_CENSLOCAL', SYSDATE, p_entrada, 'RSP: '||RSP);

   COMMIT; 

   WHEN OTHERS THEN
   error_num := SQLCODE;
   error_missatge := SUBSTR(SQLERRM, 1, 200);

   RSP:= 'ERROR '||TO_CHAR(error_num)||': '||error_missatge;

   INSERT INTO LOG (NOMPROCEDIMENT, DATA, PARAMETREENTRADA, PARAMETRESORTIDA)
   VALUES ('PKG_CENSOS.PR_MODIFICACIO_CENSLOCAL', SYSDATE, p_entrada, 'RSP: '||RSP);

   COMMIT;

END PR_MODIFICACIO_CENSLOCAL;

END PKG_CENSOS;
/