/**************************************************************************

			PACKAGE CIUTADA

**************************************************************************/

CREATE OR REPLACE PACKAGE USUARITFC11.PKG_CIUTADA AS

PROCEDURE PR_ALTA_CIUTADA  (
	p_codi	 	   IN VARCHAR2,
  p_edat		   IN DATE,
	RSP		   OUT VARCHAR2
);


PROCEDURE PR_BAIXA_CIUTADA  (
	p_codi  	IN VARCHAR2,
	RSP		OUT VARCHAR2
);


PROCEDURE PR_MODIFICACIO_CIUTADA (
	p_codi		IN VARCHAR2,
	p_edat		IN DATE,
	RSP		OUT VARCHAR2
);

PROCEDURE PR_ALTA_CIUTADAOPCIO  (
  p_idCiutada         IN VARCHAR2,
  p_idcenslocal	      IN NUMBER,
  p_idopcio	          IN NUMBER,
	RSP		              OUT VARCHAR2
);


PROCEDURE PR_BAIXA_CIUTADAOPCIO  (
	p_idCiutadaOpcio  	IN NUMBER,
	RSP		              OUT VARCHAR2
);


PROCEDURE PR_MODIFICACIO_CIUTADAOPCIO (
	p_idCiutadaOpcio    IN NUMBER,
  p_idCiutada         IN VARCHAR2,
  p_idcenslocal	      IN NUMBER,
  p_idopcio	          IN NUMBER,
	RSP		OUT VARCHAR2
);


END PKG_CIUTADA;
/

CREATE OR REPLACE PACKAGE BODY USUARITFC11.PKG_CIUTADA AS

/********************************************************
*							*
*		MANTENIMENT CIUTADA			*
*							*
********************************************************/


/*******************************************************************************************************

NOM PROCEDIMENT: P_ALTA_CIUTADA

FUNCIONALITAT:

Aquest procediment permet donar dalta un ciutad a la Base de dades, si prviament supera els controls de:
	- Tots els camps obligatoris continguin valor.
	- El registre no shagi inserit a la Base de Dades.

PARAMETRES:
ENTRADA
Obligatoris:
	p_codi		Identificador del ciutad
	p_edat		Edat del ciutad

SORTIDA
Obligatoris:
	RSP	

PRECONDICI: No existeix el ciutad a crear.
POSTCONDICI: Es dona dalta el ciutad.

RETORNA:
   SI TOT CORRECTE
	OK si loperaci sha realitzat amb xit
   SI ES PRODUEIX UNA EXCEPCI
	'ERROR: : Els camps obligatoris no poden ser nul.'
	ERROR: El ciutad ja existeix a la Base de Dades.
	ERROR: Tipus derror. En qualsevol altre cas.

*******************************************************************************************************/

PROCEDURE PR_ALTA_CIUTADA  (	
  p_codi		IN VARCHAR2,
  p_edat		IN DATE,
  RSP			OUT VARCHAR2
)

IS

E_CAMPS_NUL		EXCEPTION;
E_CIUTADA_EXISTEIX	EXCEPTION;

existeix	VARCHAR2(1);
p_entrada	VARCHAR2(1000);
error_num	NUMBER;
error_missatge	VARCHAR2(100);

BEGIN

   --S'emmagatzema els parmetres d'entrada a la variable p_entrada

   p_entrada := 'P_CODI: ' ||p_codi||', P_EDAT: '||p_edat;

   --Es verifica que tots els camps obligatoris contingin valor

   IF (TRIM(p_codi) IS NULL OR 
        TRIM(p_edat) IS NULL)
   THEN
      RAISE E_CAMPS_NUL;
   END IF;

   --Es verifica que el registre no s'hagi insertat anteriorment amb altre identificador.

   BEGIN
   
     SELECT 'Y' 
     INTO existeix 
     FROM CIUTADA
     WHERE codi = p_codi AND ROWNUM = 1;
  
     RAISE E_CIUTADA_EXISTEIX;
   EXCEPTION
     WHEN NO_DATA_FOUND THEN
       NULL;
   END;

   --Si es pasa totes les proves de validaci, es realitzar l'inserci del nou registre a la Base de Dades.

   INSERT INTO CIUTADA (codi, edat)
   VALUES (p_codi, p_edat);
	
   RSP := 'OK';

   INSERT INTO LOG (NOMPROCEDIMENT, DATA, PARAMETREENTRADA, PARAMETRESORTIDA)
   VALUES ('PKG_CIUTADA.PR_ALTA_CIUTADA', SYSDATE, p_entrada, 'RSP: '||RSP);

   COMMIT;
   
   --Excepcions
EXCEPTION
 --Els camps obligatoris sn nuls
   WHEN E_CAMPS_NUL THEN 
    RSP:= 'ERROR: : Els camps obligatoris no poden ser nul'; 

   INSERT INTO LOG (NOMPROCEDIMENT, DATA, PARAMETREENTRADA, PARAMETRESORTIDA)
   VALUES ('PKG_CIUTADA.PR_ALTA_CIUTADA', SYSDATE, p_entrada, 'RSP: '||RSP);
   
   COMMIT;

 --El ciutad existeix a la Base de Dades
   WHEN E_CIUTADA_EXISTEIX THEN 
    RSP:= 'ERROR: : El ciutad ja existeix a la Base de Dades'; 

   INSERT INTO LOG (NOMPROCEDIMENT, DATA, PARAMETREENTRADA, PARAMETRESORTIDA)
   VALUES ('PKG_CIUTADA.PR_ALTA_CIUTADA', SYSDATE, p_entrada, 'RSP: '||RSP);
   
   COMMIT;

   WHEN OTHERS THEN
     error_num := SQLCODE;
     error_missatge := SUBSTR(SQLERRM, 1, 200);

     RSP:= 'ERROR '||TO_CHAR(error_num)||': '||error_missatge;
  
     INSERT INTO LOG (NOMPROCEDIMENT, DATA, PARAMETREENTRADA, PARAMETRESORTIDA)
     VALUES ('PKG_CIUTADA.PR_ALTA_CIUTADA', SYSDATE, p_entrada, 'RSP: '||RSP);

     COMMIT;

END PR_ALTA_CIUTADA;

/*******************************************************************************************************

NOM PROCEDIMENT: P_BAIXA_CIUTADA

FUNCIONALITAT:

Aquest procediment permet donar de baixa dun ciutad a la Base de dades, si prviament supera els controls de:
	- Tots els camps obligatoris continguin valor.
	- Existeix el ciutad a la Base de Dades.

PARAMETRES:
ENTRADA
Obligatoris:
	p_codi	Identificaci del ciutad

SORTIDA
Obligatoris:
	RSP	

PRECONDICI: Existeix el ciutad a donar de baixa.
POSTCONDICI: Es dona de baixa el ciutad.

RETORNA:
   SI TOT CORRECTE
	OK si loperaci sha realitzat amb xit
   SI ES PRODUEIX UNA EXCEPCI
	ERROR: El ciutad no existeix a la Base de Dades.
	ERROR: El camp CODI no pot ser nul.
	ERROR: Tipus derror. En qualsevol altre cas.

*******************************************************************************************************/

PROCEDURE PR_BAIXA_CIUTADA  (
	p_codi  	IN VARCHAR2,
	RSP		OUT VARCHAR2
)

IS

E_CIUTADA_NOEXISTEIX	EXCEPTION;
E_CAMPS_NUL		EXCEPTION;

existeix	VARCHAR2(1);
p_entrada	VARCHAR2(1000);
error_num	NUMBER;
error_missatge	VARCHAR2(100);

BEGIN

   --S'emmagatzema els parmetres d'entrada a la variable p_entrada

   p_entrada := 'P_CODI: '||p_codi;

   --Es verifica que tots els camps obligatoris contingin valor

   IF (TRIM(p_codi) IS NULL)
   THEN
      RAISE E_CAMPS_NUL;
   END IF;

   --Es verifica que el registre no s'hagi insertat anteriorment amb altre identificador.

   BEGIN
   
   SELECT 'Y' 
   INTO existeix 
   FROM CIUTADA 
   WHERE codi = p_codi AND ROWNUM = 1;

   EXCEPTION
   WHEN NO_DATA_FOUND THEN
     RAISE E_CIUTADA_NOEXISTEIX;
   END;

   --Si es pasa totes les proves de validaci, es realitzar la baixa del registre a la Base de Dades.

   DELETE FROM CIUTADA
   WHERE codi = p_codi;
	
   RSP := 'OK';

   INSERT INTO LOG (NOMPROCEDIMENT, DATA, PARAMETREENTRADA, PARAMETRESORTIDA)
   VALUES ('PKG_CIUTADA.PR_BAIXA_CIUTADA', SYSDATE, p_entrada, 'RSP: '||RSP);

   COMMIT;

   --Excepcions
EXCEPTION

 --El cens-local no existeix a la Base de Dades
   WHEN E_CIUTADA_NOEXISTEIX THEN RSP:= 'ERROR: : El ciutad no existeix a la Base de Dades'; 

   INSERT INTO LOG (NOMPROCEDIMENT, DATA, PARAMETREENTRADA, PARAMETRESORTIDA)
   VALUES ('PKG_CIUTADA.PR_BAIXA_CIUTADA', SYSDATE, p_entrada, 'RSP: '||RSP);

   COMMIT;

 --Els camps obligatoris sn nuls
   WHEN E_CAMPS_NUL THEN RSP:= 'ERROR: : El camp CODI no pot ser nul'; 

   INSERT INTO LOG (NOMPROCEDIMENT, DATA, PARAMETREENTRADA, PARAMETRESORTIDA)
   VALUES ('PKG_CIUTADA.PR_BAIXA_CIUTADA', SYSDATE, p_entrada, 'RSP: '||RSP);

   COMMIT;

   WHEN OTHERS THEN
   error_num := SQLCODE;
   error_missatge := SUBSTR(SQLERRM, 1, 200);

   RSP:= 'ERROR '||TO_CHAR(error_num)||': '||error_missatge;

   INSERT INTO LOG (NOMPROCEDIMENT, DATA, PARAMETREENTRADA, PARAMETRESORTIDA)
   VALUES ('PKG_CIUTADA.PR_BAIXA_CIUTADA', SYSDATE, p_entrada, 'RSP: '||RSP);

   COMMIT;

END PR_BAIXA_CIUTADA;


/*******************************************************************************************************

NOM PROCEDIMENT: P_MODIFICACIO_CIUTADA

FUNCIONALITAT:

Aquest procediment permet modificar un ciutad a la Base de dades, si prviament supera els controls de:
	- El ciutad existeix a la Base de Dades.
	- Tots els camps obligatoris continguin valor.

PARAMETRES: 
ENTRADA
Obligatoris:
	p_codi		Identificaci del ciutad
Opcionals:
	edat		Edat del ciutad

SORTIDA
Obligatoris:
	RSP	

PRECONDICI: Existeix el ciutad a modificar.
POSTCONDICI: Es modifica el ciutad.

RETORNA:
   SI TOT CORRECTE
	OK si loperaci sha realitzat amb xit
   SI ES PRODUEIX UNA EXCEPCI
	ERROR: El codi del ciutad no pot ser nul.
	ERROR: El ciutad no existeix a la Base de Dades.
	ERROR: Tipus derror. En qualsevol altre cas.

*******************************************************************************************************/

PROCEDURE PR_MODIFICACIO_CIUTADA (
	p_codi		IN VARCHAR2,
	p_edat		IN DATE,
	RSP		OUT VARCHAR2
)

IS

E_CAMPS_NUL		EXCEPTION;
E_ASSIGNACIO_EXISTEIX	EXCEPTION;
E_CIUTADA_NOEXISTEIX	EXCEPTION;

existeix	VARCHAR2(1);
p_entrada	VARCHAR2(1000);
error_num	NUMBER;
error_missatge	VARCHAR2(100);

BEGIN

   --S'emmagatzema els parmetres d'entrada a la variable p_entrada
 
   p_entrada := 'P_CODI: '||p_codi||', P_EDAT: '||p_edat;

   --Es verifica que tots els camps obligatoris contingin valor

   IF (TRIM(p_codi) IS NULL) THEN
      RAISE E_CAMPS_NUL;
   END IF;

   --Es verifica que el ciutad existeix a la Base de Dades
   BEGIN

     SELECT 'Y' 
     INTO existeix 
     FROM CIUTADA
     WHERE codi = p_codi AND ROWNUM = 1;

   EXCEPTION
     WHEN NO_DATA_FOUND THEN
         RAISE E_CIUTADA_NOEXISTEIX;
   END;

   --Si es pasa totes les proves de validaci, es realitzar l'actualitzaci a la Base de Dades.

   UPDATE CIUTADA
        SET edat		  = nvl(p_edat, edat)
   WHERE codi      		= p_codi;

   RSP := 'OK';

   INSERT INTO LOG (NOMPROCEDIMENT, DATA, PARAMETREENTRADA, PARAMETRESORTIDA)
   VALUES ('PKG_CIUTADA.PR_MODIFICACIO_CIUTADA', SYSDATE, p_entrada, 'RSP: '||RSP);

   COMMIT;

   --Excepcions
EXCEPTION

 --Els camps obligatoris sn nuls
   WHEN E_CAMPS_NUL THEN RSP:= 'ERROR: : El codi del ciutad no pot ser nul'; 

   INSERT INTO LOG (NOMPROCEDIMENT, DATA, PARAMETREENTRADA, PARAMETRESORTIDA)
   VALUES ('PKG_CIUTADA.PR_MODIFICACIO_CIUTADA', SYSDATE, p_entrada, 'RSP: '||RSP);

   COMMIT;

 --El ciutad no existeix a la Base de Dades
   WHEN E_CIUTADA_NOEXISTEIX THEN RSP:= 'ERROR: : El ciutad no existeix a la Base de Dades'; 

   INSERT INTO LOG (NOMPROCEDIMENT, DATA, PARAMETREENTRADA, PARAMETRESORTIDA)
   VALUES ('PKG_CIUTADA.PR_MODIFICACIO_CIUTADA', SYSDATE, p_entrada, 'RSP: '||RSP);

   COMMIT; 

   WHEN OTHERS THEN
   error_num := SQLCODE;
   error_missatge := SUBSTR(SQLERRM, 1, 200);

   RSP:= 'ERROR '||TO_CHAR(error_num)||': '||error_missatge;

   INSERT INTO LOG (NOMPROCEDIMENT, DATA, PARAMETREENTRADA, PARAMETRESORTIDA)
   VALUES ('PKG_CIUTADA.PR_MODIFICACIO_CIUTADA', SYSDATE, p_entrada, 'RSP: '||RSP);

   COMMIT;

END PR_MODIFICACIO_CIUTADA;


/********************************************************
*							*
*		MANTENIMENT CIUTADAOPCIO			*
*							*
********************************************************/


/*******************************************************************************************************

NOM PROCEDIMENT: P_ALTA_CIUTADAOPCIO

FUNCIONALITAT:

Aquest procediment permet donar dalta un ciutad a la Base de dades, si prviament supera els controls de:
	- Tots els camps obligatoris continguin valor.
	- El registre no shagi inserit a la Base de Dades.
  - Existeix el ciutad que es passa per parmetre.
	- Existeix el cens local que es passa per parmetre.
	- Existeix el lopci que es passa per parmetre.

PARAMETRES:
ENTRADA
Obligatoris:
  p_idCiutada		    Identificador del ciutad
	p_idcenslocal	    Identificador del cens localitat
	p_idopcio	        Identificador de lopci

SORTIDA
Obligatoris:
	RSP	

PRECONDICI: No existeix el ciutad-opci a crear.
POSTCONDICI: Es dona dalta el ciutad-opci.

RETORNA:
   SI TOT CORRECTE
	OK si loperaci sha realitzat amb xit
   SI ES PRODUEIX UNA EXCEPCI
	'ERROR: Els camps obligatoris no poden ser nul.'
  'ERROR: El ciutad-opci existeix a la Base de Dades.'
	ERROR: El ciutad no existeix a la Base de Dades.
	ERROR: El cens-local no existeix a la Base de Dades.
	ERROR: L'opci no existeix a la Base de Dades.
	ERROR: Tipus derror. En qualsevol altre cas.

*******************************************************************************************************/

PROCEDURE PR_ALTA_CIUTADAOPCIO  (	
  p_idCiutada		    IN VARCHAR2,
  p_idcenslocal	   	IN NUMBER,
  p_idopcio	   	    IN NUMBER,
  RSP			          OUT VARCHAR2
)

IS

E_CAMPS_NUL		            EXCEPTION;
E_CIUTADAOPCIO_EXISTEIX	  EXCEPTION;
E_CIUTADA_NOEXISTEIX	    EXCEPTION;
E_CENSLOCAL_NOEXISTEIX	  EXCEPTION;
E_OPCIO_NOEXISTEIX	      EXCEPTION;

existeix	VARCHAR2(1);
p_entrada	VARCHAR2(1000);
error_num	NUMBER;
error_missatge	VARCHAR2(100);

BEGIN

   --S'emmagatzema els parmetres d'entrada a la variable p_entrada

   p_entrada := 'P_IDCIUTADA: '||p_idCiutada||', P_IDCENSLOCAL: '||p_idcenslocal||
   ', P_IDOPCIO: '||p_idopcio;

   --Es verifica que tots els camps obligatoris contingin valor

   IF (TRIM(p_idCiutada) IS NULL OR
        TRIM(p_idcenslocal) IS NULL OR
        TRIM(p_idopcio) IS NULL)
   THEN
      RAISE E_CAMPS_NUL;
   END IF;

   --Es verifica que existeix el ciutad a la Base de Dades.
   
   BEGIN
   
     SELECT 'Y' 
     INTO existeix 
     FROM CIUTADAOPCIO
     WHERE idCiutada = p_idCiutada AND  idCensLocal = p_idCensLocal 
          AND idOpcio = p_idOpcio AND ROWNUM = 1;
          
    RAISE E_CIUTADAOPCIO_EXISTEIX;
    
   EXCEPTION
     WHEN NO_DATA_FOUND THEN
       NULL;
   END;
   
   --Es verifica que existeix el ciutad a la Base de Dades.
   
   BEGIN
   
     SELECT 'Y' 
     INTO existeix 
     FROM CIUTADA
     WHERE codi = p_idCiutada AND ROWNUM = 1;

   EXCEPTION
     WHEN NO_DATA_FOUND THEN
       RAISE E_CIUTADA_NOEXISTEIX;
   END;

   --Es verifica que el cens-local que es pasa per parmetre existeix

   BEGIN
   
     SELECT 'Y' 
     INTO existeix 
     FROM CENSLOCAL
     WHERE idcenslocal = p_idcenslocal AND ROWNUM = 1;   

   EXCEPTION
     WHEN NO_DATA_FOUND THEN
       RAISE E_CENSLOCAL_NOEXISTEIX;
   END;

   --Es verifica que l'opci que es pasa per parmetre existeix

   BEGIN
   
     SELECT 'Y' 
     INTO existeix 
     FROM OPCIO
     WHERE idopcio = p_idopcio AND ROWNUM = 1;   

   EXCEPTION
     WHEN NO_DATA_FOUND THEN
       RAISE E_OPCIO_NOEXISTEIX;
   END;

   --Si es pasa totes les proves de validaci, es realitzar l'inserci del nou registre a la Base de Dades.

   INSERT INTO CIUTADAOPCIO (idCiutada, idcenslocal, idopcio)
   VALUES (p_idCiutada, p_idcenslocal, p_idopcio);
	
   RSP := 'OK';

   INSERT INTO LOG (NOMPROCEDIMENT, DATA, PARAMETREENTRADA, PARAMETRESORTIDA)
   VALUES ('PKG_CIUTADA.PR_ALTA_CIUTADAOPCIO', SYSDATE, p_entrada, 'RSP: '||RSP);

   COMMIT;
   
   --Excepcions
EXCEPTION
 --Els camps obligatoris sn nuls
   WHEN E_CAMPS_NUL THEN 
    RSP:= 'ERROR: : Els camps obligatoris no poden ser nul'; 

   INSERT INTO LOG (NOMPROCEDIMENT, DATA, PARAMETREENTRADA, PARAMETRESORTIDA)
   VALUES ('PKG_CIUTADA.PR_ALTA_CIUTADAOPCIO', SYSDATE, p_entrada, 'RSP: '||RSP);
   
   COMMIT;

 --El existeix el registre a la Base de Dades
   WHEN E_CIUTADAOPCIO_EXISTEIX THEN 
    RSP:= 'ERROR: : El ciutad-opci existeix a la Base de Dades'; 

   INSERT INTO LOG (NOMPROCEDIMENT, DATA, PARAMETREENTRADA, PARAMETRESORTIDA)
   VALUES ('PKG_CIUTADA.PR_ALTA_CIUTADAOPCIO', SYSDATE, p_entrada, 'RSP: '||RSP);
   
   COMMIT;

 --El ciutad existeix a la Base de Dades
   WHEN E_CIUTADA_NOEXISTEIX THEN 
    RSP:= 'ERROR: : El ciutad NO existeix a la Base de Dades'; 

   INSERT INTO LOG (NOMPROCEDIMENT, DATA, PARAMETREENTRADA, PARAMETRESORTIDA)
   VALUES ('PKG_CIUTADA.PR_ALTA_CIUTADAOPCIO', SYSDATE, p_entrada, 'RSP: '||RSP);
   
   COMMIT;

 --El cens-local no existeix a la Base de Dades
   WHEN E_CENSLOCAL_NOEXISTEIX 
    THEN RSP:= 'ERROR: : El cens-local no existeix a la Base de Dades'; 

   INSERT INTO LOG (NOMPROCEDIMENT, DATA, PARAMETREENTRADA, PARAMETRESORTIDA)
   VALUES ('PKG_CIUTADA.PR_ALTA_CIUTADAOPCIO', SYSDATE, p_entrada, 'RSP: '||RSP);
   
   COMMIT;

 --L'opci no existeix a la Base de Dades
   WHEN E_OPCIO_NOEXISTEIX THEN RSP:= 'ERROR: : Aquesta opci no existeix a la Base de Dades'; 

   INSERT INTO LOG (NOMPROCEDIMENT, DATA, PARAMETREENTRADA, PARAMETRESORTIDA)
   VALUES ('PKG_CIUTADA.PR_ALTA_CIUTADAOPCIO', SYSDATE, p_entrada, 'RSP: '||RSP);
   
   COMMIT;

   WHEN OTHERS THEN
     error_num := SQLCODE;
     error_missatge := SUBSTR(SQLERRM, 1, 200);

     RSP:= 'ERROR '||TO_CHAR(error_num)||': '||error_missatge;
  
     INSERT INTO LOG (NOMPROCEDIMENT, DATA, PARAMETREENTRADA, PARAMETRESORTIDA)
     VALUES ('PKG_CIUTADA.PR_ALTA_CIUTADAOPCIO', SYSDATE, p_entrada, 'RSP: '||RSP);

     COMMIT;

END PR_ALTA_CIUTADAOPCIO;


/*******************************************************************************************************

NOM PROCEDIMENT: P_BAIXA_CIUTADAOPCIO

FUNCIONALITAT:

Aquest procediment permet donar de baixa dun ciutad a la Base de dades, si prviament supera els controls de:
	- Tots els camps obligatoris continguin valor.
	- Existeix el ciutad a la Base de Dades.

PARAMETRES:
ENTRADA
Obligatoris:
	p_idCiutadaOpcio	Identificaci del ciutad-opci

SORTIDA
Obligatoris:
	RSP	

PRECONDICI: Existeix el ciutad-opci a donar de baixa.
POSTCONDICI: Es dona de baixa el ciutad-opci.

RETORNA:
   SI TOT CORRECTE
	OK si loperaci sha realitzat amb xit
   SI ES PRODUEIX UNA EXCEPCI
	ERROR: El ciutad-opci no existeix a la Base de Dades.
	ERROR: El camp idCiutadaOpcio no pot ser nul.
	ERROR: Tipus derror. En qualsevol altre cas.

*******************************************************************************************************/

PROCEDURE PR_BAIXA_CIUTADAOPCIO  (
	p_idCiutadaOpcio  	IN NUMBER,
	RSP		OUT VARCHAR2
)

IS

E_CIUTADAOPCIO_NOEXISTEIX	EXCEPTION;
E_CAMPS_NUL		EXCEPTION;

existeix	VARCHAR2(1);
p_entrada	VARCHAR2(1000);
error_num	NUMBER;
error_missatge	VARCHAR2(100);

BEGIN

   --S'emmagatzema els parmetres d'entrada a la variable p_entrada

   p_entrada := 'P_IDCIUTADAOPCIO: '||p_idCiutadaOpcio;

   --Es verifica que tots els camps obligatoris contingin valor

   IF (TRIM(p_idCiutadaOpcio) IS NULL)
   THEN
      RAISE E_CAMPS_NUL;
   END IF;

   --Es verifica que el registre no s'hagi insertat anteriorment amb altre identificador.

   BEGIN
   
   SELECT 'Y' 
   INTO existeix 
   FROM CIUTADAOPCIO 
   WHERE idCiutadaOpcio = p_idCiutadaOpcio AND ROWNUM = 1;

   EXCEPTION
   WHEN NO_DATA_FOUND THEN
     RAISE E_CIUTADAOPCIO_NOEXISTEIX;
   END;

   --Si es pasa totes les proves de validaci, es realitzar la baixa del registre a la Base de Dades.

   DELETE FROM CIUTADAOPCIO
   WHERE idCiutadaOpcio = p_idCiutadaOpcio;
	
   RSP := 'OK';

   INSERT INTO LOG (NOMPROCEDIMENT, DATA, PARAMETREENTRADA, PARAMETRESORTIDA)
   VALUES ('PKG_CIUTADA.PR_BAIXA_CIUTADAOPCIO', SYSDATE, p_entrada, 'RSP: '||RSP);

   COMMIT;

   --Excepcions
EXCEPTION

 --El cens-local no existeix a la Base de Dades
   WHEN E_CIUTADAOPCIO_NOEXISTEIX THEN RSP:= 'ERROR: : El ciutad-opci no existeix a la Base de Dades'; 

   INSERT INTO LOG (NOMPROCEDIMENT, DATA, PARAMETREENTRADA, PARAMETRESORTIDA)
   VALUES ('PKG_CIUTADA.PR_BAIXA_CIUTADAOPCIO', SYSDATE, p_entrada, 'RSP: '||RSP);

   COMMIT;

 --Els camps obligatoris sn nuls
   WHEN E_CAMPS_NUL THEN RSP:= 'ERROR: : El camp IDCIUTADAOPCIO no pot ser nul'; 

   INSERT INTO LOG (NOMPROCEDIMENT, DATA, PARAMETREENTRADA, PARAMETRESORTIDA)
   VALUES ('PKG_CIUTADA.PR_BAIXA_CIUTADAOPCIO', SYSDATE, p_entrada, 'RSP: '||RSP);

   COMMIT;

   WHEN OTHERS THEN
   error_num := SQLCODE;
   error_missatge := SUBSTR(SQLERRM, 1, 200);

   RSP:= 'ERROR '||TO_CHAR(error_num)||': '||error_missatge;

   INSERT INTO LOG (NOMPROCEDIMENT, DATA, PARAMETREENTRADA, PARAMETRESORTIDA)
   VALUES ('PKG_CIUTADA.PR_BAIXA_CIUTADAOPCIO', SYSDATE, p_entrada, 'RSP: '||RSP);

   COMMIT;

END PR_BAIXA_CIUTADAOPCIO;

/*******************************************************************************************************

NOM PROCEDIMENT: P_MODIFICACIO_CIUTADAOPCIO

FUNCIONALITAT:

Aquest procediment permet modificar un ciutad a la Base de dades, si prviament supera els controls de:
	- El ciutad existeix a la Base de Dades.
	- Tots els camps obligatoris continguin valor.
	- Si es vol modificar el cens localitat, sha de verificar que existeix el cens localitat a la Base de Dades.
	- Si es vol modificar lopci, sha de verificar que existeix lopci a la Base de Dades.

PARAMETRES: 
ENTRADA
Obligatoris:
	p_idCiutadaOpcio		Identificaci del ciutad-opci
Opcionals:
  p_idCiutada Identificador del ciutad
	p_idcenslocal	Identificador del cens localitat
	p_idopcio	Identificador de lopci

SORTIDA
Obligatoris:
	RSP	

PRECONDICI: Existeix el ciutad-opci a modificar.
POSTCONDICI: Es modifica el ciutad-opci.

RETORNA:
   SI TOT CORRECTE
	OK si loperaci sha realitzat amb xit
   SI ES PRODUEIX UNA EXCEPCI
	ERROR: El idCiutadaOpcio no pot ser nul.
	ERROR: El ciutad no existeix a la Base de Dades.
	ERROR: El cens-local no existeix a la Base de Dades.
	ERROR: L'opci no existeix a la Base de Dades.
	ERROR: Tipus derror. En qualsevol altre cas.

*******************************************************************************************************/

PROCEDURE PR_MODIFICACIO_CIUTADAOPCIO (
	p_idCiutadaOpcio		IN NUMBER,
  p_idCiutada   IN VARCHAR2,
	p_idcenslocal	IN NUMBER,	
	p_idopcio 	IN NUMBER,
	RSP		OUT VARCHAR2
)

IS

E_CAMPS_NUL		EXCEPTION;
E_CIUTADAOPCIO_NOEXISTEIX	EXCEPTION;
E_CIUTADA_NOEXISTEIX	EXCEPTION;
E_CENSLOCAL_NOEXISTEIX	EXCEPTION;
E_OPCIO_NOEXISTEIX	EXCEPTION;

existeix	VARCHAR2(1);
p_entrada	VARCHAR2(1000);
error_num	NUMBER;
error_missatge	VARCHAR2(100);

BEGIN

   --S'emmagatzema els parmetres d'entrada a la variable p_entrada
 
   p_entrada := 'P_IDCIUTADAOPCIO: '||p_idCiutadaOpcio||', P_IDCIUTADA: '||p_idCiutada||
   ', P_IDCENSLOCAL: ' ||p_idcenslocal||', P_IDOPCIO: '||p_idopcio;

   --Es verifica que tots els camps obligatoris contingin valor

   IF (TRIM(p_idCiutadaOpcio) IS NULL) THEN
      RAISE E_CAMPS_NUL;
   END IF;

   --Es verifica que el ciutad-opci existeix a la Base de Dades
   BEGIN

     SELECT 'Y' 
     INTO existeix 
     FROM CIUTADAOPCIO
     WHERE idCiutadaOpcio = p_idCiutadaOpcio AND ROWNUM = 1;

   EXCEPTION
     WHEN NO_DATA_FOUND THEN
         RAISE E_CIUTADAOPCIO_NOEXISTEIX;
   END;
   
   --Es verifica que el ciutad existeix a la Base de Dades
   BEGIN

     SELECT 'Y' 
     INTO existeix 
     FROM CIUTADA
     WHERE codi = p_idCiutada AND ROWNUM = 1;

   EXCEPTION
     WHEN NO_DATA_FOUND THEN
         RAISE E_CIUTADA_NOEXISTEIX;
   END;   

   --Si es modifica el cens local, es verifica que el cens local existeix a la Base de Dades
   BEGIN

     SELECT 'Y' 
     INTO existeix 
     FROM CENSLOCAL
     WHERE idcenslocal = p_idcenslocal AND ROWNUM = 1;

   EXCEPTION
     WHEN NO_DATA_FOUND THEN
        RAISE E_CENSLOCAL_NOEXISTEIX;
   END;

   --Si es modifica l'opci, es verifica que l'opci existeix a la Base de Dades
   BEGIN

     SELECT 'Y' 
     INTO existeix 
     FROM OPCIO
     WHERE idopcio = p_idopcio AND ROWNUM = 1;

   EXCEPTION
     WHEN NO_DATA_FOUND THEN
        RAISE E_OPCIO_NOEXISTEIX;
   END;

   --Si es pasa totes les proves de validaci, es realitzar l'actualitzaci a la Base de Dades.

   UPDATE CIUTADAOPCIO
        SET   idCiutada	= nvl(p_idCiutada, idCiutada),
              idcenslocal	= nvl(p_idcenslocal, idcenslocal),
              idopcio    	= nvl(p_idopcio, idopcio)
   WHERE idCiutadaOpcio	= p_idCiutadaOpcio;

   RSP := 'OK';

   INSERT INTO LOG (NOMPROCEDIMENT, DATA, PARAMETREENTRADA, PARAMETRESORTIDA)
   VALUES ('PKG_CIUTADA.PR_MODIFICACIO_CIUTADAOPCIO', SYSDATE, p_entrada, 'RSP: '||RSP);

   COMMIT;

   --Excepcions
EXCEPTION

 --Els camps obligatoris sn nuls
   WHEN E_CAMPS_NUL THEN RSP:= 'ERROR: : El idCiutadaOpcio no pot ser nul'; 

   INSERT INTO LOG (NOMPROCEDIMENT, DATA, PARAMETREENTRADA, PARAMETRESORTIDA)
   VALUES ('PKG_CIUTADA.PR_MODIFICACIO_CIUTADAOPCIO', SYSDATE, p_entrada, 'RSP: '||RSP);

   COMMIT;

--El ciutad-opci no existeix a la Base de Dades
   WHEN E_CIUTADAOPCIO_NOEXISTEIX THEN RSP:= 'ERROR: : El ciutad-opci no existeix a la Base de Dades'; 

   INSERT INTO LOG (NOMPROCEDIMENT, DATA, PARAMETREENTRADA, PARAMETRESORTIDA)
   VALUES ('PKG_CIUTADA.PR_MODIFICACIO_CIUTADAOPCIO', SYSDATE, p_entrada, 'RSP: '||RSP);

   COMMIT; 

 --El ciutad no existeix a la Base de Dades
   WHEN E_CIUTADA_NOEXISTEIX THEN RSP:= 'ERROR: : El ciutad no existeix a la Base de Dades'; 

   INSERT INTO LOG (NOMPROCEDIMENT, DATA, PARAMETREENTRADA, PARAMETRESORTIDA)
   VALUES ('PKG_CIUTADA.PR_MODIFICACIO_CIUTADAOPCIO', SYSDATE, p_entrada, 'RSP: '||RSP);

   COMMIT; 

 --El cens local no existeix a la Base de Dades
   WHEN E_CENSLOCAL_NOEXISTEIX THEN RSP:= 'ERROR: : El cens-local no existeix a la Base de Dades'; 

   INSERT INTO LOG (NOMPROCEDIMENT, DATA, PARAMETREENTRADA, PARAMETRESORTIDA)
   VALUES ('PKG_CIUTADA.PR_MODIFICACIO_CIUTADAOPCIO', SYSDATE, p_entrada, 'RSP: '||RSP);

   COMMIT; 

 --L'opci no existeix a la Base de Dades
   WHEN E_OPCIO_NOEXISTEIX THEN RSP:= 'ERROR: : Aquesta opci no existeix a la Base de Dades'; 

   INSERT INTO LOG (NOMPROCEDIMENT, DATA, PARAMETREENTRADA, PARAMETRESORTIDA)
   VALUES ('PKG_CIUTADA.PR_MODIFICACIO_CIUTADAOPCIO', SYSDATE, p_entrada, 'RSP: '||RSP);

   COMMIT; 

   WHEN OTHERS THEN
   error_num := SQLCODE;
   error_missatge := SUBSTR(SQLERRM, 1, 200);

   RSP:= 'ERROR '||TO_CHAR(error_num)||': '||error_missatge;

   INSERT INTO LOG (NOMPROCEDIMENT, DATA, PARAMETREENTRADA, PARAMETRESORTIDA)
   VALUES ('PKG_CIUTADA.PR_MODIFICACIO_CIUTADAOPCIO', SYSDATE, p_entrada, 'RSP: '||RSP);

   COMMIT;

END PR_MODIFICACIO_CIUTADAOPCIO;




END PKG_CIUTADA;
/