
/************************************************************
*							    *
*  	       Triggers mdul d'estadstiques  		    *
*							    *
************************************************************/
 
/*******************************************************************************************************
NOM:      TR_AFT_INS_CENSPAIS

FUNCIONALITAT: 
  
Desprs de donar dalta un registre a la taula CENSPAIS, aquest:
- Incrementa el nmero de votacions finalitzades dun pas i un any concret.  Taula VOTACIONSFINALITZADES.
- Incrementa el nombre de votacions que han tingut associades un cens durant un any. Taula VOTACIONSCENS.

*******************************************************************************************************/

CREATE OR REPLACE TRIGGER USUARITFC11.TR_AFT_INS_CENSPAIS
AFTER INSERT ON USUARITFC11.CENSPAIS 
REFERENCING NEW AS NEW OLD AS OLD
FOR EACH ROW
DECLARE
         
PDATAFI                       DATE;
ANYNOU                        NUMBER;
VOTACIONFINALITZADES_CONT     NUMBER;
VOTACIONSCENS_CONT            NUMBER;

BEGIN

--1 Nombre de votacions finalitzades - TAULA VOTACIONSFINALITZADES
   -- SELECCIONAR L'ANY DE FINALITZACI
   BEGIN
     SELECT Extract(Year FROM VOTACIO.DATAINICI), DATAFI 
     INTO ANYNOU, PDATAFI
     FROM VOTACIO VOTACIO
     WHERE VOTACIO.IDVOTACIO = :NEW.IDVOTACIO;  
   END;
   
   BEGIN
     SELECT COUNT(*)
     INTO VOTACIONFINALITZADES_CONT
     FROM VOTACIONSFINALITZADES
     WHERE ANYS = ANYNOU AND IDPAIS = :NEW.IDPAIS;
   END;
   
   IF VOTACIONFINALITZADES_CONT != 0 THEN
     IF PDATAFI < SYSDATE THEN   
       UPDATE VOTACIONSFINALITZADES
       SET NOMBREVOTACIONSFINALITZADES = NOMBREVOTACIONSFINALITZADES + 1
       WHERE ANYS = ANYNOU AND IDPAIS = :NEW.IDPAIS;
     END IF;
  ELSE
    INSERT INTO VOTACIONSFINALITZADES (IDPAIS, ANYS, NOMBREVOTACIONSFINALITZADES)
    VALUES (:NEW.IDPAIS, ANYNOU, 1);      
   END IF;
   
--3 Nombre de votacions finalitzades - TAULA VOTACIONSCENS
   BEGIN
     SELECT COUNT(*)
     INTO VOTACIONSCENS_CONT
     FROM VOTACIONSCENS
     WHERE ANYS = ANYNOU AND IDCENS = :NEW.IDCENSPAIS;
   END;
   
   IF VOTACIONSCENS_CONT != 0 THEN
       UPDATE VOTACIONSCENS
       SET NOMBREVOTACIONSCENS = NOMBREVOTACIONSCENS + 1
       WHERE ANYS = ANYNOU AND IDCENS = :NEW.IDCENSPAIS;
  ELSE
    INSERT INTO VOTACIONSCENS (IDCENS, ANYS, NOMBREVOTACIONSCENS)
    VALUES (:NEW.IDCENSPAIS, ANYNOU, 1);      
   END IF;  

END TR_AFT_INS_CENSPAIS;
/ 


/*******************************************************************************************************
NOM:      TR_AFT_DEL_CENSPAIS

FUNCIONALITAT: 
  
Desprs de donar de baixa un registre a la taula CENSPAIS, aquest:
- Decrement del nmero de votacions finalitzades dun pas i un any concret.  Taula VOTACIONSFINALITZADES.
- Decrement del nombre de votacions que han tingut associades un cens durant un any. Taula VOTACIONSCENS. 

*******************************************************************************************************/

CREATE OR REPLACE TRIGGER USUARITFC11.TR_AFT_DEL_CENSPAIS
AFTER DELETE ON USUARITFC11.CENSPAIS 
REFERENCING NEW AS NEW OLD AS OLD
FOR EACH ROW
DECLARE
     
PDATAFIANTIC                DATE;
ANYANTIC                    NUMBER;
VOTACIONFINALITZADES_CONT   NUMBER;
VOTACIONSCENS_CONT          NUMBER;

BEGIN

--1 Nombre de votacions finalitzades - TAULA VOTACIONSFINALITZADES
   -- SELECCIONAR L'ANY DE FINALITZACI  
   BEGIN
     SELECT Extract(Year FROM VOTACIO.DATAINICI), DATAFI 
     INTO ANYANTIC, PDATAFIANTIC
     FROM VOTACIO 
     WHERE VOTACIO.IDVOTACIO = :OLD.IDVOTACIO;   
     
     UPDATE VOTACIONSFINALITZADES
     SET NOMBREVOTACIONSFINALITZADES = NOMBREVOTACIONSFINALITZADES - 1
     WHERE ANYS = ANYANTIC AND IDPAIS = :OLD.IDPAIS;       
   END;
   
--3 Nombre de votacions finalitzades - TAULA VOTACIONSCENS

  UPDATE VOTACIONSCENS
  SET NOMBREVOTACIONSCENS = NOMBREVOTACIONSCENS - 1
  WHERE ANYS = ANYANTIC AND IDCENS = :OLD.IDCENSPAIS;    
     
  --ELIMINEM SI QUEDA EL REGISTRE A 0
  BEGIN
     SELECT NOMBREVOTACIONSCENS
     INTO VOTACIONSCENS_CONT
     FROM VOTACIONSCENS
     WHERE ANYS = ANYANTIC AND IDCENS = :OLD.IDCENSPAIS;
     
     IF VOTACIONSCENS_CONT = 0 THEN
        DELETE FROM VOTACIONSCENS
        WHERE ANYS = ANYANTIC AND IDCENS = :OLD.IDCENSPAIS;
     END IF;
  END;      

END TR_AFT_DEL_CENSPAIS;
/


/*******************************************************************************************************
NOM:      TR_AFT_UP_CENSPAIS

FUNCIONALITAT: 

Desprs dactualitzar un registre a la taula CENSPAIS, aquest:
- Incrementa el nou registre i decreix lantic registre de votacions finalitzades dun pas i un any concret.  
Taula VOTACIONSFINALITZADES.
- Incrementa el nou registre i decreix el nombre de votacions que han tingut associades un cens durant un any. 
Taula VOTACIONSCENS.

*******************************************************************************************************/

CREATE OR REPLACE TRIGGER USUARITFC11.TR_AFT_UP_CENSPAIS
AFTER UPDATE ON USUARITFC11.CENSPAIS 
REFERENCING NEW AS NEW OLD AS OLD
FOR EACH ROW
DECLARE
     
PDATAFINOU                  DATE;
PDATAFIANTIC                DATE;
ANYNOU                      NUMBER;
ANYANTIC                    NUMBER;
VOTACIONFINALITZADES_CONT   NUMBER;
VOTACIONSCENS_CONT            NUMBER;

BEGIN

--1 Nombre de votacions finalitzades - TAULA VOTACIONSFINALITZADES
   -- SELECCIONAR L'ANY DE FINALITZACI  
   BEGIN
     SELECT Extract(Year FROM VOTACIO.DATAINICI), DATAFI 
     INTO ANYANTIC, PDATAFIANTIC
     FROM VOTACIO 
     WHERE VOTACIO.IDVOTACIO = :OLD.IDVOTACIO;   
   END;
   
   BEGIN
     SELECT Extract(Year FROM VOTACIO.DATAINICI), DATAFI
     INTO ANYNOU, PDATAFINOU
     FROM VOTACIO 
     WHERE VOTACIO.IDVOTACIO = :NEW.IDVOTACIO;
   END;
   --MIREM SI EXISTEIX L'ANY NOU A LA TAULA
   BEGIN
     SELECT COUNT(*)
     INTO VOTACIONFINALITZADES_CONT
     FROM VOTACIONSFINALITZADES
     WHERE ANYS = ANYNOU;
     
     IF VOTACIONFINALITZADES_CONT !=0 THEN
       IF PDATAFINOU < SYSDATE THEN   
         UPDATE VOTACIONSFINALITZADES
         SET NOMBREVOTACIONSFINALITZADES = NOMBREVOTACIONSFINALITZADES + 1
         WHERE ANYS = ANYNOU AND IDPAIS = :NEW.IDPAIS;
       END IF;
     ELSE
        INSERT INTO VOTACIONSFINALITZADES (IDPAIS, ANYS, NOMBREVOTACIONSFINALITZADES)
        VALUES (:NEW.IDPAIS, ANYNOU, 1);        
     END IF;
     
     UPDATE VOTACIONSFINALITZADES
     SET NOMBREVOTACIONSFINALITZADES = NOMBREVOTACIONSFINALITZADES - 1
     WHERE ANYS = ANYANTIC AND IDPAIS = :OLD.IDPAIS;       
   END;
   
--3 Nombre de votacions finalitzades - TAULA VOTACIONSCENS
   BEGIN
     SELECT COUNT(*)
     INTO VOTACIONSCENS_CONT
     FROM VOTACIONSCENS
     WHERE ANYS = ANYNOU AND IDCENS = :NEW.IDCENSPAIS;
   END;
   
   IF VOTACIONSCENS_CONT != 0 THEN
       UPDATE VOTACIONSCENS
       SET NOMBREVOTACIONSCENS = NOMBREVOTACIONSCENS + 1
       WHERE ANYS = ANYNOU AND IDCENS = :NEW.IDCENSPAIS;
  ELSE
    INSERT INTO VOTACIONSCENS (IDCENS, ANYS, NOMBREVOTACIONSCENS)
    VALUES (:NEW.IDCENSPAIS, ANYNOU, 1);      
   END IF;   

  UPDATE VOTACIONSCENS
  SET NOMBREVOTACIONSCENS = NOMBREVOTACIONSCENS - 1
  WHERE ANYS = ANYANTIC AND IDCENS = :OLD.IDCENSPAIS;    
     
  --ELIMINEM SI QUEDA EL REGISTRE A 0
  BEGIN
     SELECT NOMBREVOTACIONSCENS
     INTO VOTACIONSCENS_CONT
     FROM VOTACIONSCENS
     WHERE ANYS = ANYANTIC AND IDCENS = :OLD.IDCENSPAIS;
     
     IF VOTACIONSCENS_CONT = 0 THEN
        DELETE FROM VOTACIONSCENS
        WHERE ANYS = ANYANTIC AND IDCENS = :OLD.IDCENSPAIS;
     END IF;
  END;   

END TR_AFT_UP_CENSPAIS;
/


/*******************************************************************************************************
NOM:      TR_AFT_INS_VOTACIO

FUNCIONALITAT: 

Desprs de donar un registre a la taula VOTACIO, aquest:
- Mira si no existeix la votaci amb ms participaci donat un any, si no es aix, la crea. 
Taula VOTACIOMESPARTICIPACIO.
- Mira si la nova votaci hi ha hagut ms participaci, si s aix lactualitza. 
Taula VOTHISTORICAMESPARTICIPACIO.

*******************************************************************************************************/

CREATE OR REPLACE TRIGGER USUARITFC11.TR_AFT_INS_VOTACIO
AFTER INSERT ON VOTACIO
REFERENCING NEW AS NEW OLD AS OLD
FOR EACH ROW
DECLARE
     
VOTHISMESPART_TOTAL   NUMBER;   
PARTICIPACIO_TOTAL    NUMBER;

BEGIN

-- 4 VOTACIO MS PARTICIPACI DONAT UN ANY
   --Seleccionar la votaci amb ms participaci
    --Es crea el registre a VOTACIOMESPARTICIPACIO  
   BEGIN
     SELECT COUNT(*)
     INTO PARTICIPACIO_TOTAL
     FROM VOTACIOMESPARTICIPACIO
     WHERE ANYS = Extract(Year FROM :NEW.DATAINICI);
     
     IF PARTICIPACIO_TOTAL =0 THEN   
       INSERT INTO VOTACIOMESPARTICIPACIO (ANYS, VOTACIOMESPART, PARTICIPACIO)
       VALUES (Extract(Year FROM :NEW.DATAINICI), :NEW.TITOL, 0);    
    END IF;
   END; 

-- 7 VOTACIO HISTORICAMENT MS PARTICIPACI
   --Seleccionar la votaci amb ms participaci
    --Es crea el registre a VOTHISTORICAMESPARTICIPACIO  
   BEGIN
     SELECT TOTALVOTS
     INTO VOTHISMESPART_TOTAL
     FROM VOTHISTORICAMESPARTICIPACIO
     WHERE ROWNUM= 1;    

     IF (:NEW.TOTALCIUTADANS > VOTHISMESPART_TOTAL ) THEN
      VOTHISMESPART_TOTAL := :NEW.TOTALCIUTADANS;
     END IF;
         
     UPDATE VOTHISTORICAMESPARTICIPACIO
     SET TOTALVOTS = VOTHISMESPART_TOTAL, VOTHISMESPART = :NEW.TITOL
     WHERE ROWNUM = 1; 

   EXCEPTION
      WHEN NO_DATA_FOUND THEN
       INSERT INTO VOTHISTORICAMESPARTICIPACIO (TOTALVOTS, VOTHISMESPART)
       VALUES (VOTHISMESPART_TOTAL, :NEW.TITOL);    
   END; 
   
   

END TR_AFT_INS_VOTACIO;
/


/*******************************************************************************************************
NOM:      TR_AFT_DEL_VOTACIO

FUNCIONALITAT:  

Desprs de donar de baixa un registre a la taula VOTACIO, aquest:
- Decreix la mitja de percentatge de participaci si la votaci es finalitzada donat un cens i un any. 
Taula MITJAPERCENTATGEPARTICIPACIO.
- Decreix la participaci duna votaci donat un any concret. Taula VOTACIOMESPARTICIPACIO.
- Mira si la votaci histrica a eliminar s la que ms participaci ha tingut. Si es aix, lactualitza. 
Taula VOTHISTORICAMESPARTICIPACIO.

*******************************************************************************************************/

CREATE OR REPLACE TRIGGER USUARITFC11.TR_AFT_DEL_VOTACIO
AFTER DELETE ON USUARITFC11.VOTACIO 
REFERENCING NEW AS NEW OLD AS OLD
FOR EACH ROW
DECLARE
       
VOTHISMESPART_TITOL         VARCHAR2(30);
PARTICIPACIO_TOTALNOU      NUMBER;
PARTICIPACIO_TOTALANTIC    NUMBER;
MITJAPERPARTICIPACIO_CENSANTIC   VARCHAR2(2);

BEGIN

--2 VALOR MITJA PERCENTATGE PARTICIPACI DONAT UN CENS I UN ANY 
-- TAULA MITJAPERCENTATGEPARTICIPACIO
   --ES CERCA EL CENSPAIS
   BEGIN
      SELECT IDCENSPAIS
      INTO MITJAPERPARTICIPACIO_CENSANTIC
      FROM CENSPAIS
      WHERE IDVOTACIO = :OLD.IDVOTACIO;      
      
      UPDATE MITJAPERCENTATGEPARTICIPACIO
      SET TOTALVOTACIONS = TOTALVOTACIONS - 1, 
      VALORMITJAPERPART = ((VALORMITJAPERPART * TOTALVOTACIONS) - :OLD.PERCENTATGEVOTACIO) / (TOTALVOTACIONS - 1)
      WHERE ANYS = Extract(Year FROM :OLD.DATAFI) AND IDCENS = MITJAPERPARTICIPACIO_CENSANTIC;                     
   END;

-- 4 VOTACIO MS PARTICIPACI DONAT UN ANY
   --Seleccionar la votaci amb ms participaci
    --Es crea el registre a VOTACIOMESPARTICIPACIO  
   BEGIN
     SELECT PARTICIPACIO
     INTO PARTICIPACIO_TOTALANTIC
     FROM VOTACIOMESPARTICIPACIO
     WHERE ANYS = Extract(Year FROM :OLD.DATAINICI);

     IF :OLD.TOTALCIUTADANS > PARTICIPACIO_TOTALANTIC THEN
         UPDATE VOTACIOMESPARTICIPACIO
         SET VOTACIOMESPART = '-', PARTICIPACIO = 0
         WHERE ANYS = Extract(Year FROM :OLD.DATAINICI); 
     END IF;     
     
   END; 

--7 VOTACIO HISTORICAMENT MS PARTICIPACI
   --Seleccionar la votaci amb ms participaci
   BEGIN
     --S'actualitza el registre a VOTHISTORICAMESPARTICIPACIO     
     SELECT VOTHISMESPART
     INTO VOTHISMESPART_TITOL
     FROM VOTHISTORICAMESPARTICIPACIO
     WHERE ROWNUM = 1;
         
     IF (VOTHISMESPART_TITOL = :OLD.TITOL) THEN       
        UPDATE VOTHISTORICAMESPARTICIPACIO
        SET TOTALVOTS = 0, VOTHISMESPART = '-'
        WHERE ROWNUM = 1;  
     END IF;
    END;

END TR_AFT_DEL_VOTACIO;
/


/*******************************************************************************************************
NOM:      TR_AFT_UP_VOTACIO

FUNCIONALITAT: 

Desprs dactualitzar un registre a la taula VOTACIO, aquest:
- Decreix la mitja de percentatge antiga i incrementa la nova participaci si la votaci es finalitzada donat un cens i un any. 
Taula MITJAPERCENTATGEPARTICIPACIO.
-	Decreix lantiga participaci i incrementa la nova participaci duna votaci donat un any concret. 
Taula VOTACIOMESPARTICIPACIO.
-	Mira si la nova votaci hi ha hagut ms participaci, si s aix lactualitza. 
Taula VOTHISTORICAMESPARTICIPACIO.

*******************************************************************************************************/

CREATE OR REPLACE TRIGGER USUARITFC11.TR_AFT_UP_VOTACIO
AFTER UPDATE ON USUARITFC11.VOTACIO 
REFERENCING NEW AS NEW OLD AS OLD
FOR EACH ROW
DECLARE
     
VOTHISMESPART_TOTAL   NUMBER;    
PARTICIPACIO_TOTALNOU      NUMBER;
PARTICIPACIO_TOTALANTIC    NUMBER;
MITJAPERPARTICIPACIO_CENSNOU   VARCHAR2(2);
MITJAPERPARTICIPACIO_CENSANTIC   VARCHAR2(2);
MITJAPERPARTICIPACIO_CONT   NUMBER;

BEGIN

--2 VALOR MITJA PERCENTATGE PARTICIPACI DONAT UN CENS I UN ANY 
-- TAULA MITJAPERCENTATGEPARTICIPACIO
   --ES CERCA EL CENSPAIS
   BEGIN
      SELECT IDCENSPAIS
      INTO MITJAPERPARTICIPACIO_CENSNOU
      FROM CENSPAIS
      WHERE IDVOTACIO = :NEW.IDVOTACIO;      
   END;
   BEGIN
      SELECT IDCENSPAIS
      INTO MITJAPERPARTICIPACIO_CENSANTIC
      FROM CENSPAIS
      WHERE IDVOTACIO = :OLD.IDVOTACIO;      
   END;   
   BEGIN
      SELECT COUNT(*)
      INTO MITJAPERPARTICIPACIO_CONT
      FROM MITJAPERCENTATGEPARTICIPACIO
      WHERE ANYS = Extract(Year FROM :NEW.DATAFI) 
      AND IDCENS = MITJAPERPARTICIPACIO_CENSNOU;
      
      IF MITJAPERPARTICIPACIO_CONT != 0 THEN
        UPDATE MITJAPERCENTATGEPARTICIPACIO
        SET TOTALVOTACIONS = TOTALVOTACIONS + 1, 
        VALORMITJAPERPART = ((VALORMITJAPERPART * TOTALVOTACIONS) + :NEW.PERCENTATGEVOTACIO) / (TOTALVOTACIONS + 1)
        WHERE ANYS = Extract(Year FROM :NEW.DATAFI) AND IDCENS = MITJAPERPARTICIPACIO_CENSNOU;    
           
        UPDATE MITJAPERCENTATGEPARTICIPACIO
        SET TOTALVOTACIONS = TOTALVOTACIONS - 1, 
        VALORMITJAPERPART = ((VALORMITJAPERPART * TOTALVOTACIONS) - :OLD.PERCENTATGEVOTACIO) / (TOTALVOTACIONS - 1)
        WHERE ANYS = Extract(Year FROM :OLD.DATAFI) AND IDCENS = MITJAPERPARTICIPACIO_CENSANTIC;                     
      ELSE 
        INSERT INTO MITJAPERCENTATGEPARTICIPACIO (IDCENS, ANYS, TOTALVOTACIONS,VALORMITJAPERPART)
        VALUES (MITJAPERPARTICIPACIO_CENSNOU, Extract(Year FROM :NEW.DATAFI), 1, :NEW.PERCENTATGEVOTACIO);            
      END IF;
   END;

-- 4 VOTACIO MS PARTICIPACI DONAT UN ANY
   --Seleccionar la votaci amb ms participaci
    --Es crea el registre a VOTACIOMESPARTICIPACIO  
   BEGIN
     SELECT PARTICIPACIO
     INTO PARTICIPACIO_TOTALNOU
     FROM VOTACIOMESPARTICIPACIO
     WHERE ANYS = Extract(Year FROM :NEW.DATAINICI);
   END;
   BEGIN
     SELECT PARTICIPACIO
     INTO PARTICIPACIO_TOTALANTIC
     FROM VOTACIOMESPARTICIPACIO
     WHERE ANYS = Extract(Year FROM :OLD.DATAINICI);
     
     IF :NEW.TOTALCIUTADANS > PARTICIPACIO_TOTALNOU THEN
         UPDATE VOTACIOMESPARTICIPACIO
         SET VOTACIOMESPART = :NEW.TITOL, PARTICIPACIO = :NEW.TOTALCIUTADANS
         WHERE ANYS = Extract(Year FROM :NEW.DATAINICI); 
     END IF;

     IF :OLD.TOTALCIUTADANS > PARTICIPACIO_TOTALANTIC THEN
         UPDATE VOTACIOMESPARTICIPACIO
         SET VOTACIOMESPART = :OLD.TITOL, PARTICIPACIO = :OLD.TOTALCIUTADANS
         WHERE ANYS = Extract(Year FROM :OLD.DATAINICI); 
     END IF;     
     
   END; 

--7 VOTACIO HISTORICAMENT MS PARTICIPACI
   --Seleccionar la votaci amb ms participaci
    --Es crea el registre a VOTHISTORICAMESPARTICIPACIO  
   BEGIN
     SELECT TOTALVOTS
     INTO VOTHISMESPART_TOTAL
     FROM VOTHISTORICAMESPARTICIPACIO
     WHERE ROWNUM= 1;    

     IF (:NEW.TOTALCIUTADANS > VOTHISMESPART_TOTAL ) THEN
      VOTHISMESPART_TOTAL := :NEW.TOTALCIUTADANS;
     END IF;
         
     UPDATE VOTHISTORICAMESPARTICIPACIO
     SET TOTALVOTS = VOTHISMESPART_TOTAL, VOTHISMESPART = :NEW.TITOL
     WHERE ROWNUM = 1; 

   EXCEPTION
      WHEN NO_DATA_FOUND THEN
       INSERT INTO VOTHISTORICAMESPARTICIPACIO (TOTALVOTS, VOTHISMESPART)
       VALUES (VOTHISMESPART_TOTAL, :NEW.TITOL);    
   END; 

END TR_AFT_UP_VOTACIO;
/


/*******************************************************************************************************
NOM:      TR_AFT_INS_CIUTADAOPCIO

FUNCIONALITAT: 

Desprs de donar dalta un registre a la taula CIUTADAOPCIO, aquest:
- Incrementa la localitat europea que ms vots a ems en un any concret. Taula LOCALITATMESVOTS.
- Decreix el percentatge de ciutadans que mai a votat en un any concret. Taula PERCENTATGECIUTADAMAIVOT.
- Incrementa el nmero mxim de votacions que ha participat una persona i el pas. Taula MAXVOTACIONSPERSONA.

*******************************************************************************************************/

CREATE OR REPLACE TRIGGER USUARITFC11.TR_AFT_INS_CIUTADAOPCIO
AFTER INSERT ON USUARITFC11.CIUTADAOPCIO
REFERENCING NEW AS NEW OLD AS OLD
FOR EACH ROW
DECLARE

VANY                      NUMBER;
LOCALITATNOM              VARCHAR2(30);
LOCALITATMESVOTS_CONT     NUMBER;
MAXVOTACIOPERSONA_PAIS    VARCHAR2(2);
MAXVOTACIONSPERSONA_CONT  NUMBER;
CIUTADA_CONT              NUMBER;
CIUTADA_TOTAL             NUMBER;

BEGIN

--5 Localitat que ms vots ems els ciutadans any concret - TAULA LOCALITATMESVOTS
  --ES CERCA L'ANY
  BEGIN
      SELECT Extract(Year FROM V.DATAINICI) AS DATAINICI
      INTO VANY
      FROM OPCIO O, VOTACIO V 
      WHERE V.IDVOTACIO= O.IDVOTACIO
      AND O.IDOPCIO = :NEW.IDOPCIO 
      GROUP BY V.DATAINICI;
  END;
  --ES CERCA EL NOM DE LA LOCALITAT
  BEGIN    
      SELECT L.LOCALITAT 
      INTO LOCALITATNOM
      FROM LOCALITAT L, CENSLOCAL CL 
      WHERE L.IDLOCALITAT = CL.IDLOCALITAT
      AND CL.IDCENSLOCAL = :NEW.IDCENSLOCAL 
      GROUP BY L.LOCALITAT;           
  END;
  BEGIN
      SELECT COUNT(*) 
      INTO LOCALITATMESVOTS_CONT
      FROM LOCALITATMESVOTS
      WHERE ANYS = VANY AND LOCALITATMESVOTS = LOCALITATNOM;
      
      IF LOCALITATMESVOTS_CONT != 0 THEN
         UPDATE LOCALITATMESVOTS
         SET VOTSTOTALS = VOTSTOTALS + 1
         WHERE ANYS = VANY AND LOCALITATMESVOTS = LOCALITATNOM;
      ELSE      
        INSERT INTO LOCALITATMESVOTS (ANYS, LOCALITATMESVOTS, VOTSTOTALS)
        VALUES (VANY, LOCALITATNOM, 1);       
      END IF;      
  END;
  
--6 PERCENTATGE CIUTADANS MAI VOT - TAULA PERCENTATGECIUTADAMAIVOT
  --ES CALCULA EL NOMBRE DE CIUTADANS TOTALS 
  BEGIN
     SELECT COUNT(*)
     INTO CIUTADA_TOTAL
     FROM CIUTADA;
  END;

  BEGIN 
     SELECT COUNT(*)
     INTO CIUTADA_CONT
     FROM PERCENTATGECIUTADAMAIVOT
     WHERE ANYS = VANY;
    
     IF CIUTADA_CONT != 0 THEN     
         UPDATE PERCENTATGECIUTADAMAIVOT
         SET PERCIUTADAMAIVOT = PERCIUTADAMAIVOT - 1
         WHERE ANYS = VANY;     
     ELSE
        INSERT INTO PERCENTATGECIUTADAMAIVOT (ANYS, PERCIUTADAMAIVOT)
        VALUES (VANY, (CIUTADA_TOTAL - 1));       
     END IF;     
  END;
  
--10 NMERO MXIM DE VOTACIONS MATEIXA PERSONA - TAULA MAXVOTACIONSPERSONA
   BEGIN
      SELECT CP.IDPAIS
      INTO MAXVOTACIOPERSONA_PAIS
      FROM CENSPAIS CP,CENSPAISREGIO CPR, CENSREGIO CR, CENSLOCAL CL
      WHERE CR.IDCENSREGIO = CL.IDCENSREGIO 
      AND CPR.IDCENSPAIS= CP.IDCENSPAIS 
      AND CPR.IDCENSREGIO = CR.IDCENSREGIO
      AND CL.IDCENSLOCAL = :NEW.IDCENSLOCAL;   
   END;

   BEGIN
     SELECT COUNT(*)
     INTO MAXVOTACIONSPERSONA_CONT
     FROM MAXVOTACIONSPERSONA
     WHERE CODICIUTADA = :NEW.IDCIUTADA AND PAIS = MAXVOTACIOPERSONA_PAIS;
     
      IF MAXVOTACIONSPERSONA_CONT != 0 THEN
         UPDATE MAXVOTACIONSPERSONA
         SET NUMEROMAXVOTPERSONA = NUMEROMAXVOTPERSONA + 1
         WHERE CODICIUTADA = :NEW.IDCIUTADA AND PAIS = MAXVOTACIOPERSONA_PAIS;
      ELSE      
        INSERT INTO MAXVOTACIONSPERSONA (CODICIUTADA, PAIS, NUMEROMAXVOTPERSONA)
        VALUES (:NEW.IDCIUTADA, MAXVOTACIOPERSONA_PAIS, 1);       
      END IF;             
   END;
   

END TR_AFT_INS_CIUTADAOPCIO;
/


/*******************************************************************************************************
NOM:      TR_AFT_DEL_CIUTADAOPCIO

FUNCIONALITAT: 

Desprs de donar de baixa un registre a la taula CIUTADAOPCIO, aquest:
-	Decreix la localitat europea que ms vots a ems en un any concret. Taula LOCALITATMESVOTS.
-	Incrementa el percentatge de ciutadans que mai a votat en un any concret. Taula PERCENTATGECIUTADAMAIVOT.
-	Decreix el nmero mxim de votacions que ha participat una persona i el pas. Taula MAXVOTACIONSPERSONA.

*******************************************************************************************************/

CREATE OR REPLACE TRIGGER USUARITFC11.TR_AFT_DEL_CIUTADAOPCIO
AFTER DELETE ON USUARITFC11.CIUTADAOPCIO
REFERENCING NEW AS NEW OLD AS OLD
FOR EACH ROW
DECLARE

MAXVOTACIOPERSONA_PAISANTIC     VARCHAR2(2);
VANY                            NUMBER;
LOCALITATNOM                    VARCHAR2(30);
LOCALITATMESVOTS_CONT           NUMBER;

BEGIN

--5 Localitat que ms vots ems els ciutadans any concret - TAULA LOCALITATMESVOTS
  --ES CERCA L'ANY
  BEGIN
      SELECT Extract(Year FROM V.DATAINICI) AS DATAINICI
      INTO VANY
      FROM OPCIO O, VOTACIO V 
      WHERE V.IDVOTACIO= O.IDVOTACIO
      AND O.IDOPCIO = :OLD.IDOPCIO 
      GROUP BY V.DATAINICI;
  END;
  --ES CERCA EL NOM DE LA LOCALITAT
  BEGIN    
      SELECT L.LOCALITAT 
      INTO LOCALITATNOM
      FROM LOCALITAT L, CENSLOCAL CL 
      WHERE L.IDLOCALITAT = CL.IDLOCALITAT
      AND CL.IDCENSLOCAL = :OLD.IDCENSLOCAL 
      GROUP BY L.LOCALITAT;           
      
      UPDATE LOCALITATMESVOTS
      SET VOTSTOTALS = VOTSTOTALS - 1
      WHERE ANYS = VANY AND LOCALITATMESVOTS = LOCALITATNOM;
  END;

--6 PERCENTATGE CIUTADANS MAI VOT - TAULA PERCENTATGECIUTADAMAIVOT

      UPDATE PERCENTATGECIUTADAMAIVOT
      SET PERCIUTADAMAIVOT = PERCIUTADAMAIVOT + 1
      WHERE ANYS = VANY;     

--10 NMERO MXIM DE VOTACIONS MATEIXA PERSONA - TAULA MAXVOTACIONSPERSONA
   BEGIN
      SELECT CP.IDPAIS
      INTO MAXVOTACIOPERSONA_PAISANTIC
      FROM CENSPAIS CP,CENSPAISREGIO CPR, CENSREGIO CR, CENSLOCAL CL
      WHERE CR.IDCENSREGIO = CL.IDCENSREGIO 
      AND CPR.IDCENSPAIS= CP.IDCENSPAIS 
      AND CPR.IDCENSREGIO = CR.IDCENSREGIO
      AND CL.IDCENSLOCAL = :OLD.IDCENSLOCAL;   
   END;   
        
   UPDATE MAXVOTACIONSPERSONA
   SET NUMEROMAXVOTPERSONA = NUMEROMAXVOTPERSONA - 1
   WHERE CODICIUTADA = :OLD.IDCIUTADA AND PAIS = MAXVOTACIOPERSONA_PAISANTIC;   

END TR_AFT_DEL_CIUTADAOPCIO;
/

/*******************************************************************************************************
NOM:      TR_AFT_UP_CIUTADA

FUNCIONALITAT: 

Desprs de donar de baixa un registre a la taula CIUTADAOPCIO, aquest:
-	Decreix lantiga localitat i incrementa la nova localitat europea que ms vots a ems en un any concret. 
Taula LOCALITATMESVOTS.
-	Incrementa lantic percentatge i decreix el nou percentatge de ciutadans que mai a votat en un any concret. 
Taula PERCENTATGECIUTADAMAIVOT.
-	Decreix lantic i incrementa el nou nmero mxim de votacions que ha participat una persona i el pas. 
Taula MAXVOTACIONSPERSONA.

*******************************************************************************************************/

CREATE OR REPLACE TRIGGER USUARITFC11.TR_AFT_UP_CIUTADAOPCIO
AFTER UPDATE ON USUARITFC11.CIUTADAOPCIO 
REFERENCING NEW AS NEW OLD AS OLD
FOR EACH ROW
DECLARE

MAXVOTACIOPERSONA_PAISNOU       VARCHAR2(2);
MAXVOTACIOPERSONA_PAISANTIC     VARCHAR2(2);
MAXVOTACIONSPERSONA_CONT        NUMBER;
VANYNOU                         NUMBER;
VANYANTIC                       NUMBER;
LOCALITATNOMNOU                 VARCHAR2(2);
LOCALITATNOMANTIC               VARCHAR2(2);

BEGIN

--5 Localitat que ms vots ems els ciutadans any concret - TAULA LOCALITATMESVOTS
  --ES CERCA L'ANY
  BEGIN
      SELECT Extract(Year FROM V.DATAINICI) AS DATAINICI
      INTO VANYNOU
      FROM OPCIO O, VOTACIO V 
      WHERE V.IDVOTACIO= O.IDVOTACIO
      AND O.IDOPCIO = :NEW.IDOPCIO 
      GROUP BY V.DATAINICI;
  END;
  BEGIN
      SELECT Extract(Year FROM V.DATAINICI) AS DATAINICI
      INTO VANYANTIC
      FROM OPCIO O, VOTACIO V 
      WHERE V.IDVOTACIO= O.IDVOTACIO
      AND O.IDOPCIO = :OLD.IDOPCIO 
      GROUP BY V.DATAINICI;
  END;  
  --ES CERCA EL NOM DE LA LOCALITAT NOVA I ANTIGA
  BEGIN    
      SELECT L.LOCALITAT 
      INTO LOCALITATNOMNOU
      FROM LOCALITAT L, CENSLOCAL CL 
      WHERE L.IDLOCALITAT = CL.IDLOCALITAT
      AND CL.IDCENSLOCAL = :NEW.IDCENSLOCAL 
      GROUP BY L.LOCALITAT;           
  END;
  BEGIN    
      SELECT L.LOCALITAT 
      INTO LOCALITATNOMANTIC
      FROM LOCALITAT L, CENSLOCAL CL 
      WHERE L.IDLOCALITAT = CL.IDLOCALITAT
      AND CL.IDCENSLOCAL = :OLD.IDCENSLOCAL 
      GROUP BY L.LOCALITAT;           

      UPDATE LOCALITATMESVOTS
      SET VOTSTOTALS = VOTSTOTALS + 1
      WHERE ANYS = VANYNOU AND LOCALITATMESVOTS = LOCALITATNOMNOU;

      UPDATE LOCALITATMESVOTS
      SET VOTSTOTALS = VOTSTOTALS - 1
      WHERE ANYS = VANYANTIC AND LOCALITATMESVOTS = LOCALITATNOMANTIC;
 
  END;

--6 PERCENTATGE CIUTADANS MAI VOT - TAULA PERCENTATGECIUTADAMAIVOT
  --ES CERCA L'ANY
  BEGIN
      UPDATE PERCENTATGECIUTADAMAIVOT
      SET PERCIUTADAMAIVOT = PERCIUTADAMAIVOT - 1
      WHERE ANYS = VANYNOU;     

      UPDATE PERCENTATGECIUTADAMAIVOT
      SET PERCIUTADAMAIVOT = PERCIUTADAMAIVOT + 1
      WHERE ANYS = VANYANTIC;             
  END;

--10 NMERO MXIM DE VOTACIONS MATEIXA PERSONA - TAULA MAXVOTACIONSPERSONA
   BEGIN
      SELECT CP.IDPAIS
      INTO MAXVOTACIOPERSONA_PAISNOU
      FROM CENSPAIS CP,CENSPAISREGIO CPR, CENSREGIO CR, CENSLOCAL CL
      WHERE CR.IDCENSREGIO = CL.IDCENSREGIO 
      AND CPR.IDCENSPAIS= CP.IDCENSPAIS 
      AND CPR.IDCENSREGIO = CR.IDCENSREGIO
      AND CL.IDCENSLOCAL = :NEW.IDCENSLOCAL;   
   END;
   BEGIN
      SELECT CP.IDPAIS
      INTO MAXVOTACIOPERSONA_PAISANTIC
      FROM CENSPAIS CP,CENSPAISREGIO CPR, CENSREGIO CR, CENSLOCAL CL
      WHERE CR.IDCENSREGIO = CL.IDCENSREGIO 
      AND CPR.IDCENSPAIS= CP.IDCENSPAIS 
      AND CPR.IDCENSREGIO = CR.IDCENSREGIO
      AND CL.IDCENSLOCAL = :OLD.IDCENSLOCAL;   
   END;   

   BEGIN
     SELECT COUNT(*)
     INTO MAXVOTACIONSPERSONA_CONT
     FROM MAXVOTACIONSPERSONA
     WHERE CODICIUTADA = :NEW.IDCIUTADA AND PAIS = MAXVOTACIOPERSONA_PAISNOU;
     
      IF MAXVOTACIONSPERSONA_CONT != 0 THEN
         UPDATE MAXVOTACIONSPERSONA
         SET NUMEROMAXVOTPERSONA = NUMEROMAXVOTPERSONA + 1
         WHERE CODICIUTADA = :NEW.IDCIUTADA AND PAIS = MAXVOTACIOPERSONA_PAISNOU;
      ELSE      
        INSERT INTO MAXVOTACIONSPERSONA (CODICIUTADA, PAIS, NUMEROMAXVOTPERSONA)
        VALUES (:NEW.IDCIUTADA, MAXVOTACIOPERSONA_PAISNOU, 1);       
      END IF;             
   END;
         
   UPDATE MAXVOTACIONSPERSONA
   SET NUMEROMAXVOTPERSONA = NUMEROMAXVOTPERSONA - 1
   WHERE CODICIUTADA = :OLD.IDCIUTADA AND PAIS = MAXVOTACIOPERSONA_PAISANTIC;   

END TR_AFT_UP_CIUTADAOPCIO;
/

/*******************************************************************************************************
NOM:      TR_AFT_DEL_OPCIO

FUNCIONALITAT: 

Desprs de donar de baixa un registre a la taula OPCIO, aquest:
-	Recalcula la diferncia percentual entre l'opci ms votada i la de menys, actualitza l'antiga opci. 
Taula VOTHISTORICADIFERENCIA.


*******************************************************************************************************/

CREATE OR REPLACE TRIGGER USUARITFC11.TR_AFT_DEL_OPCIO
AFTER DELETE ON USUARITFC11.OPCIO
REFERENCING NEW AS NEW OLD AS OLD
FOR EACH ROW
DECLARE

VOTHISDIFERENCIA_CONT  NUMBER;
VOTHISDIFERENCIA_TITOL VARCHAR2(30);
VOTHISDIFERENCIA_MAX  NUMBER;
VOTHISDIFERENCIA_MIN  NUMBER;

BEGIN

--8 9  VOTACIO HISTORICA MS/MENSY DIFERNCIA PERCENTUAL ENTRE L'OPCI MS VOTADA
-- I LA DE MENYS -- TAULA VOTHISTORICADIFERENCIA
   BEGIN
      SELECT V.TITOL
      INTO VOTHISDIFERENCIA_TITOL 
      FROM VOTACIO V
      WHERE V.IDVOTACIO = :OLD.IDVOTACIO;      
   END;

   BEGIN
      SELECT COUNT(*)
      INTO VOTHISDIFERENCIA_CONT
      FROM VOTHISTORICADIFERENCIA
      WHERE VOTHISDIF = VOTHISDIFERENCIA_TITOL;
   END;
      IF VOTHISDIFERENCIA_CONT != 0 THEN
        BEGIN
          SELECT MAXPERCENTUAL, MINPERCENTUAL
          INTO VOTHISDIFERENCIA_MAX, VOTHISDIFERENCIA_MIN
          FROM VOTHISTORICADIFERENCIA
          WHERE VOTHISDIF = VOTHISDIFERENCIA_TITOL; 
        END;
        IF :OLD.PERCENTATGE > VOTHISDIFERENCIA_MAX THEN
          VOTHISDIFERENCIA_MAX := :OLD.PERCENTATGE;
        END IF;
        IF :OLD.PERCENTATGE < VOTHISDIFERENCIA_MIN THEN
          VOTHISDIFERENCIA_MIN := :OLD.PERCENTATGE;
        END IF;
        
        UPDATE VOTHISTORICADIFERENCIA
        SET MAXPERCENTUAL = VOTHISDIFERENCIA_MAX, MINPERCENTUAL = VOTHISDIFERENCIA_MIN,
        DIFPERCENTUAL = VOTHISDIFERENCIA_MAX - VOTHISDIFERENCIA_MIN
        WHERE VOTHISDIF = VOTHISDIFERENCIA_TITOL;  
      END IF;  

END TR_AFT_DEL_OPCIO;
/

/*******************************************************************************************************
NOM:      TR_AFT_UP_OPCIO

FUNCIONALITAT: 

Desprs de donar de baixa un registre a la taula OPCIO, aquest:
-	Recalcula la diferncia percentual entre lopci ms votada i la de menys, actualitza lantiga i la nova opci. 
Taula VOTHISTORICADIFERENCIA.


*******************************************************************************************************/

CREATE OR REPLACE TRIGGER USUARITFC11.TR_AFT_UP_OPCIO
AFTER UPDATE ON USUARITFC11.OPCIO
REFERENCING NEW AS NEW OLD AS OLD
FOR EACH ROW
DECLARE

VOTHISDIFERENCIA_CONT  NUMBER;
VOTHISDIFERENCIA_TITOL VARCHAR2(30);
VOTHISDIFERENCIA_MAX  NUMBER;
VOTHISDIFERENCIA_MIN  NUMBER;

BEGIN

--8 9  vOTACIO HISTORICA MS/MENSY DIFERNCIA PERCENTUAL ENTRE L'OPCI MS VOTADA
-- I LA DE MENYS -- TAULA VOTHISTORICADIFERENCIA
   BEGIN
      SELECT V.TITOL
      INTO VOTHISDIFERENCIA_TITOL 
      FROM VOTACIO V
      WHERE V.IDVOTACIO = :NEW.IDVOTACIO;      
   END;

   BEGIN
      SELECT COUNT(*)
      INTO VOTHISDIFERENCIA_CONT
      FROM VOTHISTORICADIFERENCIA
      WHERE VOTHISDIF = VOTHISDIFERENCIA_TITOL;
   END;
      IF VOTHISDIFERENCIA_CONT != 0 THEN
        BEGIN
          SELECT MAXPERCENTUAL, MINPERCENTUAL
          INTO VOTHISDIFERENCIA_MAX, VOTHISDIFERENCIA_MIN
          FROM VOTHISTORICADIFERENCIA
          WHERE VOTHISDIF = VOTHISDIFERENCIA_TITOL; 
        END;
        IF :NEW.PERCENTATGE > VOTHISDIFERENCIA_MAX THEN
          VOTHISDIFERENCIA_MAX := :NEW.PERCENTATGE;
        END IF;
        IF :NEW.PERCENTATGE < VOTHISDIFERENCIA_MIN THEN
          VOTHISDIFERENCIA_MIN := :NEW.PERCENTATGE;
        END IF;
        
         UPDATE VOTHISTORICADIFERENCIA
         SET MAXPERCENTUAL = VOTHISDIFERENCIA_MAX, MINPERCENTUAL = VOTHISDIFERENCIA_MIN,
         DIFPERCENTUAL = VOTHISDIFERENCIA_MAX - VOTHISDIFERENCIA_MIN
         WHERE VOTHISDIF = VOTHISDIFERENCIA_TITOL;
      ELSE      
        INSERT INTO VOTHISTORICADIFERENCIA (VOTHISDIF, MAXPERCENTUAL, MINPERCENTUAL, DIFPERCENTUAL)
        VALUES (VOTHISDIFERENCIA_TITOL, :NEW.PERCENTATGE, :NEW.PERCENTATGE, VOTHISDIFERENCIA_MAX - VOTHISDIFERENCIA_MIN);       
      END IF;   

END TR_AFT_UP_OPCIO;
/

